;-------------------------------------------------------------------------------
; NAME: bandscan.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the bandscan processing routine.
;
;
;REVISION HISTORY:
;
;-------------------------------------------------------------------------------		
;


#ifdef BANDSCAN
handle_bandscan:
	; get pointer to readings
	movlr	BANK1
	tstfsz	ScanStartDelayCount	; wait for initial delay ?
	goto	bandscan_initial_delay	; yes - jump
	movfp	XCount,WREG
	andlw	7
	addlw	LOW ( BandScanReadings )
	movwf	FSR1

	; get reading and store in next slot

	movlr	BANK0
	movlb	BANK5
	movpf	ADRESH,	S_Meter_ADC	; read s meter ADC count ( 0 .. 255 )

	; convert to dB
	movfp	S0,WREG			; read S0 offset
	btfsc	RXFlags2,RXF2_CWFilter	; check if offset is for CW filter or not
	movfp	S0CW,WREG		; and if so use other S0 level
	subwf	S_Meter_ADC,W		; offset it to zero
	btfss	ALUSTA,C		; if no carry, result negative
	clrf	WREG,F			; else force to zero
	mulwf	Smeter_slope_A		; multiply by slope_A
	movpf	PRODH,AARGB0		; and divide by slope_B
	movpf	PRODL,AARGB1
	movfp	Smeter_slope_B,WREG
	movpf	WREG,BARGB0
	btfss	ALUSTA,Z		; ( but only if divisor non zero )
	call	FXD1507U
	movfp	AARGB1,WREG
	movlr	BANK1

	; later - if freq < min || freq > max, clear wreg
	movwf	INDF1

	movfp	ScanCursor,WREG
	cpfseq	ScanCursorOld
	call	scan_cursor_display

	movfp	XCount,WREG
	andlw	7
	sublw	7
	btfsc	ALUSTA,Z
	call	scan_col_disp

	movlr	BANK1
	incf	XCount,F
	movlw	D'200'
	cpfseq	XCount
	goto	bandscan_step
	movfp	ScanStartB3,WREG
	movwf	ScanCurrentB3
	movfp	ScanStartB2,WREG
	movwf	ScanCurrentB2
	movfp	ScanStartB1,WREG
	movwf	ScanCurrentB1
	movfp	ScanStartB0,WREG
	movwf	ScanCurrentB0
	clrf	XCount,F
	movfp	ScanStartDelay,WREG
	movwf	ScanStartDelayCount
	goto	bandscan_fset

bandscan_step:
	movfp	ScanStepB3,WREG
	addwf	ScanCurrentB3,F
	movfp	ScanStepB2,WREG
	addwfc	ScanCurrentB2,F
	movfp	ScanStepB1,WREG
	addwfc	ScanCurrentB1,F
	movfp	ScanStepB0,WREG
	addwfc	ScanCurrentB0,F
bandscan_fset:
	movfp	ScanCurrentB0, WREG
	movfp	ScanCurrentB1, PRODH
	movfp	ScanCurrentB2, PRODL
	movfp	ScanCurrentB3, FSR0
	movlr	BANK0
	movpf	WREG,AARGB0
	movpf	PRODH,AARGB1
	movpf	PRODL,AARGB2
	movpf	FSR0,AARGB3
	movlw	AARGB0
	goto	setdds1_1
	;; return implicit in goto


scan_col_disp:
	movlr	BANK0
	movlw	0x4E			; CURSOR DIRECTION
	call	LCDCTL			; now set to 'up'

	movlr	BANK1
	movlw	SCANYLOC
	mullw	LCD_COLS
	rrncf	XCount,W		; divide Xcount by 8
	rrncf	WREG,F			; to give char position
	rrncf	WREG,F
	andlw	0x1f
	addlw	SCANXLOC		; x location on screen
	btfsc	ALUSTA,C
	incf	PRODH,F
	addwf	PRODL,F
	movlw	0
	addwfc	PRODH,F		
	movlr	BANK0
	movpf	PRODH,J			; set 'y' for cursor move
	movpf	PRODL,I			; set 'x' for cursor move
	call	LCDGOTOXYGRAPH

	movlr	BANK1
	setf	ValTemp,F
	clrf	YCount,F
scan_col_disp_loop:
	movfp	YCount,WREG
	cpfsgt	BandScanReadings
	bcf	ValTemp,7
	cpfsgt	BandScanReadings+1
	bcf	ValTemp,6
	cpfsgt	BandScanReadings+2
	bcf	ValTemp,5
	cpfsgt	BandScanReadings+3
	bcf	ValTemp,4
	cpfsgt	BandScanReadings+4
	bcf	ValTemp,3
	cpfsgt	BandScanReadings+5
	bcf	ValTemp,2
	cpfsgt	BandScanReadings+6
	bcf	ValTemp,1
	cpfsgt	BandScanReadings+7
	bcf	ValTemp,0
	movfp	ValTemp,WREG
	movlr	BANK0
	call	LCDDATAFAST
	movlr	BANK1
	incf	YCount,F
	movlw	D'120'
	cpfseq	YCount
	goto	scan_col_disp_loop
	movlr	BANK0
	movlw	0x4C			; CURSOR DIRECTION
	goto	LCDCTL			; now set to 'down'
	;; return implicit in goto

bandscan_initial_delay:
	decf	ScanStartDelayCount,F
	movlr	BANK0
	return

bandscan_clear:
	movlw	0x4E			; CURSOR DIRECTION
	call	LCDCTL			; now set to 'up'

	movlw	SCANYLOC+8
	mullw	LCD_COLS
	movlw	D'200'/8 + SCANXLOC + 1
	movwf	K
	goto	bandscan_clear_start
bandscan_clear_loop_0:
	incf	PRODL,F
	btfsc	ALUSTA,C
	incf	PRODH,F
bandscan_clear_start:
	movpf	PRODH,J			; set 'y' for cursor move
	movpf	PRODL,I			; set 'x' for cursor move
	call	LCDGOTOXYGRAPH
	clrf	I,F
bandscan_clear_loop_1:
	movlw	0
	call	LCDDATAFAST
	incf	I,F
	movlw	D'128'
	cpfseq	I
	goto	bandscan_clear_loop_1
	decfsz	K,F
	goto	bandscan_clear_loop_0
	movlw	0x4C			; CURSOR DIRECTION
	goto	LCDCTL			; now set to 'down'
	;; return implicit in goto

;
; display a scan cursor
; after erasing old cursor ( if present )
;
scan_cursor_display:
	movlr	BANK0
	movlw	0x4F			; CURSOR DIRECTION
	call	LCDCTL			; now set to 'down'

	movlr	BANK1
	infsnz	ScanCursorOld,W		; check for 'ff'
	goto	scan_cursor_display2	; and do not erase old cursor
	movlw	SCANYLOC+1
	mullw	LCD_COLS
	rrncf	ScanCursorOld,W		; divide Xcount by 8
	rrncf	WREG,F			; to give char position
	rrncf	WREG,F
	andlw	0x1f
	addlw	SCANXLOC		; x location on screen
	btfsc	ALUSTA,C
	incf	PRODH,F
	addwf	PRODL,F
	movlw	0
	addwfc	PRODH,F		
	movlr	BANK0
	movpf	PRODH,J			; set 'y' for cursor move
	movpf	PRODL,I			; set 'x' for cursor move
	call	LCDGOTOXYGRAPH

; note - LCDDATAFAST preserves W reg so no need to reload
	movlw	0x00			; clear old cursor
	call	LCDDATAFAST
;	movlw	0x00			; clear old cursor
	call	LCDDATAFAST
;	movlw	0x00			; clear old cursor
	call	LCDDATAFAST
;	movlw	0x00			; clear old cursor
	call	LCDDATAFAST
;	movlw	0x00			; clear old cursor
	call	LCDDATAFAST
;	movlw	0x00			; clear old cursor
	call	LCDDATAFAST
;	movlw	0x00			; clear old cursor
	call	LCDDATAFAST
;	movlw	0x00			; clear old cursor
	call	LCDDATAFAST

	movlr	BANK1
scan_cursor_display2:
	movlw	SCANYLOC
	mullw	LCD_COLS
	rrncf	ScanCursor,W		; divide Xcount by 8
	rrncf	WREG,F			; to give char position
	rrncf	WREG,F
	andlw	0x1f
	addlw	SCANXLOC		; x location on screen
	btfsc	ALUSTA,C
	incf	PRODH,F
	addwf	PRODL,F
	movlw	0
	addwfc	PRODH,F		
	movlr	BANK0
	movpf	PRODH,J			; set 'y' for cursor move
	movpf	PRODL,I			; set 'x' for cursor move
	call	LCDGOTOXYGRAPH

	movlr	BANK1
	movfp	ScanCursor,WREG
	movlr	BANK0
	andlw	7
	movwf	J
	movlw	0x80
	movwf	I
	tstfsz	J
	goto	scancsrdisp0
	goto 	scancsrdisp1
scancsrdisp0:
	rrncf	I,F
	decfsz	J,F
	goto 	scancsrdisp0
scancsrdisp1:

; note - LCDDATAFAST preserves W reg so no need to reload
	movfp	I,WREG
	call	LCDDATAFAST
;	movfp	I,WREG
	call	LCDDATAFAST
;	movfp	I,WREG
	call	LCDDATAFAST
;	movfp	I,WREG
	call	LCDDATAFAST
;	movfp	I,WREG
	call	LCDDATAFAST
;	movfp	I,WREG
	call	LCDDATAFAST
;	movfp	I,WREG
	call	LCDDATAFAST
;	movfp	I,WREG
	call	LCDDATAFAST

	movlr	BANK1
	movfp	ScanCursor,WREG
	movwf	ScanCursorOld
	return

#endif



	
