;------------------------------------------------------------------------------
; NAME: bandtab.asm
;
; LABEL PREFIX: N/A
;
; AUTHOR: 	Dave Roberts G8KBB
;
; PURPOSE:	Initial data structures for band change and DDS setting
;
; SPECIAL CONDITIONS:
;	
; REVISION HISTORY:
;		15 06 99	G8KBB	Initial version
;		10 05 00	G8KBB	Ported to 17C766
;------------------------------------------------------------------------------		

band_table_1:

; each band1 table entry is 24 bytes long.
; if this is changed, the label in init.asm needs change also
; there are 6 entries in each, each of 4 bytes
; the first is Misc Flags byte, secondary checksum, s meter adjust and VCO control
; the second is misc flags, LPF relays, FEB1 & FEB2 relays
; the third the bandmin - the band edge in units on dds count
; the 4th is the bandmax - the band edge in units on dds count
; the 5th is the step value - min dds step in units of dds count
;	note main cheksum is first byte of step to save space
; the 6th is the band offset
;
; note that first 8 bytes are checksummed using first checksum
; the next 16 are checksummed using first byte of step
 
; note the mappings shown are for the production front end board

; band 0 - 1.8 MHz
	db	0x00, 0x04, 0x7f, 0xe4		; Misc Flags, checksum, smeter and vco
	db	0x13, 0x01, 0x10, 0x00		; modeflags & RFUs; relay control
	db	0x01, 0x03, 0x33, 0x34		; bandmin
	db	0x01, 0x08, 0x00, 0x00		; bandmax
	db	0x0f, 0x00, 0x00, 0x06		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 1 - 3.5 MHz
	db	0x00, 0x33, 0x7f, 0xc4		; Misc Flags, checksum, smeter and vco
	db	0x13, 0x02, 0x08, 0x00		; modeflags & RFUs; relay control
	db	0x01, 0x2c, 0x00, 0x00		; bandmin
	db	0x01, 0x33, 0x33, 0x34		; bandmax
	db	0xba, 0x00, 0x00, 0x06		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 2 - 7.0 MHz
	db	0x00, 0x21, 0x7f, 0xe5		; Misc Flags, checksum, smeter and vco
	db	0x12, 0x04, 0x04, 0x00		; modeflags & RFUs; relay control
	db	0x01, 0x00, 0x00, 0x00		; bandmin
	db	0x01, 0x01, 0x99, 0x9a		; bandmax
	db	0x4e, 0x00, 0x00, 0x04		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 3 - 10 MHz
	db	0x00, 0x41, 0x7f, 0xc5		; Misc Flags, checksum, smeter and vco
#ifdef LPF_V3
	db	0x0a, 0x04, 0x02, 0x00		; modeflags & RFUs; relay control
#else
	db	0x0a, 0x08, 0x02, 0x00		; modeflags & RFUs; relay control
#endif
	db	0x01, 0x31, 0x99, 0x99		; bandmin
	db	0x01, 0x32, 0x66, 0x67		; bandmax
	db	0x20, 0x00, 0x00, 0x04		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 4 - 14 MHz
	db	0x00, 0x24, 0x7f, 0xf8		; Misc Flags, checksum, smeter and vco
	db	0x08, 0x08, 0x01, 0x00		; modeflags & RFUs; relay control
	db	0x00, 0xb8, 0x00, 0x00		; bandmin
	db	0x00, 0xba, 0xcc, 0xcd		; bandmax
	db	0x7b, 0x00, 0x00, 0x02		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 5 - 18 MHz
	db	0x00, 0x2d, 0x7f, 0xd8		; Misc Flags, checksum, smeter and vco
#ifdef LPF_V3
	db	0x08, 0x08, 0x00, 0x01		; modeflags & RFUs; relay control
#else
	db	0x08, 0x20, 0x00, 0x01		; modeflags & RFUs; relay control
#endif
	db	0x00, 0xd8, 0x8b, 0x43		; bandmin
	db	0x00, 0xd9, 0x58, 0x11		; bandmax
	db	0x9e, 0x00, 0x00, 0x02		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 6 - 21 MHz
	db	0x00, 0x1c, 0x7f, 0xe8		; Misc Flags, checksum, smeter and vco
#ifdef LPF_V3
	db	0x08, 0x10, 0x00, 0x02		; modeflags & RFUs; relay control
#else
	db	0x08, 0x20, 0x00, 0x02		; modeflags & RFUs; relay control
#endif
	db	0x00, 0xf0, 0x00, 0x00		; bandmin
	db	0x00, 0xf3, 0x99, 0x9a		; bandmax
	db	0x70, 0x00, 0x00, 0x02		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 7 - 24 MHz
	db	0x00, 0x2a, 0x7f, 0xe8		; Misc Flags, checksum, smeter and vco
	db	0x08, 0x10, 0x00, 0x04		; modeflags & RFUs; relay control
	db	0x01, 0x0e, 0x66, 0x66		; bandmin
	db	0x01, 0x10, 0x00, 0x00		; bandmax
	db	0x9a, 0x00, 0x00, 0x02		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 8 - 28 MHz
	db	0x00, 0x46, 0x7f, 0xc8		; Misc Flags, checksum, smeter and vco
#ifdef LPF_V3
	db	0x08, 0x20, 0x00, 0x08		; modeflags & RFUs; relay control
#else
	db	0x08, 0x10, 0x00, 0x08		; modeflags & RFUs; relay control
#endif
	db	0x01, 0x28, 0x00, 0x00		; bandmin
	db	0x01, 0x38, 0x00, 0x00		; bandmax
	db	0x24, 0x00, 0x00, 0x02		; step
	db	0x00, 0x89, 0x54, 0x40		; offset
; band 9 - 50 MHz
;	db	0x00, 0x00, 0x00, 0x00		; Misc Flags, checksum, smeter and vco
;	db	0x08, 0x40, 0x00, 0x10		; modeflags & RFUs; relay control
;	db	0x07, 0xc1, 0xf0, 0x7d		; bandmin
;	db	0x17, 0x45, 0xd1, 0x76		; bandmax
;	db	0x00, 0x00, 0x01, 0xfc		; step
;	db	0x02, 0xeb, 0xae, 0x40		; offset
; band 10 - 70 MHz
;	db	0x00, 0x00, 0x00, 0x00		; Misc Flags, checksum, smeter and vco
;	db	0x08, 0x40, 0x00, 0x20		; modeflags & RFUs; relay control
;	db	0x07, 0xc1, 0xf0, 0x7d		; bandmin
;	db	0x0b, 0xa2, 0xe8, 0xbb		; bandmax
;	db	0x00, 0x00, 0x01, 0xfc		; step
;	db	0x04, 0x1c, 0xdb, 0x40		; offset
#ifdef INCLUDE_TRANSVERTER_DAVE
; band xx - 144 MHz
	db	0x00, 0x00, 0x00, 0xc8		; Misc Flags, checksum, smeter and vco
	db	0x29, 0x10, 0x00, 0x08		; modeflags & RFUs; relay control
	db	0x01, 0x28, 0x00, 0x00		; bandmin
	db	0x01, 0x38, 0x00, 0x00		; bandmax
	db	0x24, 0x00, 0x00, 0x02		; step
	db	0x06, 0x60, 0xb0, 0xc0		; offset
; band xx - 432 MHz
	db	0x00, 0x00, 0x00, 0xc8		; Misc Flags, checksum, smeter and vco
	db	0x29, 0x10, 0x00, 0x08		; modeflags & RFUs; relay control
	db	0x01, 0x28, 0x00, 0x00		; bandmin
	db	0x01, 0x38, 0x00, 0x00		; bandmax
	db	0x24, 0x00, 0x00, 0x02		; step
	db	0x17, 0x8b, 0x38, 0xc0		; offset
#endif

; --------------------------------------------------------
; dds initialisation data
; order ( 1 then 2 ) must be contiguous and must
; match order in RAM
; a 32 bit integer value in Hz

dds1_init:
	db	0x01, 0xe8, 0x48, 0x00	; 33 MHz
dds2_init:
	db	0x02, 0xfa, 0xf0, 0x80	; 50 MHz
csumandcwfilterinit:
	db	0,0,0,0			; no offset & dummy csum
UsbLsbInit
	db	0x05, 0xdc		; 1500 Hz USB offset 
	db	0x05, 0xdc		; 1500 Hz LSB offset


