;-------------------------------------------------------------------------------
; NAME: Config.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This contains config data tables foe calibration
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;		22/04/00	G8KBB	ported to V2 controller 17C766
;

;-------------------------------------------------------------------------------		
; Config table data contains one entry per configurable data item
; Each entry contains the following:
;	Address in ram page 0 of data item followed by BSR (page) data
;	flag for what area of memory to write to EEPROM
;	note flag occupies one byte - the high  byte is spare
;       note flags are in EEPROM_UPD_3
;		bit	what
;		0	Secondary band data (SEC_BAND_UPD)
;		1	Main band data (PRI_BAND_UPD)
;		2	Attenuator data (ATTN_CFG_UPD)
;		3	DDS calibration data & CW offset (DDS_DATA_UPD)
;		4	acceleration data (ACCEL_DATA_UPD)
;			also UI flags & S meter data
;		5	Keyer (KEYER_DATA_UPD)
;			also scan parameters
;		6	DSP data update (DSP_DATA_UPD)
;		7	special functions - reset from ROM (EE_RESERVED_UPD)
;	Dispay string name, null terminated
;
; The address is a handle for the function not a physical address when
; the special function bit is set.
; Two values are defined at the moment:
;	0 = reset band data
;	1 = reset DDS clock data

config_data:
	db	Atten_cal, HIGH( Atten_cal ) << 4
	db	0, (1<<ATTN_CFG_UPD)
config_data0:
	db	"Atten  ",0
config_data1:

	db	MIN_ACCEL, HIGH( MIN_ACCEL ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"Accel  ",0

#ifdef ONSCREEN_SMETER
	db	S0, HIGH( S0 ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"S0 Levl",0

	db	S0CW, HIGH( S0CW ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"S0 CW  ",0

	db	Smeter_slope_A, HIGH( Smeter_slope_A ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"S Mult ",0

	db	Smeter_slope_B, HIGH( Smeter_slope_B ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"S Divr ",0

	db	dBm_offset, HIGH( dBm_offset ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"dBm ofs",0

	db	dB_per_spoint, HIGH( dB_per_spoint ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"dB / S ",0

	db	SHang, HIGH( SHang ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"S HangT",0

	db	Pmeter_slope_A, HIGH( Pmeter_slope_A ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"P Mult ",0

	db	Pmeter_slope_B, HIGH( Pmeter_slope_B ) << 4
	db	0, (1<<ACCEL_DATA_UPD)
	db	"P Divr ",0
#endif

	db	VCO_Mode, HIGH( VCO_Mode ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"VCOctrl",0

	db	SMeter_cal, HIGH( SMeter_cal ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"S-meter",0

	db	Bdata_misc, HIGH( Bdata_misc ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"MiscFlg",0

	db	ModeFlags, HIGH( ModeFlags ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"ModeFlg",0

	db	BandSetLPF, HIGH( BandSetLPF ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BandLPF",0

	db	BandSetFEB1, HIGH( BandSetFEB1 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BPass 1",0

	db	BandSetFEB2, HIGH( BandSetFEB2 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BPass 2",0

	db	BANDMIN0, HIGH( BANDMIN0 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BMin 0 ",0

	db	BANDMIN1, HIGH( BANDMIN1 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BMin 1 ",0

	db	BANDMIN2, HIGH( BANDMIN2 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BMin 2 ",0

	db	BANDMIN3, HIGH( BANDMIN3 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BMin 3 ",0

	db	BANDMAX0, HIGH( BANDMAX0 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BMax 0 ",0

	db	BANDMAX1, HIGH( BANDMAX1 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BMax 1 ",0

	db	BANDMAX2, HIGH( BANDMAX2 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BMax 2 ",0

	db	BANDMAX3, HIGH( BANDMAX3 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"BMax 3 ",0

; no entry for stepb0 as it is used for checksum in eeprom and
; zeroed on load / save

	db	STEPB1, HIGH( STEPB1 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"Step 1 ",0

	dB	STEPB2, HIGH( STEPB2 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"Step 2 ",0

	db	STEPB3, HIGH( STEPB3 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"Step 3 ",0

	db	fOffB0, HIGH( fOffB0 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"IFOfs 0",0

	db	fOffB1, HIGH( fOffB1 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"IFOfs 1",0

	db	fOffB2, HIGH( fOffB2 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"IFOfs 2",0

	db	fOffB3, HIGH( fOffB3 ) << 4
	db	0, (1<<SEC_BAND_UPD)|(1<<PRI_BAND_UPD)
	db	"IFOfs 3",0

	db	DDS1_osc0, HIGH( DDS1_osc0 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"DDS1 0 ",0

	db	DDS1_osc1, HIGH( DDS1_osc1 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"DDS1 1 ",0

	db	DDS1_osc2, HIGH( DDS1_osc2 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"DDS1 2 ",0

	db	DDS1_osc3, HIGH( DDS1_osc3 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"DDS1 3 ",0

	db	DDS2_osc0, HIGH( DDS2_osc0 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"DDS2 0 ",0

	db	DDS2_osc1, HIGH( DDS2_osc1 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"DDS2 1 ",0

	db	DDS2_osc2, HIGH( DDS2_osc2 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"DDS2 2 ",0

	db	DDS2_osc3, HIGH( DDS2_osc3 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"DDS2 3 ",0

	db	CWofs1, HIGH( CWofs1 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"CWofs 1",0

	db	CWofs2, HIGH( CWofs2 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"CWofs 2",0

	db	CWofs3, HIGH( CWofs3 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"CWofs 3",0

	db	UsbOffset2, HIGH( UsbOffset2 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"UsbOff2",0

	db	UsbOffset3, HIGH( UsbOffset3 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"UsbOff3",0

	db	LsbOffset2, HIGH( LsbOffset2 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"LsbOff2",0

	db	LsbOffset3, HIGH( LsbOffset3 ) << 4
	db	0, (1<<DDS_DATA_UPD)
	db	"LsbOff3",0

#ifdef INCLUDE_KEYER
;	db	Keyer_dot, HIGH( Keyer_dot ) << 4
;	db	0, (1<<KEYER_DATA_UPD)
;	db	"KeyDot ",0

	db	Keyer_hang, HIGH( Keyer_hang ) << 4
	db	0, (1<<KEYER_DATA_UPD)
	db	"KeyHang",0
#endif

#ifdef DSP_DDS
	db	LOW(DSP_CW_SIDETONE_VOL), HIGH( DSP_CW_SIDETONE_VOL ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"STonVol",0

	db	LOW(DSP_CW_SIDETONE_FREQ_H), HIGH( DSP_CW_SIDETONE_FREQ_H ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"STfreqH",0

	db	LOW(DSP_CW_SIDETONE_FREQ_L), HIGH( DSP_CW_SIDETONE_FREQ_L ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"STfreqL",0

	db	LOW(DSP_LMS_BETA_H), HIGH( DSP_LMS_BETA_H ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"LMSBetH",0

	db	LOW(DSP_LMS_BETA_L), HIGH( DSP_LMS_BETA_L ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"LMSBetL",0

	db	LOW(DSP_LMS_DECAY_H), HIGH( DSP_LMS_DECAY_H ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"LMSDecH",0

	db	LOW(DSP_LMS_DECAY_L), HIGH( DSP_LMS_DECAY_L ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"LMSDecL",0

	db	LOW(DSP_NOTCH_BETA_H), HIGH( DSP_NOTCH_BETA_H ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"NotBetH",0

	db	LOW(DSP_NOTCH_BETA_L), HIGH( DSP_NOTCH_BETA_L ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"NotBetL",0

	db	LOW(DSP_NOTCH_DECAY_H), HIGH( DSP_NOTCH_DECAY_H ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"NotDecH",0

	db	LOW(DSP_NOTCH_DECAY_L), HIGH( DSP_NOTCH_DECAY_L ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"NotDecL",0

	db	LOW(DSP_TX_AGC_HANG_H), HIGH( DSP_TX_AGC_HANG_H ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"TxAGCHH",0

	db	LOW(DSP_TX_AGC_HANG_L), HIGH( DSP_TX_AGC_HANG_L ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"TxAGCHL",0

	db	LOW(DSP_TX_COMPRESSION), HIGH( DSP_TX_COMPRESSION ) << 4
	db	0, (1<<DSP_DATA_UPD)
	db	"TxCompr",0
#endif
#ifdef BANDSCAN
	db	LOW(ScanStepSSBHigh), HIGH( ScanStepSSBHigh ) << 4
	db	0, (1<<KEYER_DATA_UPD)
	db	"ScanSSH",0

	db	LOW(ScanStepSSBLow), HIGH( ScanStepSSBLow ) << 4
	db	0, (1<<KEYER_DATA_UPD)
	db	"ScanSSL",0

	db	LOW(ScanStepCWHigh), HIGH( ScanStepCWHigh ) << 4
	db	0, (1<<KEYER_DATA_UPD)
	db	"ScanCWH",0

	db	LOW(ScanStepCWLow), HIGH( ScanStepCWLow ) << 4
	db	0, (1<<KEYER_DATA_UPD)
	db	"ScanCWL",0

	db	LOW(ScanStartDelay), HIGH( ScanStartDelay ) << 4
	db	0, (1<<KEYER_DATA_UPD)
	db	"ScanDly",0

#endif
	db	0,0
	db	0,(1<<EE_RESERVED_UPD)
	db	"ResBDat",0

	db	1,0
	db	0,(1<<EE_RESERVED_UPD)
	db	"ResDDS ",0

config_data_end:

size_config_data_entry	EQU	(config_data1 - config_data)
size_config_data	EQU	(config_data_end - config_data)
config_data_entries	EQU	(size_config_data / size_config_data_entry)
size_config_data_str	EQU	(config_data1 - config_data0)

