;-------------------------------------------------------------------------------
; NAME: eeprom.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the handler for eeprom updates
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;		10/05/00	G8KBB	Ported to 17C766
;
;-------------------------------------------------------------------------------		
;
do_eeprom_update:
	movfp	EEPROM_UPD_1,WREG	; check we have something to do !
	andlw	0x7f			; mask unused bit ( DDS1 update )
	btfss	ALUSTA,Z
	goto	do_the_update
	movfp	EEPROM_UPD_2,WREG
#ifdef COMMON_DSP_FUNCTIONS
	andlw	B'00001011'		; mask unused bits
#esle
	andlw	B'00000011'		; mask unused bits
#endif
	btfss	ALUSTA,Z
	goto	do_the_update
	movfp	EEPROM_UPD_3,WREG
	andlw	0x7f			; mask unused bits
	btfsc	ALUSTA,Z
	return
do_the_update:
	movfp	EEPROM_UPD_1,WREG
	andlw	( 1 << VOL_OR_MIC_CH ) | ( 1 << VOX_CHANGED ) | ( 1 << IF_CHANGED ) | ( 1 << RF_CHANGE )
	btfsc	ALUSTA,Z
	goto	eeprom_bit_1
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM	; i2c address
	movwf	INDF0
	movlw	0x10
	movwf	INDF0		; sub address
	movlw	(RXFlags - Volume + 1)
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	Volume
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	movlw	~(( 1 << VOL_OR_MIC_CH ) | ( 1 << VOX_CHANGED ) | ( 1 << IF_CHANGED ) | ( 1 << RF_CHANGE ))
	andwf	EEPROM_UPD_1,F
eeprom_bit_1:
	btfss	EEPROM_UPD_1,BAND_CHANGE
	goto	eeprom_bit_1a
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM	; i2c address
	movwf	INDF0
	movlw	0x00
	movwf	INDF0		; sub address
	movlw	1
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	BAND
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_1,BAND_CHANGE
eeprom_bit_1a:
	btfss	EEPROM_UPD_1,MISC_CHANGE
	goto	eeprom_bit_2
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM	; i2c address
	movwf	INDF0
	movlw	0x01
	movwf	INDF0		; sub address
	movlw	(UI_Flags2-UI_Flags+1)
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	OPTO_2_MODE
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_1,MISC_CHANGE
eeprom_bit_2:
	btfss	EEPROM_UPD_2,BUTTON_A_CH
	goto	eeprom_bit_3
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM | 0x0e ; i2c address
	movwf	INDF0
	movlr	BANK0
	swapf	UI_Flags,W		; get menu ( bits 2,3 ) into bits 6,7
	movlr	BANK2
	rrcf	WREG,F			; put them in bits 5,6
	andlw	0x60			; gives index into eeprom data
	movwf	INDF0			; and use this as i2c subaddress
	movlw	0x10
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	PBindirect
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_2,BUTTON_A_CH
eeprom_bit_3:
	btfss	EEPROM_UPD_2,BUTTON_B_CH
	goto	eeprom_3_bit_0
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM | 0x0e ; i2c address
	movwf	INDF0
	movlr	BANK0
	swapf	UI_Flags,W		; get menu ( bits 2,3 ) into bits 6,7
	movlr	BANK2
	rrcf	WREG,F			; put them in bits 5,6
	andlw	0x60			; gives index into eeprom data
	addlw	0x10			; second bank of switches
	movwf	INDF0			; and use this as i2c subaddress
	movlw	0x10			; write 16 bytes
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	PBindirect+0x10
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_2,BUTTON_B_CH
eeprom_3_bit_0:
#ifdef COMMON_DSP_FUNCTIONS
	btfss	EEPROM_UPD_2,DSP_CHANGED
	goto	eeprom_3_not_dsp
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM	 ; i2c address
	movwf	INDF0
	movlw	0x40
	movwf	INDF0		; sub address
	movlw	DSP_INIT_END - DSP_CTRL + 1
	movwf	INDF0		; count
	movlw	(HIGH DSP_CTRL) << 4
	movwf	INDF0		; bsr
	movlw	LOW DSP_CTRL
	movwf	INDF0		; ram pointer
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_2,DSP_CHANGED
eeprom_3_not_dsp:
	btfss	EEPROM_UPD_3,DSP_DATA_UPD
	goto	eeprom_not_dsp
	bsf	UPDATE_2,DSP_CHANGED
	bcf	EEPROM_UPD_3,DSP_DATA_UPD
eeprom_not_dsp:
#endif
	btfss	EEPROM_UPD_3,SEC_BAND_UPD
	goto	eeprom_3_bit_1
	movlw	Bdata_start	; first, checksum the  data
	movwf	FSR0
	movlw	0x08
	call	checksum	; note we include the checksum
	addwf	Bdata_csum2,F	; and correct the checksum
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM | 0x0a	 ; i2c address page 5
	movwf	INDF0
	movlr	BANK0
	swapf	BAND,W
	andlw	0xf0		; get band << 4
	movlr	BANK2
	movwf	INDF0		; sub address
	movlw	0x08
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	Bdata_start
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_3,SEC_BAND_UPD
eeprom_3_bit_1:
	btfss	EEPROM_UPD_3,PRI_BAND_UPD
	goto	eeprom_3_bit_2
	movlw	BANDMIN0	; first, checksum the  data
	movwf	FSR0
	movlw	0x10
	call	checksum	; note we include the checksum
	addwf	Bdata_csum1,F	; and correct the checksum
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM | 0x0c	 ; i2c address page 6
	movwf	INDF0
	movlr	BANK0
	swapf	BAND,W
	andlw	0xf0		; get band << 4
	movlr	BANK2
	movwf	INDF0		; sub address
	movlw	0x10
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	BANDMIN0
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_3,PRI_BAND_UPD
eeprom_3_bit_2:
	btfss	EEPROM_UPD_3,ATTN_CFG_UPD
	goto	eeprom_3_bit_3
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM 	 ; i2c address
	movwf	INDF0
	movlr	BANK0
	movfp	RXFlags,WREG
	andlw	3		; get attenuator 0..3
	addlw	0x20		; index into attenuator settings
	movlr	BANK2
	movwf	INDF0		; sub address
	movlw	1
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	Atten_cal
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_3,ATTN_CFG_UPD
eeprom_3_bit_3:
	btfss	EEPROM_UPD_3,DDS_DATA_UPD
	goto	eeprom_3_bit_4
	movlw	DDS1_osc0	; first, checksum the dds data
	movwf	FSR0
	movlw	D'16'
	call	checksum	; note we include the checksum
	addwf	DDS_cksum,F	; and correct the checksum
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM | 0x0e	 ; i2c address
	movwf	INDF0
	movlw	0x80
	movwf	INDF0		; sub address
	movlw	D'16'
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	DDS1_osc0
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_3,DDS_DATA_UPD
eeprom_3_bit_4:
	btfss	EEPROM_UPD_3,ACCEL_DATA_UPD
	goto	eeprom_3_bit_5
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM	 ; i2c address
	movwf	INDF0
	movlw	2
	movwf	INDF0		; sub address
	movlw	UI_Flags2-MIN_ACCEL+1
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	MIN_ACCEL
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	
	bcf	EEPROM_UPD_3,ACCEL_DATA_UPD
eeprom_3_bit_5:
	btfss	EEPROM_UPD_3,KEYER_DATA_UPD
	goto	eeprom_bit_4
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM	 ; i2c address
	movwf	INDF0
	movlw	0x30
	movwf	INDF0		; sub address
	movlw	Keyer_hang-Keyer_dot+1
	movwf	INDF0		; count
	clrf	INDF0,F		; bsr
	movlw	Keyer_dot
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	

	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	eeprom_exit
	movlw	I2C_EEPROM	 ; i2c address
	movwf	INDF0
	movlw	0x60
	movwf	INDF0		; sub address
	movlw	ScanStartDelay - ScanStepSSBHigh + 1
	movwf	INDF0		; count
	movlw	HIGH ScanStepSSBHigh << 4
	movwf	INDF0		; bsr
	movlw	LOW ScanStepSSBHigh
	movwf	INDF0		; address
	movlw	0x5e		; 5 goes, auto clear, retry, sub adress, write
	movwf	INDF0
	call	I2C_Startup	; and go to it !
	clrf	BSR,F	

	bcf	EEPROM_UPD_3,KEYER_DATA_UPD
eeprom_bit_4:
	btfss	EEPROM_UPD_1,DDS1_CHANGED
	goto	eeprom_exit
	btfsc	EE_FREQ_IDLE,6
	call	wr_vco_data
eeprom_exit:
	return

;
; helper utility  - writes current band vcos to eeprom
;
wr_vco_data:
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	wr_vco_data_0
	movlw	I2C_EEPROM | 4
	movwf	INDF0		; eeprom address of band vco data page (2)
	movlr	BANK0
	swapf	BAND,W		; get band
	andlw	0xf0
	movlr	BANK2
	movwf	INDF0		; sub address of ( band << 4 )
	movlw	8
	movwf	INDF0		; write 8 bytes
	clrf	INDF0,F		; BSR = 0
	movlw	DDS1A_0
	movwf	INDF0		; source = VCO data
	movlw	B'00000010'	; no auto clear, no retry, sub address, write
	movwf	INDF0
	call	I2C_Startup
	call	eeprom_wait
wr_vco_data_0:
	bcf	EEPROM_UPD_1,DDS1_CHANGED
	return

