;-------------------------------------------------------------------------------
; NAME: fdisp.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This updates displayed frequencies.
;
; INPUTS:	none			
;			      
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;
;-------------------------------------------------------------------------------		
;
; 	if display_flag is not set
;		return
;	else
;		let BARGB := DDS1 clock frequency in Hz
;		let AARGB := DDS1 accumulator setting ( 32 bit integer )
;		let AARGB := AARGB * BARGB
;		let AARGB := AARGB / 2^32 ( gives frequency in Hz )
;		let AARGB := AARGB * 256 / n where n = 1,2,3... for vco divisor
;		let AARGB := AARGB + fOffs ( calclate display frequency )
;		for i = 9 to 0
;			let buffer[i] := (AARGB % 10) + '0'
;			let AARGB := AARGB / 10
;			if i == 4
;				let buffer[i] := '.'
;				let i := i - 1
;			endif
;		endfor
;		if buffer[0] == '0'
;			let buffer[0] := space
;			if buffer[1] == '0'
;				let buffer[1] := space
;			endif
;		endif
;		set cursor position on screen
;		for i = 0 to 8
;			display( buffer[i] )
;		endfor
;	endif
;
do_display:
	btfss	UPDATE_1,DDS1_CHANGED	; is update_fdisp_flag set ?
	goto	check_mode_update	; if not, do other events
	clrf	Idle_Count,F		; mark change to UI

	movlw	DDS1A_0			; move from DDS1?_0..3 to AARGB0..3
	btfsc	RXFlags,RXF_RX_B
	movlw	DDS1B_0
	call	mk_fdisp
;----------------------------------------------------------------------------
; set display location and print it
;-----------------------------------------------------------------------------
;
	movlw	s_fdisp-string_index
	btfsc	RXFlags,RXF_RX_B
	movlw	s_fdispB-string_index
#ifdef INCLUDE_GRAPHICS
	call	g_pstr_xy
#else
	call	p_string_xy
#endif

;----------------------------------------------------------------------------
; display VCO usage characters
;-----------------------------------------------------------------------------

	swapf	RXFlags,W	; get VCO mode bits ( RXFlags bits 6 & 5 )
	rrcf	WREG,F
	andlw	3		; and put it in wreg[1..0]
	btfss	ALUSTA,Z	; zero ?
	goto	do_disp_0	; if not, check if 01
	movlw	'>'		; 00 means VCOA
	movwf	DISP_BUF
	movlw	' '
	goto	do_disp_3
do_disp_0:
	decfsz	WREG,F		; 01 ?
	goto	do_disp_1	; if not, try 10
	movlw	'R'		; 01 means VCOA TX, VCOB RX
	movwf	DISP_BUF
	movlw	'T'
	goto	do_disp_3
do_disp_1:
	decfsz	WREG,F		; 10 ?
	goto	do_disp_2	; if not try 11
	movlw	'T'		; 10 means VCOA RX, VCOB TX
	movwf	DISP_BUF
	movlw	'R'
	goto	do_disp_3
do_disp_2:
	movlw	' '		; must be 11, i.e. VCOB
	movwf	DISP_BUF
	movlw	'>'
do_disp_3:
	movwf	DISP_BUF+2	; save second char in temp locn
	clrf	DISP_BUF+1,F
	movlw	s_vcoamode-string_index
#ifdef INCLUDE_GRAPHICS
	call	g_pstr_xy
#else
	call	p_string_xy
#endif
	movfp	DISP_BUF+2,WREG
	movwf	DISP_BUF
	movlw	s_vcobmode-string_index
#ifdef INCLUDE_GRAPHICS
	call	g_pstr_xy
#else
	call	p_string_xy
#endif

; iic	bcf	UPDATE_1,DDS1_CHANGED	;clear update_fdisp_flag
;	return
; drop through
; --------------------------------- modes etc display update ----------------
;
check_mode_update:
	btfss	UPDATE_2,MODE_CHANGE	; is flag set ?
	goto	check_rf_change
	movlw	0		; now get bits 0,1 to same as mode
	btfsc	ModeFlags,3
	iorlw	1
	btfsc	ModeFlags,4
	iorlw	2
; extra for pwr control
	movwf	I
	movlb	BANK5
	bcf	PORTF_COPY,PWR_CW
	bcf	PORTF_COPY,PWR_USB
	bcf	PORTF_COPY,PWR_LSB
	addlw	0
	btfsc	ALUSTA,Z
	bsf	PORTF_COPY,PWR_CW
	decf	WREG,F
	btfsc	ALUSTA,Z
	bsf	PORTF_COPY,PWR_USB
	decf	WREG,F
	btfsc	ALUSTA,Z
	bsf	PORTF_COPY,PWR_LSB
	movfp	PORTF_COPY,PORTF
	movfp	I,WREG
; end of extra
	addlw 	s_mode-string_index	; point to modes
;;	clrf	DISP_BUF,F		; no trailing string
	call	p_string_xy_nobuf
check_rf_change:
	btfss	UPDATE_1,RF_CHANGE	; is flag set ?
	goto	check_vol_change
; iic	bcf	UPDATE_1,RF_CHANGE
	movfp	RXFlags,WREG
	andlw	3
	addlw	s_attendb-string_index
;;	clrf	DISP_BUF,F
	call	p_string_xy_nobuf
	; drop through
check_vol_change:
	btfss	UPDATE_1,VOL_OR_MIC_CH	; is flag set ?
	goto	check_config_change
; iic	bcf	UPDATE_1,VOL_OR_MIC_CH
	; now do volume
	movfp	Volume,WREG
	call	pr_to_buf
	movlw	s_voldatstr-string_index
	call	p_string_xy
;	return
; drop thru
; --------------------------------- config update -------------------------
;
check_config_change:
; iic	BANK0
; iic	movlw	~(( 1 << VOL_OR_MIC_CH ) | ( 1 << VOX_CHANGED ) | ( 1 << IF_CHANGED ) | ( 1 << RF_CHANGE ))
; iic	andwf	UPDATE_1,F
;
; cw filter change
;
	btfss	UPDATE_2,FILTER_CHANGE	; change marked ?
	goto	not_filter
	clrf	DISP_BUF,F		; clear buffer of extra to send
	movlw	s_wide-string_index
	btfsc	RXFlags2, RXF2_CWFilter	; set narrow ?
	movlw	s_narrow-string_index
	call	p_string_xy
;;;	bcf	UPDATE_2,FILTER_CHANGE	; do not clear - misc_events does it

not_filter:
	return


;
;----------------------------------------------------------------------------
; convert integer in W reg into string in disp buf
; ( fixed length 3 characters, null terminated )
;----------------------------------------------------------------------------
;		
pr_to_buf:
	clrf	DISP_BUF+3,F
	movwf	DISP_BUF+2
	clrf	DISP_BUF+1,F
	clrf	DISP_BUF+0,F
	movlw	0x0a
pr_buf_loop:
	incf	DISP_BUF+1,F
	subwf	DISP_BUF+2,F
	btfsc	ALUSTA,C
	goto	pr_buf_loop
	decf	DISP_BUF+1,F
	addwf	DISP_BUF+2,F
pr_buf_loop2:
	incf	DISP_BUF+0,F
	subwf	DISP_BUF+1,F
	btfsc	ALUSTA,C
	goto	pr_buf_loop
	decf	DISP_BUF+0,F
	addwf	DISP_BUF+1,F
	movlw	0x30
	addwf	DISP_BUF+0,F
	addwf	DISP_BUF+1,F
	addwf	DISP_BUF+2,F
;
;----------------------------------------------------------------------------
; Now trash up to 2 leading zeros ( note entry point )
; unless 3 digit frequency, when move it right one.
;-----------------------------------------------------------------------------
;
del_zeros:
	movfp	DISP_BUF+0,WREG		; get first character
	sublw	0x30			; equal to '0'
	btfss	ALUSTA,Z
	goto	pr_buf_3digit		; if not, move it right one
	movlw	0x20			; else set to space
	movwf	DISP_BUF		; and check second character
	movfp	DISP_BUF+1,WREG
	sublw	0x30
	btfss	ALUSTA,Z
	goto	pr_buf_3
	movlw	0x20
	movwf	DISP_BUF+1
pr_buf_3:
	return
;
; move string to right one place ( F > 99 MHz )
; and truncate string length if needed
;
pr_buf_3digit:
	movlw	DISP_BUF_END	; source pointer
	movwf	FSR1
	movlw	DISP_BUF_END-1	; dest pointer
	movwf	FSR0
	movlw	0x00		; auto increment both
	movwf	ALUSTA		; fsr0 and fsr1
	movlw	DISP_BUF_END-DISP_BUF
pr_buf_3digit_loop:
	movfp	INDF0,INDF1
	decfsz	WREG,F
	goto	pr_buf_3digit_loop
	movlw	0x20
	movwf	INDF1
	movlw	B'10010000'	; ALUSTA - set FSR1 no auto increment or decrement
	movwf	ALUSTA		; ALUSTA - and set FSR0 to auto increment
	clrf	DISP_BUF+0x0a,F	; truncate string
	return

;
;----------------------------------------------------------------------------
; AARGB0..3 contains display frequency in binary. Convert to ASCII and display
;-----------------------------------------------------------------------------
;
aarg_to_ascii:
	movlw	DISP_BUF+9		; point to end of display buffer
	movwf	FSR1
	movlw	9			; set char count to 9
	movwf	temp_count
	clrf	BARGB0,F		; set value for B to '10'
	movlw	0ah
	movwf	BARGB1
aarg_to_ascii_2:
	call	FXD3115U		; divide data by 10
	movfp	REMB1,WREG		; get remainder ( i.e. next digit )
	addlw	0x30			; convert to character
	movwf	INDF1			; store in buffer
	decf	FSR1,f
	movlw	4			; are we at MHz decimal point ?
	subwf	temp_count,W
	btfss	ALUSTA,Z
	goto	aarg_to_ascii_3		; skip over if not
	movlw	0x2e			; decimal point
	movwf	INDF1			; store in buffer
	decf	FSR1,f			; bump buffer
aarg_to_ascii_3:
	decfsz	temp_count,f		; and loop thru all digits
	goto 	aarg_to_ascii_2
	movlw	0x20			; space at end
#ifdef DO_NOT_DISPLAY_HZ
	movwf	DISP_BUF+9
	clrf	DISP_BUF+0x0a,F		; put null marker on string
#else
	movwf	DISP_BUF+0x0a
	clrf	DISP_BUF+0x0b,F		; put null marker on string
#endif
	return

;----------------------------------------------------------------------------
; --------------------------------- form display freq -------------------------
; called from above to create a display frequency in DISP_BUF
;
; input parameter: Address of DDS ( DDS1A_0 or DDS1B_0 ) in WREG

mk_fdisp:
;----------------------------------------------------------------------------
; Let AARGB := DDS1 current setting
;-----------------------------------------------------------------------------
	movwf	FSR0
	movpf	INDF0,AARGB0
	movpf	INDF0,AARGB1
	movpf	INDF0,AARGB2
	movpf	INDF0,AARGB3

;----------------------------------------------------------------------------
; Let BARGB := DDS1 clock frequency
;-----------------------------------------------------------------------------
	movlw	DDS1_osc0	; move from DDS1_osc0..3 to BARGB0..3
	movwf	FSR0
	movpf	INDF0,BARGB0
	movpf	INDF0,BARGB1
	movpf	INDF0,BARGB2
	movpf	INDF0,BARGB3

;----------------------------------------------------------------------------
; Let AARGB := (64 bit integer) DDS1 clock * DDS1 setting
; giving dds frequency in integer form in aarg0..3 ( i.e. divide by 2^32 )
;-----------------------------------------------------------------------------

	call	FXM3232U	; was FXM3131U

;----------------------------------------------------------------------------
; now, we still actually have aargb4, and we want to multiply by 256
; and divide by 1 2 3 etc according to vco divisor ( 2 4 6 ... )
; so start by shifting right 1 byte ( aargb1..4 moved to aargb0..3 )
; then divide if needed
;-----------------------------------------------------------------------------
	movlw	AARGB1		; point to AARGB1
	movwf	FSR0	
	movpf	INDF0,AARGB0	; and move 4 bytes, multiplying by 256
	movpf	INDF0,AARGB1
	movpf	INDF0,AARGB2
	movpf	INDF0,AARGB3

	movfp	ModeFlags,WREG	; get vco mode to get the vco divisor
	andlw	7		; now get just the divisor ratio ( 1..7 )
	btfsc	ALUSTA,Z	; defensive programming - skip if 0
	goto	no_divide
	movwf	BARGB1		; set up divisor
	clrf	BARGB0,F
	CALL 	FXD3115U	; and divide. Ignore remainder
no_divide:

;----------------------------------------------------------------------------
; now decide on add / subtract Foffset & which way round
; based on mode flags bits 5 & 6 - see defintions in rmap.h
;-----------------------------------------------------------------------------

	btfsc	ModeFlags,5
	goto	fdisp_osc_p_if
	btfss	ModeFlags,6
	goto 	fdisp_osc_m_if
;;	goto	fdisp_if_m_osc	; drop thru
;----------------------------------------------------------------------------
; now substract from foffset to get display freq, i.e.
; Let AARGB := fOffB - AARGB
;-----------------------------------------------------------------------------
; drop thru from above
;;fdisp_if_m_osc:
	movfp	AARGB3,WREG
	subwf	fOffB3,W
	movwf	AARGB3
	movfp	AARGB2,WREG
	subwfb	fOffB2,W
	movwf	AARGB2
	movfp	AARGB1,WREG
	subwfb	fOffB1,W
	movwf	AARGB1
	movfp	AARGB0,WREG
	subwfb	fOffB0,W
	movwf	AARGB0
	goto	fdisp_end_calc_offset

;----------------------------------------------------------------------------
; now add foffset to get display freq, i.e.
; Let AARGB := AARGB + fOffB
;-----------------------------------------------------------------------------
;
fdisp_osc_p_if:
	movfp	fOffB3,WREG	
	addwf	AARGB3,F
	movfp	fOffB2,WREG
	addwfc	AARGB2,F
	movfp	fOffB1,WREG
	addwfc	AARGB1,F
	movfp	fOffB0,WREG
	addwfc	AARGB0,F
	goto	fdisp_end_calc_offset

;----------------------------------------------------------------------------
; now substract foffset to get display freq, i.e.
; Let AARGB := AARGB - fOffB
;-----------------------------------------------------------------------------
;
fdisp_osc_m_if:
	movfp	fOffB3,WREG	
	subwf	AARGB3,F
	movfp	fOffB2,WREG
	subwfb	AARGB2,F
	movfp	fOffB1,WREG
	subwfb	AARGB1,F
	movfp	fOffB0,WREG
	subwfb	AARGB0,F

fdisp_end_calc_offset:
;----------------------------------------------------------------------------
; now correct usb / lsb offsets to get display freq
; if required
; i.e. if Bdata_misc flag bit BDSsbOffset is set then correct
; Correction is by selecting usb / lsb offset according to mode
; and adding lsb / subtracting usb if BDSsbReverse is clear
; or subtracting lsb / adding usb if  BDSsbReverse is set
;-----------------------------------------------------------------------------
;
	btfss	Bdata_misc, BDSsbOffset
	goto	fdisp_end_sideband_offset
	movfp	ModeFlags,WREG
	andlw	0x18		; select mode flags
	btfsc	ALUSTA,Z	; check CW mode 
	goto	fdisp_end_sideband_offset ; and if cw, skip over
	movlw	UsbOffset3
	btfsc	ModeFlags,4	; check for lsb
	movlw	LsbOffset3
	movwf	FSR1		; point to operand
	movlw	0		; 0 for usb
	btfsc	ModeFlags,4
	movlw	1		; 1 for lsb
	btfsc	Bdata_misc, BDSsbReverse
	xorlw	1		; invert flag
	; flag now means subtract if clear, add if set
	btfsc	WREG,0
	goto	fdisp_sideband_add
	movfp	INDF1,WREG
	decf	FSR1,F
	subwf	AARGB3,F
	movfp	INDF1,WREG
	subwfb	AARGB2,F
	movlw	0
	subwfb	AARGB1,F
	subwfb	AARGB0,F
	goto	fdisp_end_sideband_offset
fdisp_sideband_add:
	movfp	INDF1,WREG	
	decf	FSR1,F
	addwf	AARGB3,F
	movfp	INDF1,WREG
	addwfc	AARGB2,F
	movlw	0
	addwfc	AARGB1,F
	addwfc	AARGB0,F
fdisp_end_sideband_offset:
;
;----------------------------------------------------------------------------
; AARGB0..3 contains display frequency in binary. Convert to ASCII and display
;-----------------------------------------------------------------------------
;
	call	aarg_to_ascii
;
;----------------------------------------------------------------------------
; Now trash leading zeros from hundreds or tens of MHz
;-----------------------------------------------------------------------------
;
	goto	del_zeros
;	return implicit in goto


;-----------------------------------------------------------------------------
; ----------------------- display "other" VCO data --------------------------
;-----------------------------------------------------------------------------
do_other_display_if_needed:
	rrncf	RXFlags,W	; we display the 'other' VCO
	xorwf	RXFlags,W	; so exor the bits and check
	andlw	1 << RXF_TX_B	; if RX & TX are different
	btfsc	ALUSTA,Z	; for zero
	return
do_other_display:
	movlw	DDS1A_0		; move from DDS1?_0..3 to AARGB0..3
	btfss	RXFlags,RXF_RX_B
	movlw	DDS1B_0
	call	mk_fdisp
;----------------------------------------------------------------------------
; set display location and print it
;-----------------------------------------------------------------------------
;
	movlw	s_fdisp-string_index
	btfss	RXFlags,RXF_RX_B
	movlw	s_fdispB-string_index
#ifdef INCLUDE_GRAPHICS
	call	g_pstr_xy
#else
	call	p_string_xy
#endif
	return

