;-------------------------------------------------------------------------------
; NAME: i2c_evt.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the i2c external device updates ( apart from eeprom ).
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;
;-------------------------------------------------------------------------------		
;
; In this module, there are a series of very similar code blocks.
; In each case, a test is made as to whether there is need for a
; data update. If so, it allocates an I2C data buffer and queues
; it for execution by the I2C handler.
; In many cases, minor data reorganisation is needed first.
;
do_i2c_events:
	btfss	UPDATE_1,BAND_CHANGE
	goto	i2c_chk_vol
	bcf	UPDATE_1,BAND_CHANGE
	call	i2c_set_LPF
	call	i2c_set_FEB1		; set front end bandpass filters etc
	call	i2c_set_FEB2		; ditto
#ifdef RELAY_BOARD
	call	i2c_set_relay_board
#endif
	movlw	I2C_VCO_CTRL		; set VCO control word
	movwf	DISP_BUF
	movfp	VCO_Mode,WREG
	movwf	DISP_BUF+2
	movlw	1
	movwf	DISP_BUF+1
	call	i2c_helper
	call	i2c_set_smeter
i2c_chk_vol:
	btfss	UPDATE_1,VOL_OR_MIC_CH
	goto	i2c_chk_vox
	bcf	UPDATE_1,VOL_OR_MIC_CH
#ifdef DSP_DDS
	movlw	2
	movwf	DISP_BUF+2
	movfp	Volume,WREG
#ifdef BANDSCAN
	btfsc	RXFlags2, RXF2_BANDSCAN ; now check if band scan active
	movlw	0			; and mute if scanning
#endif
	movwf	DISP_BUF+3
	call	i2c_helper_ddsdsp
	movlw	3
	movwf	DISP_BUF+2
	movfp	MicGain,WREG
	movwf	DISP_BUF+3
	call	i2c_helper_ddsdsp
#else
	movlw	I2C_VOL_MIC
	movwf	DISP_BUF
	movlw	3
	movwf	DISP_BUF+1
	movlw	0xa9
	movwf	DISP_BUF+2
	comf	Volume,W		; invert volume
#ifdef BANDSCAN
	btfsc	RXFlags2, RXF2_BANDSCAN ; now check if band scan active
	setf	WREG,F			; and mute if scanning
#endif
	andlw	0x3f			; clear mute bit (6)
	movwf	DISP_BUF+4
	comf	MicGain,W		; invert mic gain
	andlw	0x3f			; clear mute
	movwf	DISP_BUF+3
	call	i2c_helper
#endif
i2c_chk_vox:
	btfss	UPDATE_1,VOX_CHANGED
	goto	i2c_chk_if
	bcf	UPDATE_1,VOX_CHANGED
#ifdef DSP_DDS
	movlw	6
	movwf	DISP_BUF+2
	movfp	VoxGain,WREG
	btfsc	UI_Flags,UIF_Vox_off	; if vox off
	clrf	WREG,F			; set vox gain to zero
	movwf	DISP_BUF+3
	call	i2c_helper_ddsdsp
	movlw	4
	movwf	DISP_BUF+2
	movfp	AntiVox,WREG
	movwf	DISP_BUF+3
	call	i2c_helper_ddsdsp
	movlw	5
	movwf	DISP_BUF+2
	movfp	VoxDelay,WREG
	movwf	DISP_BUF+3
	call	i2c_helper_ddsdsp
	call	i2c_update_dsp		; used for update of vox control bit
#else
	movlw	I2C_VOX_AVOX
	movwf	DISP_BUF
	movlw	3
	movwf	DISP_BUF+1
	movlw	0xa9
	movwf	DISP_BUF+2
	movfp	VoxGain,WREG		; get vox gain
	andlw	0x3f			; force to 6 bits
	btfsc	UI_Flags,UIF_Vox_off	; if vox off
	clrf	WREG,F			; set vox gain to zero
	movwf	DISP_BUF+3
	movfp	AntiVox,WREG
	andlw	0x3f
	movwf	DISP_BUF+4
	call	i2c_helper
	movlw	I2C_DAC_1
	movwf	DISP_BUF
	movlw	2
	movwf	DISP_BUF+1
	movlw	I2C_VOX_DLY
	movwf	DISP_BUF+2		; DAC port Vox delay
	movfp	VoxDelay,WREG
	movwf	DISP_BUF+3
	call	i2c_helper
#ifdef COMMON_DSP_FUNCTIONS
	call	i2c_update_dsp		; used for update of vox control bit
#endif
#endif
i2c_chk_if:
	btfss	UPDATE_1,IF_CHANGED
	goto	i2c_chk_rf
	movlw	I2C_DAC_1
	movwf	DISP_BUF
	movlw	2
	movwf	DISP_BUF+1
	movlw	I2C_IF_GAIN
	movwf	DISP_BUF+2
	comf	IFGain,W		; read data and invert
	movwf	DISP_BUF+3
	call	i2c_helper		; set IF gain
	movlw	I2C_DAC_1
	movwf	DISP_BUF
	movlw	2
	movwf	DISP_BUF+1
	movlw	I2C_HANG_TIME
	movwf	DISP_BUF+2
#ifdef BANDSCAN
	movlw	0xff			; switch hang time off
	btfss	RXFlags2, RXF2_BANDSCAN ; if scanning (i.e. hangtime 0)
#endif
	comf	HangTime,W		; read data and invert
	movwf	DISP_BUF+3
	call	i2c_helper		; set hang time
	bcf	UPDATE_1,IF_CHANGED
i2c_chk_rf:
	btfss	UPDATE_1,RF_CHANGE
	goto	i2c_chk_dds1
	bcf	UPDATE_1,RF_CHANGE
	movlw	I2C_DAC_1
	movwf	DISP_BUF
	movlw	2
	movwf	DISP_BUF+1
	movlw	I2C_RF_POWER			; DAC port for RF power
	movwf	DISP_BUF+2
	comf	RFPower,W
	movwf	DISP_BUF+3
	call	i2c_helper
	call	i2c_set_FEB1
	call	read_attenuator
	call	i2c_set_smeter
i2c_chk_dds1:
	btfss	UPDATE_1,DDS1_CHANGED
	goto	i2c_chk_misc
	bcf	UPDATE_1,DDS1_CHANGED
i2c_chk_misc:
	btfss	UPDATE_1,MISC_CHANGE
	goto	i2c_chk_mode
#ifdef RELAY_BOARD
	call	i2c_set_relay_board
	bsf	EEPROM_UPD_3,ACCEL_DATA_UPD
#endif
	bcf	UPDATE_1,MISC_CHANGE
i2c_chk_mode:
	btfss	UPDATE_2,MODE_CHANGE
	goto	i2c_chk_dsp
#ifdef DSP_DDS
	bsf	UPDATE_2,DSP_CHANGED
#endif
	bcf	UPDATE_2,MODE_CHANGE
i2c_chk_dsp:
	btfss	UPDATE_2,DSP_CHANGED
	goto	i2c_chk_end
	bcf	UPDATE_2,DSP_CHANGED
#ifdef DSP_DDS
	call	full_set_dsp
	movlw	9
	movwf	DISP_BUF+2
	rlncf	ModeFlags,W
	andlw 	0x30
	movwf	I
	swapf	WREG,F
	iorwf	I,W
	movwf	DISP_BUF+3
	call	i2c_helper_ddsdsp
#endif
#ifdef COMMON_DSP_FUNCTIONS
	call	i2c_update_dsp
#endif
i2c_chk_end:
	return

; -------------------------------------------------------------
; I2C events for Front end board

i2c_set_FEB2:
	movfp	BandSetFEB2,WREG
	movwf	DISP_BUF+2
	bcf	DISP_BUF+2,7
	bcf	DISP_BUF+2,4
	btfsc	RXFlags, RXF_AntRly
	bsf	DISP_BUF+2,7
	btfsc	RXFlags, RXF_AntRly
	bsf	DISP_BUF+2,4
	movlw	I2C_FEB2
	goto	i2c_set_FEB
;
i2c_set_FEB1:
	movfp	BandSetFEB1,WREG
	movwf	DISP_BUF+2
	bcf	DISP_BUF+2,7
	bcf	DISP_BUF+2,6
	btfsc	RXFlags, RXF_AntRly
	bsf	DISP_BUF+2,7
	btfsc	RXFlags, RXF_Atten
	bsf	DISP_BUF+2,6
#ifndef PROTOTYPE_FEB
	bcf	DISP_BUF+2,5
	btfsc	RXFlags, RXF_Atten2
	bsf	DISP_BUF+2,5
#endif
	movlw	I2C_FEB1
i2c_set_FEB:
	movwf	DISP_BUF
	movlw	1
	movwf	DISP_BUF+1
;	goto	i2c_helper
; drop thru to i2c_helper
; -------------------------------------------------------------
; helper to above functions
; data structure has been set up as follows
;	DISP_BUF	I2C Address
;	DISP_BUF+1	Byte count
;	DISP_BUF+2..5	Up to 4 bytes of data to be written
; the function attempts to send (Byte count) bytes of data
; from (DISP_BUF+2) to the I2C device at address (DISP_BUF)
; with no sub address, no retry, auto clear and by moving the
; data into the buffer at the end of the I2C command buffer

; set up for I2C action and execute
i2c_helper:
	call	I2C_Find_Buf2	; find a free buffer. Return if none
	btfsc	ALUSTA,Z
	return
	bsf	CPUSTA,GLINTD		; disable ints while using unbank
	movlr	BANK0
	movfp	DISP_BUF+1,UNBANK1	; get I2C byte count
	movfp	DISP_BUF,WREG		; get I2C address
	movlr	BANK2
	movwf	INDF0			; set I2C address
	incf	FSR0,F			; skip sub address ( zero )
	movpf	UNBANK1,INDF0		; set byte count
	movpf	BSR,INDF0		; set I2C page to here
	movpf	FSR0,WREG		; and point to space at end of buffer
	addlw	2
	movwf	INDF0			; and store this as address in page
	movlw	8			; command = auto clear, no sub addr, no retry, write
	movwf	INDF0
	movlr	BANK0			; now go get data from page 0 DISP_BUF+2
	movlw	DISP_BUF+2
	movwf	FSR0
	movfp	INDF0,UNBANK1		; up to 4 bytes of data only
	movfp	INDF0,UNBANK2
	movfp	INDF0,UNBANK3
	movfp	INDF0,UNBANK4
	movlr	BANK2
	movpf	FSR1,WREG
	addlw	8
	movwf	FSR0
	movfp	UNBANK1,INDF0		; copy data into I2C buffer data area
	movfp	UNBANK2,INDF0
	movfp	UNBANK3,INDF0
	movfp	UNBANK4,INDF0
	bcf	CPUSTA,GLINTD		; ints on now we have finished with UNBANK*
	call	I2C_Startup		; start execution
	movlr	BANK0			; reset bank register
	return

; -------------------------------------------------------------

#ifdef DSP_DDS
i2c_helper_ddsdsp:
	movlw	2
	movwf	DISP_BUF+1
	movlw	I2C_DSP_DDS
	movwf	DISP_BUF
	goto	i2c_helper
#endif

; -------------------------------------------------------------

i2c_set_smeter:
	movlw	I2C_DAC_1
	movwf	DISP_BUF
	movlw	2
	movwf	DISP_BUF+1
	movlw	I2C_SMETER_CAL
	movwf	DISP_BUF+2
	movfp	SMeter_cal,WREG
	addwf	Atten_cal,W
	addwf	S0CW,W
	movwf	DISP_BUF+3
	goto	i2c_helper
	; return implicit in goto

; -------------------------------------------------------------

read_attenuator:
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	return
	movlw	I2C_EEPROM	; i2c address page 0
	movwf	INDF0
	movlr	BANK0
	movfp	RXFlags,WREG
	andlw	3		; get attenuator 0..3
	addlw	0x20		; index into attenuator settings
	movlr	BANK2
	movwf	INDF0		; i2c subaddress
	movlw	1
	movwf	INDF0		; count
	clrf	INDF0,F		; page
	movlw	Atten_cal
	movwf	INDF0		; ram pointer
	goto	read_eeprom_and_wait
;;	return implicit in goto

; -------------------------------------------------------------

i2c_set_LPF:
#ifdef EXTERNAL_LPF_DRIVER
	movlw	I2C_TX_FILTER2		; set other TX filter
	movwf	DISP_BUF
	movfp	BandSetLPF,WREG	
	movwf	DISP_BUF+2
	bcf	DISP_BUF+2,7
	btfsc	RXFlags, RXF_AntRly
	bsf	DISP_BUF+2,7
	movlw	1
	movwf	DISP_BUF+1
	call	i2c_helper
#endif
	movlw	I2C_TX_FILTER		; set TX filter
	movwf	DISP_BUF
	movfp	BandSetLPF,WREG	
	movwf	DISP_BUF+2
	bcf	DISP_BUF+2,7
	btfsc	RXFlags, RXF_AntRly
	bsf	DISP_BUF+2,7
	movlw	1
	movwf	DISP_BUF+1
	goto	i2c_helper
	;; return implicit in goto

; -------------------------------------------------------------
; utility to set up DSP control lines
i2c_update_dsp:
#ifdef DSP_CONTROL
	movlw	I2C_DSP_CTRL
	movwf	DISP_BUF
	movlr	BANK1
	movfp	DSP_CTRL,WREG	
	movlr	BANK0
	bcf	WREG,DSP_CTRL_VOX_ON
	btfsc	UI_Flags,UIF_Vox_off
	bsf	WREG,DSP_CTRL_VOX_ON
	movwf	DISP_BUF+2
	movlw	1
	movwf	DISP_BUF+1
	goto	i2c_helper
	;; return implicit in goto
#endif
#ifdef DSP_DDS
	movlr	BANK1
	movfp	DSP_CTRL,WREG	
	movlr	BANK0
	bcf	WREG,DSP_CTRL_VOX_ON
	btfsc	UI_Flags,UIF_Vox_off
	bsf	WREG,DSP_CTRL_VOX_ON
	movwf	DISP_BUF+3
	movlw	7
	movwf	DISP_BUF+2
	call	i2c_helper_ddsdsp
	movlr	BANK1
	movfp	DSP_TX_CTRL,WREG	
	movlr	BANK0
	movwf	DISP_BUF+3
	movlw	8
	movwf	DISP_BUF+2
	goto	i2c_helper_ddsdsp
	;; return implicit in goto
#endif


i2c_set_relay_board:
	btfsc	Bdata_misc, BDMisc_xvert0
	goto	xvertbit0set
	btfsc	Bdata_misc, BDMisc_xvert1
	goto	xvertbit1set
	movlw	4
	btfss	UI_Flags2,UIF2_INDEP_RX
	movlw	2
	btfss	UI_Flags2,UIF2_INDEP_TX
	iorlw	80
	btfsc	UI_Flags2,UIF2_INDEP_TX
	iorlw	40
	btfsc	RXFlags,RXF_AntRly
	iorlw	1
	goto 	xvert_end
xvertbit1set:
	movlw	0x20
	goto 	xvert_end
xvertbit0set:
	movlw	0x10
	btfsc	Bdata_misc, BDMisc_xvert1
	movlw	8
xvert_end:
	movwf	DISP_BUF+2
	movlw	I2C_RELAY_BOARD		; set VCO control word
	movwf	DISP_BUF
	movlw	1
	movwf	DISP_BUF+1
	goto	i2c_helper
	;; return implicit in goto

#ifdef DSP_DDS
full_set_dsp:
	movlr	BANK1
	movlw	0x14
	movwf	DSP_DDS_I2C_CMD
	movlr	BANK0
	call	I2C_Find_Buf2	; find a free buffer. Return if none
	btfsc	ALUSTA,Z
	goto	full_set_dsp_exit
	movlw	I2C_DSP_DDS
	movwf	INDF0		; i2c address
	incf	FSR0,F		; skip subaddress
	movlw	0x11		; command + 16 bytes
	movwf	INDF0		; byte count
	movlw	HIGH( DSP_DDS_I2C_CMD ) << 4
	movwf	INDF0		; ram bsr
	movlw	LOW( DSP_DDS_I2C_CMD )
	movwf	INDF0		; ram address
	movlw	0x2C		; 2 goes, write, no sub addr, auto clear
	movwf	INDF0
	call	I2C_Startup		; start execution
full_set_dsp_exit:
	movlr	BANK0			; reset bank register
	return
#endif

