;-------------------------------------------------------------------------------
; NAME: Init.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the power on initialization function and main loop.
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;		22/04/00	G8KBB	ported to V2 controller 17C766
;
;-------------------------------------------------------------------------------		
;
	list st=off
	
Init:
	bsf	CPUSTA,GLINTD	; disable all interrupts

;----------------------------------------------------------------------------
; setup SFR defaults
; In this section, all necessary SFR defaults are set. Note that all
; ports are listed for completeness. Where an SFR does not need to be
; changed, is is shown just as a comment line.
;-----------------------------------------------------------------------------

	clrf	BSR,F		; should be unnecessary but to be sure...

; unbanked SFRs
	movlw	B'00100000'	; timer 0 internal clock no prescale PTT INT falling edge
	movwf	T0STA
	clrf	INTSTA,F	; INTSTA 
; SFRs Bank 0
	movlb	BANK0		; select bank 0
	bcf	PORTA,RBPU	; portA - enable port B pullups
	movlw	B'00111111'	; PORTB - bits 6,7 inputs
	movwf	PORTB		; PortB - all others spare outputs
	movlw	0xc0		; DDRB  - set bits 6,7 to inputs
	movwf	DDRB		; DDRB  - others output
	clrf	RCSTA1,F	; RCSTA1
	; RCREG1
	clrf	TXSTA1,F	; TXSTA1
	; TXREG1
	; SPBRG1
; SFRs Bank 1
	movlb	BANK1
	setf	PORTC,F		; PORTC
	setf	PORTD,F		; PortD
	setf	DDRD,F		; DDRD
	movlw	B'11000000'	; DDRC
	movwf	DDRC		; DDRC
	movlw	B'00001111'	; PortE
	movwf	PORTE		; PortE
	clrf	DDRE,F		; DDRE
	; PIR1
	clrf	PIE1,F		; PIE1
; SFRs Bank 2
	movlb	BANK2
	; TMR1
	; TMR2
	; TMR3L
	; TMR3H
	; PR1
	; PR2
	; PR3L
	; PR3H
; SFRs Bank 3
	movlb	BANK3
	; PW1DCL
	; PW2DCL
	; PW1DCH
	; PW2DCH
	; CA2L
	; CA2H
	movlw	B'00001000'	; TCON1
	movwf	TCON1		; TCON1
	clrf	TCON2,F		; TCON2
; SFRs Bank 4
	movlb	BANK4
	; PIR2
	movlw	B'11000000'	
	movwf	PIE2		; PIE2
	movlw	B'11000000'	; RCSTA2
	movwf	RCSTA2		; RCSTA2
	; RCREG2
	movlw	B'00100000'	; TXSTA2
	movwf	TXSTA2		; TXSTA2
	; SPBRG2
; SFRs Bank 5
; note we keep a copy of port F to enable correct bit operations
	movlb	BANK5
	clrf	PORTF,F		; PortF
	clrf	PORTF_COPY,F	; copy of port F
	clrf	DDRF,F		; DDRF
#ifdef INCLUDE_KEYER
#define PORTG_KEYER_ENA	B'00000011'
#else
#define PORTG_KEYER_ENA	B'00000000'
#endif
#ifdef ONSCREEN_SMETER
#define PORTG_SMETER_ENA B'00001100'
#else
#define PORTG_SMETER_ENA B'00000000'
#endif
	movlw	PORTG_KEYER_ENA | PORTG_SMETER_ENA
	movwf	PORTG		; PortG
	movwf	DDRG		; DDRG
#ifdef ONSCREEN_SMETER
	movlw	B'00000001'	; ADCON0
	movwf	ADCON0		; ADC on, channel 0
	movlw	B'10001100'	; clk/64,AN0,1,8,9 i/p
	movwf	ADCON1		; ADCON1
#else
	movlw	B'00000000'	; ADCON0
	movwf	ADCON0		; ADC off
	movlw	B'00001110'	; all digital ports
	movwf	ADCON1		; ADCON1
#endif
	; ADRESL
	; ADRESH
; SFRs Bank 6
	movlb	BANK6
	movlw	D'80'
	movwf	SSPADD		; SSPADD
	movlw	B'00111000'	; SSPCON1
	movwf	SSPCON1		; SSPCON1
	clrf	SSPCON2,F	; SSPCON2
	bcf	SSPSTAT,SMP	; SSPSTAT
	; SSPBUF
; SFRs Bank 7
	movlb	BANK7
	; PW3DCL
	; PW3DCH
	; CA3L
	; CA3H
	; CA4L
	; CA4H
	clrf	TCON3,F		; TCON3
; SFRs Bank 8
	movlb	BANK7+1
	clrf	PORTH,F		; PORTH
	clrf	PORTJ,F		; PORTJ
#ifdef KBD_4X4
	movlw	0xf0
	movwf	DDRH
	movwf	DDRJ
#else
	setf	DDRH,F		; DDRH
	setf	DDRJ,F		; DDRJ
#endif

;----------------------------------------------------------------------------
; clear RAM in all banks
;-----------------------------------------------------------------------------
	clrf	ALUSTA,F
	movlr	BANK3
	call	init_clr_ram
	movlr	BANK2
	call	init_clr_ram
	movlr	BANK1
	call	init_clr_ram
	movlr	BANK0
	call	init_clr_ram
;----------------------------------------------------------------------------
; set FSR0 to auto increment & FSR1 to no auto change.
; this is a convention that software modules will assume.
; If a function does not follow this convention it MUST
; change it back before exit !!!!!!!!!
;-----------------------------------------------------------------------------

	movlw	B'10010000'	; ALUSTA - set FSR1 no auto increment or decrement
	movwf	ALUSTA		; ALUSTA - and set FSR0 to auto increment

;----------------------------------------------------------------------------
; setup miscellaneous program register RAM data
;-----------------------------------------------------------------------------


;----------------------------------------------------------------------------
; setup interrupts and supporting registers apart from external events
;-----------------------------------------------------------------------------
;
; set up timer for 1msec events
	movlw	(0-T0INIT) & 0xff
	movwf	UNBANK5
	movlw	((0-T0INIT) >> 8) & 0xff
	movwf	UNBANK6
	MOVFP 	UNBANK5, TMR0L ;
	MOVFP 	UNBANK6, TMR0H ;

	bsf	INTSTA,T0IE		; enable timer events
	bsf	INTSTA,PEIE		; enable peripheral events

;----------------------------------------------------------------------------
; set up initial values of switch port registers
;-----------------------------------------------------------------------------
;
#ifdef	KBD_4X4
	movlr	BANK1
	setf	TSW_STATE,F		; initial values of switches
	setf	TSW_STATE+1,F		; is all high
	setf	TSW_STATE+2,F		; is all high
	setf	TSW_STATE+3,F		; is all high
	movlb	BANK7+1
	movlw	0xfe			; set for initial switch poll
	movwf	PORTH			; so first interrupt works ok
	movwf	PORTJ			; so first interrupt works ok
	clrf	BSR,F
#else
	movlr	BANK1
	movpf	PORTJ,TSW_STATE		; read real state of switches
	movpf	PORTH,TSW_STATE+1	; and set 'old' state to this
	movlr	BANK0
#endif

;
;----------------------------------------------------------------------------
; initialise  final stages - do eeprom data setups
;-----------------------------------------------------------------------------
;
	call	SysConfig		; note interrupts come on here
	clrwdt
	call	BandTableInit		; initialise band table data
;
	movlw	DDS1A_3			; set up fsr1 to point to DDS1A
	movwf	FSR1			; do that we can check its value 
	call	do_opto_1_checkminimum	; and set dds

; Tell the code that the band, mic gain, volume, vox, IF settings 
; and everything else has been changed to cause all to be written.

	movlw	(1<<BAND_CHANGE) | (1<<VOL_OR_MIC_CH) | (1<<VOX_CHANGED) | (1<<RF_CHANGE) | (1<<MISC_CHANGE) | (1<<IF_CHANGED)
	movwf	UPDATE_1

;
;----------------------------------------------------------------------------
; initialise  display - show basic info & pb labels
;-----------------------------------------------------------------------------
;
	call	lcdInit			; reset LCD
#ifdef BACKLIGHT_CONTROL
	movlw	3
	andwf	UI_Flags,W		; if backlight flags not zero,
	btfss	ALUSTA,Z		; then display not off, so
	call	backlight_on		; turn light on
#endif

	movlw	0xff			; parameter 255 means
	call	pb_label		; label all buttons

;;	clrf	DISP_BUF,F
	movlw	s_rx-string_index
	call	p_string_xy_nobuf

#ifndef SIMPLE_DISPLAY
	movlw	s_modestr-string_index
	call	p_string_xy

	movlw	s_volstr-string_index
	call	p_string_xy

	movlw	s_freqstr-string_index
	call	p_string_xy

	movlw	s_txofsstr-string_index
	call	p_string_xy

	movlw	s_rxofsstr-string_index
	call	p_string_xy

	movlw	s_attenstr-string_index
	call	p_string_xy

#ifdef ONSCREEN_SMETER
	tstfsz	Smeter_slope_A
	goto	init_smeter
	goto	no_init_smeter
init_smeter:
	movlw	s_smeter1-string_index
	call	p_string_xy
	movlw	s_smeter2-string_index
	call	p_string_xy
no_init_smeter:
#endif
#ifdef DISPLAY_320_240
	call	draw_display
#endif

	btfsc	UI_Flags2,UIF2_VOX_DIS
	bsf	UI_Flags, UIF_Vox_off	; disable vox

#ifdef COMMON_DSP_FUNCTIONS
	call	dsp_display
#endif
	call	vox_ctrl_disp
#ifdef RELAY_BOARD
	call	indep_txrx_display
#endif
#endif


;----------------------------------------------------------------------------
; switch power on and set other controls
;-----------------------------------------------------------------------------
	movlb	BANK5
	bsf	PORTF_COPY,PWR_RX
	movfp	PORTF_COPY,PORTF
;
; check if keyer lines are low. If either is, then disable keyer
;
	btfss	PORTG,KEYER_DOT
	bsf	RXFlags2, RXF2_KEYER_DIS
	btfss	PORTG,KEYER_DASH
	bsf	RXFlags2, RXF2_KEYER_DIS

;----------------------------------------------------------------------------
; finally setup interrupts for external events
;-----------------------------------------------------------------------------

; set up port B for interrupt on change
	movlb	BANK0
	movpf	PORTB,PORTB		; clear any pending port B interrupts
	movpf	PORTB, OLD_PORT_B	; and set old to current to get bits 6,7
	movlb	BANK1
	bcf	PIR1,RBIF		; before clearing old pending flag
	bsf	PIE1,RBIE		; and enabling interrupt

	bsf	INTSTA,INTE		; enable PTT events
;
;----------------------------------------------------------------------------
; LOOP HERE FOREVER !
;
; Mainloop consists of 3 parts, an idle loop where we sit waiting for
; the next 10msec timer tick to happen, a set of functions executed every
; 10 msec, and set of functions executed every second.
;-----------------------------------------------------------------------------

MAINLOOP
	CLRWDT				; clear WDT in mainloop or LCD write
	bcf	CPUSTA,GLINTD		; enable interrupts
	movlw	0x0a			; have we reached 10 msecs ?
	subwf	ONE_MSEC,W
	btfss	ALUSTA,C		; if we have not, skip all other code
	goto	main_not_timer
	movlw	0x0a			; else bump 1 msec counter back 10
	subwf	ONE_MSEC,F		; and fall thru to 10 msec handler
;
; put 10msec events here
;
	incf	TEN_MSEC,F		; bump counter for 1 second events
	call	pb_scan			; scan for changes and update map
	call	pb_expand		; expand push button data
	call	do_pb			; process all button events
	CLRWDT
	call	do_opto			; process all shaft encoder events
	CLRWDT
;
	bcf	RXFlags,RXF_On_Xmit	; clear xmit flag
	movlw	TX_REQUESTED_MASK	; then check if anyone asking for xmit
	andwf	RXFlags2,W		; by anding with mask
	btfss	ALUSTA,Z		; so Z flag set means none
	bsf	RXFlags,RXF_On_Xmit	; else set xmit flag
	btfsc	RXFlags2, RXF2_TX_DISABLE ; now check if tx disabled
	bcf	RXFlags,RXF_On_Xmit	; and if so clear xmit flag
#ifdef BANDSCAN
	btfsc	RXFlags2, RXF2_BANDSCAN ; now check if band scan active
	bcf	RXFlags,RXF_On_Xmit	; and if so clear xmit flag
#endif

;
	btfsc	UPDATE_1,BAND_CHANGE	; if band change is called for
	call	do_band_change		; then do band change
	CLRWDT
	movfp	UPDATE_1,WREG		; copy update flags
	iorwf	EEPROM_UPD_1,F		; into eeprom update words
	movfp	UPDATE_2,WREG		; so that we do the corresponding
	iorwf	EEPROM_UPD_2,F		; eeprom changes
	call	do_display		; process all display events
	call	misc_events
	CLRWDT
	call	do_dds_events		; set dds if needed
	call	do_i2c_events		; process any I2C actions
	CLRWDT
	call	exec_txrx_state_machine	; do tx rx finite state machine
	CLRWDT
#ifdef INCLUDE_KEYER
	btfss	RXFlags2,RXF2_KEYER_DIS
	call	exec_keyer_fsm		; do keyer functions
#endif
#ifdef ONSCREEN_SMETER
	movlb	BANK5			; access ADCON0
	movlw	7			; check in on xmit ( state 7 )
	cpfseq	tx_rx_state		; if so we do power meter
	goto	main_smeter		; if not we do s meter
	movlw	(1<<4)|(1<<GO)|(1<<ADON) ; start power meter reading
	goto	main_meter
main_smeter:
	movlw	(1<<GO)|(1<<ADON)	; start s meter reading
main_meter:
	movwf	ADCON0
	clrf	BSR,F
#endif
	call	I2C_10msec		; kick stalled I2C events
	CLRWDT
#ifdef ONSCREEN_SMETER
#ifdef BANDSCAN
	btfss	RXFlags2, RXF2_BANDSCAN	; bandscan active ? 
#endif
	call	disp_smeter		; if not, display S meter
#endif
#ifdef BANDSCAN
	btfsc	RXFlags2, RXF2_BANDSCAN	; bandscan active ?
	call	handle_bandscan
#endif

;
; end of 10msec events. Now check for 1 second events
;
	movlw	0x64			; 100 * 10 msec ? - i.e. 1 second
	subwf	TEN_MSEC,W		; carry means overflow means 1 second
	btfss	ALUSTA,C
	goto 	main_not_timer		; no carry - keep looping
	movwf	TEN_MSEC		; store 10 msec remainder
;
; put 1 second events here
;
	incf	ONE_SEC,F		; bump general purpose counter
	btfsc	ALUSTA,Z		; count every 256 seconds for backlight fn
	call	Do_Idle_Event		; handle idle events like backlight off

	CLRWDT
#ifdef BACKLIGHT_CONTROL
	movfp	UI_Flags,WREG		; get UI flags to get backlight flags
	andlw	3			; flag bits. If 00, stay off
	btfsc	ALUSTA,Z
	goto	bl_skip
	tstfsz	Idle_Count		; timer at zero for backlight ?
	goto	bl_skip
	call	backlight_on
bl_skip:
#endif

	incf	EE_FREQ_IDLE,F		; timer used for eeprom update at 60 seconds
	CLRWDT

	; check button programming countdown

	movfp	PB_PROG_FLAGS,WREG	; check if zero.
	addlw	0
	btfsc	ALUSTA,Z		; if so, skip over
	goto	main_no_prog_change
	btfsc	PB_PROG_FLAGS,7		; check if < 0 ( already programming )
	goto	main_no_prog_change	; if so, skip over
	decf	PB_PROG_FLAGS,F		; decrement
	btfss	ALUSTA,Z		; if reached zero, enter program state
	goto	main_no_prog_change
	decf	PB_PROG_FLAGS,F		; by setting to minus 1
	clrf	OPTO_1_MODE,F		; clear any other opto mode
	clrf	DISP_BUF,F
	movlw	s_blank_f-string_index
#ifdef INCLUDE_GRAPHICS
	call	g_pstr_xy
#else
	call	p_string_xy
#endif
main_no_prog_change:
	CLRWDT
	call	do_eeprom_update
;
main_not_timer:
	goto	MAINLOOP


;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
; initialise band table from EEPROM
; this is done using eeprom data if valid, else rom data
;-----------------------------------------------------------------------------
;

#define BAND_TABLE_ENTRY_SIZE	D'24'

BandTableInit:
;
; first try to read main bandtable data from eeprom
; if fails, or if checksum invalid, reset to rom values
;
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	BandTableInitFromRom
	movlw	I2C_EEPROM | 0x0c	; i2c address
	movwf	INDF0
	movlr	BANK0
	swapf	BAND,W			; sub addr = band << 4
	movlr	BANK2
	andlw	0xf0
	movwf	INDF0			; i2c subaddress
	movlw	0x10			; 16 bytes primary band data
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	BANDMIN0
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
	btfsc	ALUSTA,Z		; if not read OK
	goto	BandTableInitFromRom	; reset to rom default
	movlw	BANDMIN0		; point to data read in
	movwf	FSR0
	movlw	0x10			; and over 16 bytes of data,
	call	checksum		; checksum it.
	btfss	ALUSTA,Z		; skip if OK, else reset
	goto	BandTableInitFromRom	; reset to rom default
;
; now try secondary bandtable data. Again, if fails or if
; checksum wrong, revert to rom data
;
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	BandTableInitFromRom
	movlw	I2C_EEPROM | 0x0a	; i2c address
	movwf	INDF0
	movlr	BANK0
	swapf	BAND,W
	movlr	BANK2
	andlw	0xf0
	movwf	INDF0			; i2c subaddress
	movlw	8			; 8 byte secondary band data
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	Bdata_start
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
	btfsc	ALUSTA,Z		; if not read OK
	goto	BandTableInitFromRom	; reset to rom default
	movlw	Bdata_start		; point to data read in
	movwf	FSR0
	movlw	8			; and over 8 bytes of data,
	call	checksum		; checksum it.
	btfsc	ALUSTA,Z		; skip if OK, else reset
	return
BandTableInitFromRom:
	movlw	BAND_TABLE_ENTRY_SIZE/2
	movwf	TEMPB0			; move 24 bytes of data ( 12 words )
	movlw	Bdata_start
	movwf	FSR0			; move to band data RAM
	movfp	BAND,WREG
	mullw	BAND_TABLE_ENTRY_SIZE/2	; multiply band by ( size of entries / 2)		
	movfp	PRODL,WREG
	addlw	LOW band_table_1	; add to create index
	movwf	TBLPTRL			; set up table pointer
	movlw	HIGH band_table_1
	addwfc	PRODH,W
	movwf	TBLPTRH
table_move:
	TABLRD 	0, 1, I 		; Dummy read, get hibyte
band_tab_init_0:
	TLRD 	1, INDF0		; store hibyte & increment
	TABLRD 	0, 1, INDF0		; store lobyte & increment
	decfsz	TEMPB0,F
	goto	band_tab_init_0
	return

;----------------------------------------------------------------------------
; initialise system data from EEPROM
;-----------------------------------------------------------------------------
;
SysConfig:
;
; read DDS clock data from eeprom ( or set to rom defult if not )
; note 12 bytes; 2 4 byte DDS frequencies, a checksum and a 3 byte CW offset
;
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	sys_no_dds_data		; force to rom default if no eeprom buffers
	movlw	I2C_EEPROM | 0x0e	; i2c address
	movwf	INDF0
	movlw	0x80
	movwf	INDF0			; i2c subaddress
	movlw	D'16'			; 2 * 4 byte dds + checksum + 3 byte CWoffs + sideband offsets
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	DDS1_osc0
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
	btfsc	ALUSTA,Z		; if not read OK
	goto	sys_no_dds_data		; reset to rom default
	movlw	DDS1_osc0		; point to data read in
	movwf	FSR0
	movlw	D'16'			; and over 16 bytes of data,
	call	checksum		; checksum it.
	btfss	ALUSTA,Z		; skip if OK, else reset to rom default
sys_no_dds_data:
	call	reset_dds_clock_data
;
;  read (SHang - BAND + 1) bytes of data from eeprom address 00 into BAND
;  this causes the following to be read:
;	BAND
;	OPTO_2_MODE
;	MIN_ACCEL
;	UI_Flags
;	S0
;	Smeter_slope_A
;	Smeter_slope_B
;	dBm_offset
;	dB_per_spoint
;	S0CW
;	SHang
;	Pmeter_slope_A
;	Pmeter_slope_B
;	UI_Flags2
;
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z		; if cannot access I2C
	goto	sys_no_eeprom		; then go reset to defaults
	movlw	I2C_EEPROM		; i2c address
	movwf	INDF0
	clrf	INDF0,F			; i2c subaddress zero
	movlw	(UI_Flags2 - BAND + 1)
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	BAND
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
	btfsc	ALUSTA,Z
	goto	sys_no_eeprom		; read error - reset
;
; check band
;
	movfP	BAND,WREG		; get band
	sublw	NUM_BANDS-1		; check if exceeds number of bands -1
	btfss	ALUSTA,C		; and if so,
	clrf	BAND,F			; go to band 0
;
; check acceleration. If set to zero, turn acceleration off
;
	tstfsz	MIN_ACCEL
	goto	accel_not_zero
	bsf	UPDATE_2,FREQ_ACCEL	; set flag to say no acceleration
accel_not_zero:
;
; now check opto_2_mode 
; valid values are zero or between first and last
;
	movlw	pb_opto2_first - pb_func_index	; check if < first
	subwf	OPTO_2_MODE,W
	btfss	ALUSTA,C			; and if so,
	goto	error_opto_2			; error - reset it
	movlw	pb_opto2_last - pb_func_index	; now check > last
	subwf	OPTO_2_MODE,W
	btfsc	ALUSTA,C			; and if so
error_opto_2:
	clrf	OPTO_2_MODE,F			; reset it
;
; switch backlight on if requested - not done here - see init
; AND FORCE TO MENU 0 

	movlw	(1<<UIF_Backlight)|(1<<UIF_Backlight2)||(1<<UIF_Vox_off)||(1<<UIF_Keyer_Type)
	andwf	UI_Flags,F
;	btfss	ALUSTA,Z
;	call	backlight_on
;
; Read (RXFlags - Volume + 1) bytes of data from eeprom address 0x10 into Volume
; This initialises:
;	Volume
;	Mic Gain
;	IF Gain
;	HangTime
;	Vox Gain
;	Antivox
;	Vox Delay
;	RF Power
;	RX Flags
;
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	sys_no_eeprom
	movlw	I2C_EEPROM		; i2c address
	movwf	INDF0
	movlw	0x10
	movwf	INDF0			; i2c subaddress
	movlw	(RXFlags - Volume + 1)
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	Volume
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
	btfsc	ALUSTA,Z
	goto	sys_no_eeprom

	movlw	3
	andwf	RXFlags,F		; force all flags other than mode to zero
;
; Read (Keyer_hang - Keyer_dot + 1) bytes of data from eeprom 
; This initialises:
;	Keyer_dot
;	Keyer_hang
;
#ifdef	INCLUDE_KEYER
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	sys_no_eeprom
	movlw	I2C_EEPROM		; i2c address
	movwf	INDF0
	movlw	0x30
	movwf	INDF0			; i2c subaddress
	movlw	(Keyer_hang - Keyer_dot + 1)
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	Keyer_dot
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
	btfsc	ALUSTA,Z
	goto	sys_no_eeprom
#endif
;
; initialise dsp control
; all dsp control words and data
;
#ifdef COMMON_DSP_FUNCTIONS
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	sys_no_eeprom
	movlw	I2C_EEPROM		; i2c address
	movwf	INDF0
	movlw	0x40
	movwf	INDF0			; i2c subaddress
	movlw	DSP_INIT_END - DSP_CTRL + 1
	movwf	INDF0			; count
	movlw	(HIGH DSP_CTRL) << 4	; BSR
	movwf	INDF0			; page
	movlw	LOW DSP_CTRL
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
	call	i2c_update_dsp

	movlr	BANK1
	bsf	DSP_TX_CTRL, DSP_TX_PROC_AGC_ON
	movlr	BANK0

#endif

; Read bandscan configuration data

#ifdef	BANDSCAN
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	sys_no_eeprom
	movlw	I2C_EEPROM		; i2c address
	movwf	INDF0
	movlw	0x60
	movwf	INDF0			; i2c subaddress
	movlw	ScanStartDelay - ScanStepSSBHigh + 1
	movwf	INDF0			; count
	movlw	(HIGH ScanStepSSBHigh) << 4	; BSR
	movwf	INDF0			; page
	movlw	LOW ScanStepSSBHigh
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
#endif

;
;  read pushbutton data from eeprom 
;
	call	read_pushbutton_data
	goto	sys_config_end
;
; jump to here if failures happen in above code reading from eeprom
;
sys_no_eeprom:
	clrf	BAND,F
	clrf	OPTO_2_MODE,F
	movlw	6
	movwf	MIN_ACCEL
	movlw	2
	movwf	Volume
sys_config_end:
;
; check s meter data
;
#ifdef ONSCREEN_SMETER
	tstfsz	Smeter_slope_B		; ensure slope B ( divisor )
	goto	smeter_slope_b_ok	; is non zero
	incf	Smeter_slope_B,F	; set to 1 if it is
smeter_slope_b_ok:
	tstfsz	Pmeter_slope_B		; ensure slope B ( divisor )
	goto	pmeter_slope_b_ok	; is non zero
	incf	Pmeter_slope_B,F	; set to 1 if it is
pmeter_slope_b_ok:
	movlw	2
	cpfsgt	dB_per_spoint		; bounds check db per spoint
	goto	smeter_spoint_nok	; in range 3 .. 6
	movlw	6
	cpfsgt	dB_per_spoint
	goto	smeter_spoint_ok
smeter_spoint_nok:
	movlw	6			; and set to 6 if not
	movwf	dB_per_spoint
smeter_spoint_ok:
#endif
#ifdef	INCLUDE_KEYER
;
; check keyer data values
;
	movlw	1
	cpfsgt	Keyer_dot
	goto	reset_keyer_data
	movlw	D'32'
	cpfslt	Keyer_dot
	goto	reset_keyer_data
	movlw	2
	cpfsgt	Keyer_hang
	goto	reset_keyer_data
	movlw	D'10'
	cpfsgt	Keyer_hang
	goto	no_reset_keyer_data
reset_keyer_data:
	movlw	D'10'
	movwf	Keyer_dot
	movlw	D'5'
	movwf	Keyer_hang
no_reset_keyer_data:
#endif
	return

;----------------------------------------------------------------------------
; misc helper routines to above code ( and elsewhere )
;  note that eeprom_wait is also used as an entry point by write functions
;-----------------------------------------------------------------------------
read_eeprom_and_wait:
	movlw	B'00000011'	; no buffer clear, no retry, subaddress, read
read_device_and_wait:
	movwf	INDF0		; set command in byte 7 of I2C data structure
	call	I2C_Startup	; and go to it !
eeprom_wait:
	incf	INDF1,W		; wait until complete
#ifndef DEBUG_VERSION
	btfss	ALUSTA,Z
	goto	eeprom_wait 
#endif
	clrf	INDF1,F		; free the buffer
	incf	FSR1,F		; point to result register
	bcf	ALUSTA,Z	; clear zero bit
	btfss	INDF1,7		; check if operation succeeded
	bsf	ALUSTA,Z	; and set zero if it failed
	clrf	BSR,F		; reinstate BSR
	return

; ---------------------------------------------------------------------------
; band change helper
; ---------------------------------------------------------------------------
; initalise band tables
; then try to get stored vco data
; The data then needs checking and flags setting.

do_band_change:
	call	BandTableInit
;
; now get VCO data from eeprom
;
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	do_band_change_0	; if fails, skip over read
	movlw	I2C_EEPROM | 0x04	; i2c address
	movwf	INDF0			; ram page 2
	movlr	BANK0
	swapf	BAND,W			; get band << 4
	andlw	0xf0			; and use this as subaddress
	movlr	BANK2
	movwf	INDF0			; i2c subaddress
	movlw	8			; initalise 
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	DDS1A_0			; initalise 
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
;
; whether it succeeded or failed, now check data values
;
do_band_change_0:
	movlw	DDS1A_3
	movwf	FSR1
	call	do_opto_1_checkminimum	; check dds
	bsf	UPDATE_1,DDS1_CHANGED	; mark dds1 as changed to set it
	bsf	UPDATE_1,DDS1_REWRITE	; mark as changed for eeprom update
	clrf	EE_FREQ_IDLE,F		; but set timer to now for delay
	bsf	UPDATE_2,MODE_CHANGE	; mark mode changes for attention
	bcf	RXFlags,RXF_TX_B	; force TX to VCOA
	bcf	RXFlags,RXF_RX_B	; force RX to VCOA
	clrf	DISP_BUF,F
	movlw	s_blank_fB-string_index	; blank VCOB display
#ifdef INCLUDE_GRAPHICS
	call	g_pstr_xy
#else
	call	p_string_xy
#endif
	return

;
; ----------------------------------------------------------------
; helper utility to clear ram
; ----------------------------------------------------------------
;
init_clr_ram:
	movlw	MKPageendb		; initialize file registers (RAM)
	movwf	FSR0			; page assumed to be set before call here
Init0:
	clrf	INDF0,F			; clear memory byte 
	movfp	FSR0,WREG
	sublw	MKPagebegb		; end of file register page ?
	btfss	ALUSTA,Z
	goto	Init0
	return

;
; ----------------------------------------------------------------
; helper utility for checksums
; ----------------------------------------------------------------
;
; given a count in WREG
; and a pointer in FSR0 ( ASSUMES SET TO AUTO INCREMENT )
; calculate a checksum.
; returns Z set if checksum correct, else Z clear
; returns checksum error in WREG ( 0 if no error )
; so if a checksum is wrong, add the result value to
; any of the checksummed bytes to cause it to be correct
; next time
;
; uses PRODL as loop counter

checksum:
	movwf	PRODL
	movlw	0
csloop:
	addwf	INDF0,W
	decfsz	PRODL,F
	goto	csloop
	sublw	CKSUM_TOTAL
	return

;
; ----------------------------------------------------------------
; reset dss clock data to rom default
; ----------------------------------------------------------------


reset_dds_clock_data:
	movlw	LOW dds1_init	; point to dds default data
	movwf	TBLPTRL
	movlw	HIGH dds1_init
	movwf	TBLPTRH
	movlw	8		; move 8 words, 16 bytes - 2 DDSes, cw & sideband offsets
	movwf	TEMPB0
	movlw	DDS1_osc0	; move to DDS1_osc0
	movwf	FSR0
	goto	table_move	; set up data
	;; return implicit in goto

;
; ----------------------------------------------------------------
; misc events - like handling cw filter changes
; ----------------------------------------------------------------
;
misc_events:
	btfss	UPDATE_2,FILTER_CHANGE	; change marked ?
	return				; no return
	movlb	BANK5
	btfsc	RXFlags2, RXF2_CWFilter	; yes. set narrow ?
	bsf	PORTF_COPY,PWR_RXFILTCW	; yes. set pin high
	btfss	RXFlags2, RXF2_CWFilter	; set wide ?
	bcf	PORTF_COPY,PWR_RXFILTCW	; yes. set pin low
	bcf	UPDATE_2,FILTER_CHANGE	; change marked ?
	movfp	PORTF_COPY,PORTF	; make the actual port F change
	return


;
; ----------------------------------------------------------------
; Idle events - like handling backlight control
; ----------------------------------------------------------------
;
; backlight settings
;	00 = off, 01 = on - ignore these
;	10 = long delay, 11 = short delay, handle these
;	For 10 and 11 settings, switch lamp off if delay exceeded
;	short delay 8 minutes 28 seconds ( count reaches 2 )
;	long delay 34 minutes 8 seconds ( count reaches 8 )

Do_Idle_Event:
	incf	Idle_Count,W		; is count sitting at 255 ?
	btfsc	ALUSTA,Z		; if so, Z will be set
	return				; so just return
#ifdef BACKLIGHT_CONTROL
	incf	Idle_Count,F		; bump 4 minute 16 second timer
	btfss	UI_Flags,UIF_Backlight2 
	return				; ignore 00 & 01
	btfss	UI_Flags,UIF_Backlight	; if not 11 ( i.e. 10 )
	movlw	8			; long delay mask
	btfsc	UI_Flags,UIF_Backlight	; if not 10 ( i.e. 11 )
	movlw	2			; short delay mask
	andwf	Idle_Count,W		; is count reached ?
	btfsc	ALUSTA,Z		; if so, Z clear
	return
	setf	Idle_Count,F		; mark event of auto switch off
backlight_off:
	bcf	PORTF_COPY,PWR_BACKLIGHT	; power off
	goto	backlight_common_exit

backlight_on:
	bsf	PORTF_COPY,PWR_BACKLIGHT	; power on
backlight_common_exit:
	movlb	BANK5
	movfp	PORTF_COPY,PORTF	; make the actual port F change
#endif
	return

; ----------------------------------------------------------------
;  read pushbutton data from eeprom 
; ----------------------------------------------------------------
; a helper utility that reads pushbutton data according to
; current menu selection in UI_Flags
;
read_pushbutton_data:
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	goto	read_pushbutton_data_check

	movlw	I2C_EEPROM | 0x0e	; i2c address
	movwf	INDF0
	movlr	BANK0
	swapf	UI_Flags,W		; get menu ( bits 2,3 ) into bits 6,7
	movlr	BANK2
	rrcf	WREG,F			; put them in bits 5,6
	andlw	0x60			; gives index into eeprom data
	movwf	INDF0			; and use this as i2c subaddress
	movlw	(PBindirectEnd - PBindirect + 1)
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	PBindirect
	movwf	INDF0			; ram pointer
	call	read_eeprom_and_wait
;	btfsc	ALUSTA,Z			; no point checking for error
;	goto	read_pushbutton_data_check	; coz we go there anyway !
;
; check that pushbuttons have valid values, clear any that do not
;
read_pushbutton_data_check:
	movlw	(PBindirectEnd - PBindirect + 1)
	movwf	I
	movlw	PBindirect
	movwf	FSR1
read_pushbutton_data_check_loop:
	movfp	INDF1,WREG
	sublw	(pb_func_index_end - pb_func_index - 1)
	btfss	ALUSTA,C
	clrf	INDF1,F
	incf	FSR1,F
	decfsz	I,F
	goto	read_pushbutton_data_check_loop
	return


