;-------------------------------------------------------------------------------
; NAME: lcd.asm
;
; LABEL PREFIX: "XCVR"
;
; COMPANY:	
;
; PURPOSE:	This is the low level code for the display
;
;
;REVISION HISTORY:
;		7 jan 99	G8KBB	createdfor pwr meter
;		aug 99		G8KBB	reused for transciever
;		apr 00		G8KBB	ported for 17C766
;
;-------------------------------------------------------------------------------		
;
;----------------------------------------------------------------------------
; initialise LCD display
;-----------------------------------------------------------------------------
lcdInit:
	movlb	BANK1
#ifdef DISPLAY_320_240
	bsf	PORTC,LCDRW		; set WR high
	bsf	PORTC,LCDRS		; set RD high
	bsf	PORTC,LCDCS		; chip select high
	bcf	PORTC,LCDRESET		; assert reset low
	movlw	D'20'
	call	small_delay		; wait 2 msec
	bsf	PORTC,LCDRESET		; reset high
	movlw	D'58'
	call	small_delay		; wait 5.5 msec
	bcf	PORTC,LCDE		; set Register Select low
	movlw	D'10'
	call	small_delay		; wait 1 msec
#else
	bcf	PORTC,LCDRW		; set RW low
	bcf	PORTC,LCDE		; set E low
#ifdef HD44780_X_2_DISPLAY
	bcf	PORTC,LCDE2		; set E2 low
#endif
	bcf	PORTC,LCDRS		; set Register Select low
	bcf	PORTC,LCDRESET		; assert reset low
	movlw	D'20'
	call	small_delay		; wait 2 msec
	bsf	PORTC,LCDRESET		; reset high
	movlw	D'58'
	call	small_delay		; wait 5.5 msec
#ifndef HD44780_X_2_DISPLAY
	bcf	PORTC,LCDCS		; enable display
#endif
	movlw	D'10'
	call	small_delay		; wait 1 msec
#endif

#ifdef HD44780_X_2_DISPLAY
	bsf	RXFlags2,RXF2_LCD_USE_E2
	movlw	0x38
	call	LCDCTL
	movlw	0x08
	call	LCDCTL
	movlw	0x01
	call	LCDCTL
	movlw	0x06
	call	LCDCTL
	movlw	0x0c
	call	LCDCTL
#ifdef NON_GRAPHIC_SMETER
	movlw	0x40
	call	LCDCTL
	movlw	(CG_Data_End - CG_Data)
	movwf	TEMPB0			; byte count to write / 2
	movlw	LOW CG_Data
	movwf	TBLPTRL			; set up table pointer
	movlw	HIGH CG_Data
	movwf	TBLPTRH
	TABLRD 	0, 1, I 		; Dummy read, get hibyte
cgram_init_0:
	TLRD 	1, WREG			; store hibyte & increment
	call	LCDDATA
	TABLRD 	0, 1, WREG		; store lobyte & increment
	call	LCDDATA
	decfsz	TEMPB0,F
	goto	cgram_init_0
#endif
	bcf	RXFlags2,RXF2_LCD_USE_E2
	movlw	0x38
	call	LCDCTL
	movlw	0x08
	call	LCDCTL
	movlw	0x01
	call	LCDCTL
	movlw	0x06
	call	LCDCTL
	movlw	0x0c
	call	LCDCTL
#else
#ifdef SIMPLE_DISPLAY
	movlw	0x38
	call	LCDCTL
	movlw	0x08
	call	LCDCTL
	movlw	0x01
	call	LCDCTL
	movlw	0x06
	call	LCDCTL
	movlw	0x0c
	call	LCDCTL
#else
#ifdef DISPLAY_320_240
	movlw	0x40			; SYSTEM SET
	call	LCDCTL
	movlw	0x30			; 8 bit chars ( 0x34 for 16 bit )
	call	LCDDATA
	movlw	0x87			; 2 line AC ( 0x07 for 16 line )
	call	LCDDATA
	movlw	0x09			; 10 lines / char
	call	LCDDATA
	movlw	LCD_COLS-1		; 40 chars / line
	call	LCDDATA
	movlw	0x34			; LCD_COLS+3 <= T/CR <= 42
	call	LCDDATA
	movlw	LCD_PIXELS_Y-1		; 240 lines
	call	LCDDATA
	movlw	LCD_COLS		; AP = 40
	call	LCDDATA
	movlw	0x00
	call	LCDDATA
	movlw	0x44			; SCROLL
	call	LCDCTL
	movlw	LOW TEXT_LAYER_ADDR	; ls byte of layer 1 address
	call	LCDDATA
	movlw	HIGH TEXT_LAYER_ADDR	; ms byte of address
	call	LCDDATA
	movlw	LCD_PIXELS_Y		; 240 lines
	call	LCDDATA
	movlw	LOW GRAPHICS_LAYER_ADDR	; ls byte of layer 2
	call	LCDDATA
	movlw	HIGH GRAPHICS_LAYER_ADDR ; ms byte of address
	call	LCDDATA
	movlw	LCD_PIXELS_Y		; 240 lines
	call	LCDDATA
	movlw	0x5b			; OVLAY
	call	LCDCTL
	movlw	0x00			; text + graphics
	call	LCDDATA
	movlw	0x58			; DISPLAY OFF
	call	LCDCTL
	movlw	0x4C			; CURSOR DIRECTION
	call	LCDCTL
	movlw	0x5C			; CGRAM ADDRESS
	call	LCDCTL
	movlw	LOW CGRAM_ADDR		; ls byte address
	call	LCDDATA
	movlw	HIGH CGRAM_ADDR		; ms byte of address
	call	LCDDATA
#else
	movlw	0x00			; access LCD Mode control register
	call	LCDCTL
	movlw	LCD_MODE		; set LCD mode register
	call	LCDDATA	
	movlw	0x01			; access LCD character pitch register
	call	LCDCTL
	movlw	LCD_PITCH		; set pitch
	call	LCDDATA	
	movlw	0x02			; access chars / line register
	call	LCDCTL
	movlw	LCD_CPL			; set characters per line
	call	LCDDATA	
	movlw	0x03			; access controller duty cycle register
	call	LCDCTL
	movlw	LCD_DUTY		; set duty cycle
	call	LCDDATA
	movlw	0x04			; access cursor position register
	call	LCDCTL
	movlw	7			; set it to row 8
	call	LCDDATA	
	movlw	0x08			; access display start low address
	call	LCDCTL
	movlw	0x00			; set to 0
	call	LCDDATA	
	movlw	0x09			; access display start high address
	call	LCDCTL
	movlw	0x00			; set to 0
	call	LCDDATA
#endif
#endif
#endif

;----------------------------------------------------------------------------
; clear display ram by setting to start and writing space to all locations
; note for double controller displays this does not clear second screen
; but then this is overkill - we gave it a clear display command earlier
;-----------------------------------------------------------------------------

	clrf	I,F			; I & J zero means start of display
	clrf	J,F
	call	LCDGOTOXY
	movlw	LCD_ROWS
	movwf	I
LCDInit1:
	movlw	LCD_COLS
	movwf	J
LCDInit0:
	movlw	0x20
	call	LCDDATA
#ifndef DEBUG_VERSION
	decfsz	J,F
	goto	LCDInit0
	decfsz	I,F
	goto	LCDInit1	
#endif
#ifdef DISPLAY_320_240
	clrf	I,F			; I & J zero means start of display
	clrf	J,F
	call	LCDGOTOXYGRAPH
	movlw	LCD_PIXELS_Y
	movwf	I
LCDInit2:
	movlw	LCD_PIXELS_X / 8
	movwf	J
LCDInit3:
	movlw	0x00
	call	LCDDATA
#ifndef DEBUG_VERSION
	decfsz	J,F
	goto	LCDInit3
	decfsz	I,F
	goto	LCDInit2	
#endif
	movlw	0x59			; DISPLAY ON
	call	LCDCTL
	movlw	0x14			; no cursor, layers 1 and 2 on
	call	LCDDATA
#endif
	return

; ------------------------------------------------------------------------
; small delay function

small_delay:
	movwf	J
lcdinitloop0:
	clrf	I,F
lcdinitloop1:
#ifndef DEBUG_VERSION
	decfsz	I,F
	goto	lcdinitloop1
	decfsz	J,F
	goto	lcdinitloop1
#endif
	return
;
;----------------------------------------------------------------------------
; send data byte in W to LCD controller instruction register
;-----------------------------------------------------------------------------
; accepts parameter in W regsiter
; sends that to LCD display control register
; other labels are internal and not program entry points

LCDCTL:
	bsf	CPUSTA,GLINTD	; disable all interrupts
;
	movlb	BANK1
	movwf	PORTD		; put data byte in port D register
	movlw	0FFh		; set data bus to input
	movwf	DDRD
#ifdef DISPLAY_320_240
lcdctlrdy:
	bcf	PORTC,LCDCS	; chip select low
	bsf	PORTC,LCDRS	; ensure RD high ( in case looping )
	bsf	PORTC,LCDRW	; make sure WR high for read
#ifdef INVERSE_LCD_RS
	bsf	PORTC,LCDE	; register select high
#else
	bcf	PORTC,LCDE	; register select low
#endif
	nop
	bcf	PORTC,LCDRS	; now set RD low to read cotroller
	clrwdt
	nop
	nop
	btfsc	PORTD,LCDBF	; if busy flag is clear...
	goto	lcdctlrdy	; ... then wait till ready
	bsf	PORTC,LCDRS	; set RD high to end read cycle
#ifdef INVERSE_LCD_RS
	bcf	PORTC,LCDE	; register select high
#else
	bsf	PORTC,LCDE	; register select low
#endif

#else ; not DISPLAY_320_240
lcdctlrdy:
	bcf	PORTC,LCDE	; ensure E low ( in case looping )
	bsf	PORTC,LCDRW	; make sure R/W high for read
#ifdef HD44780_X_2_DISPLAY
	bcf	PORTC,LCDE2	; ensure E low ( in case looping )
#endif
#ifdef INVERSE_LCD_RS
	bsf	PORTC,LCDRS	; register select high
#else
	bcf	PORTC,LCDRS	; register select low
#endif
	movlw	2		; delay loop of ( n*2)+1 cycles
lcdctlrdy0:
	decfsz	WREG,F
	goto	lcdctlrdy0
#ifdef HD44780_X_2_DISPLAY
	movlw	(1 << LCDE)
	btfsc	RXFlags2,RXF2_LCD_USE_E2
	movlw	(1 << LCDE2)
	iorwf	PORTC,F
#else
	bsf	PORTC,LCDE	; now set E high to read cotroller
#endif
	clrwdt
	movlw	2		; delay loop of ( n*2)+1 cycles
lcdctlrdy1:
	decfsz	WREG,F
	goto	lcdctlrdy1
	btfsc	PORTD,LCDBF		; if busy flag is set...
	goto	lcdctlrdy	; ... then wait till ready
	bcf	PORTC,LCDE	; set E low to end read cycle
#ifdef HD44780_X_2_DISPLAY
	bcf	PORTC,LCDE2	; ensure E low ( in case looping )
#endif
#ifdef INVERSE_LCD_RS
	bsf	PORTC,LCDRS	; register select high
#else
	bcf	PORTC,LCDRS	; register select low
#endif
#endif  ; DISPLAY_320_240
	goto	lcd_send	; drop thru to common code

;
;----------------------------------------------------------------------------
; send data byte in W to LCD controller data register
;-----------------------------------------------------------------------------
; accepts parameter in W regsiter
; sends that to LCD display data register
; LCDDATA is the only entry point

LCDDATA:
	bsf	CPUSTA,GLINTD	; disable all interrupts
;
	movlb	BANK1
	movwf	PORTD		; put data byte in port D register
	movlw	0FFh		; set data bus to input
	movwf	DDRD
#ifdef DISPLAY_320_240
lcddatardy:
	bcf	PORTC,LCDCS	; chip select low
	bsf	PORTC,LCDRS	; ensure RD high ( in case looping )
	bsf	PORTC,LCDRW	; make sure WR high for read
#ifdef INVERSE_LCD_RS
	bsf	PORTC,LCDE	; register select high
#else
	bcf	PORTC,LCDE	; register select low
#endif
	nop
	bcf	PORTC,LCDRS	; now set RD low to read controller
	nop
	nop
	clrwdt
	btfsc	PORTD,LCDBF	; if busy flag is clear...
	goto	lcddatardy	; ... then wait till ready
	bsf	PORTC,LCDRS	; set RD high to end read cycle
	nop
;
; note following code block is common to LCDCTL function above
;
lcd_send:
	clrf	DDRD,F		; set data bus to output
	nop
	nop
	bcf	PORTC,LCDRW	; pulse WR low ( then high )
	nop
	nop
	nop
	bsf	PORTC,LCDRW
	bsf	PORTC,LCDCS	; chip select high

#else ; not DISPLAY_320_240
lcddatardy:
	bcf	PORTC,LCDE	; ensure E low ( in case looping )
#ifdef HD44780_X_2_DISPLAY
	bcf	PORTC,LCDE2	; ensure E low ( in case looping )
#endif
	bsf	PORTC,LCDRW	; make sure R/W high for read
#ifdef INVERSE_LCD_RS
	bsf	PORTC,LCDRS	; register select high
#else
	bcf	PORTC,LCDRS	; register select low
#endif
#ifdef HD44780_X_2_DISPLAY
	movlw	(1 << LCDE)
	btfsc	RXFlags2,RXF2_LCD_USE_E2
	movlw	(1 << LCDE2)
	iorwf	PORTC,F
#else
	nop
	nop
	nop
	bsf	PORTC,LCDE	; now set E high to read cotroller
#endif
	movlw	2		; delay loop of ( n*2)+1 cycles
lcddata1:
	decfsz	WREG,F
	goto	lcddata1
	clrwdt
	btfsc	PORTD,LCDBF		; if busy flag is set...
	goto	lcddatardy	; ... then wait till ready
	bcf	PORTC,LCDE	; set E low to end read cycle
#ifdef HD44780_X_2_DISPLAY
	bcf	PORTC,LCDE2	; ensure E low ( in case looping )
#endif
#ifdef INVERSE_LCD_RS
	bcf	PORTC,LCDRS	; register select low
#else
	bsf	PORTC,LCDRS	; register select high
#endif
;
; note following code block is common to LCDCTL function above
;
lcd_send:
	clrf	DDRD,F		; set data bus to output
	bcf	PORTC,LCDRW	; set R/W low ( write )
	movlw	3		; delay loop of ( n*2)+1 cycles
lcdsend1:
	decfsz	WREG,F
	goto	lcdsend1
#ifdef HD44780_X_2_DISPLAY
	movlw	(1 << LCDE)
	btfsc	RXFlags2,RXF2_LCD_USE_E2
	movlw	(1 << LCDE2)
	iorwf	PORTC,F
#else
	bsf	PORTC,LCDE	; now set E high to read cotroller
#endif
	movlw	3		; delay loop of ( n*2)+1 cycles
lcdsend2:
	decfsz	WREG,F
	goto	lcdsend2
	bcf	PORTC,LCDE
#ifdef HD44780_X_2_DISPLAY
	bcf	PORTC,LCDE2	; ensure E low ( in case looping )
#endif
#endif  ; DISPLAY_320_240
	bcf	CPUSTA,GLINTD	; enable all interrupts
	return

;----------------------------------------------------------------------------
LCDDATAFAST:
	bsf	CPUSTA,GLINTD	; disable all interrupts

	movlb	BANK1
	movwf	PORTD		; put data byte in port D register
#ifdef HD44780_X_2_DISPLAY
	bcf	PORTC,LCDE2	; ensure E low ( in case looping )
#else
	bcf	PORTC,LCDCS	; chip select low
#endif
	bsf	PORTC,LCDRS	; ensure RD high ( in case looping )
	bsf	PORTC,LCDRW	; make sure WR high for read
#ifdef INVERSE_LCD_RS
	bsf	PORTC,LCDE	; register select high
#else
	bcf	PORTC,LCDE	; register select low
#endif
	clrwdt
	goto	lcd_send

;
;----------------------------------------------------------------------------
; set cursor position to X,Y location in display RAM
;-----------------------------------------------------------------------------
;
; change LCD cursor position to X,Y coordinate I,J
; then set register to data write ready for screen update
; note X,Y is 16 bit data value NOT direct x,y coordinate
; need to set :
;	I to ((Y * LCD_CPL ) + X ) & 0xff
;	J to ((Y * LCD_CPL ) + X ) >> 8
;
LCDGOTOXY:
#ifdef HD44780_X_2_DISPLAY
	bcf	RXFlags2,RXF2_LCD_USE_E2
	movlw	(LCD_ROWS/2)*LCD_COLS
	cpfslt	I
	goto	LCDGOTOXTNOFIX
	movfp	I,WREG
	goto	LCDGOTOXTNOFIX2
LCDGOTOXTNOFIX:
	bsf	RXFlags2,RXF2_LCD_USE_E2
	subwf	I,W
LCDGOTOXTNOFIX2:
	bsf	WREG,7
	goto	LCDCTL
#else
#ifdef SIMPLE_DISPLAY
	movfp	I,WREG
	bsf	WREG,7
	goto	LCDCTL
#else
#ifdef DISPLAY_320_240
	movlw	0x46
	call	LCDCTL
	movfp	I,WREG
	call	LCDDATA
	movfp	J,WREG
	call	LCDDATA
	movlw	0x42
	goto	LCDCTL	
#else
	movlw	0x0a
	call	LCDCTL
	movfp	I,WREG
	call	LCDDATA
	movlw	0x0b
	call	LCDCTL
	movfp	J,WREG
	call	LCDDATA
	movlw	0x0c
	goto	LCDCTL
#endif
#endif
#endif
;	return implicit in goto !
;	return

#ifdef DISPLAY_320_240
LCDGOTOXYGRAPH:
	movlw	0x46
	call	LCDCTL
	movlw	LOW GRAPHICS_LAYER_ADDR
	addwf	I,W
	call	LCDDATA
	movlw	LOW GRAPHICS_LAYER_ADDR	; do it again just to get carry
	addwf	I,W
	movlw	HIGH GRAPHICS_LAYER_ADDR
	addwfc	J,W
	call	LCDDATA
	movlw	0x42
	goto	LCDCTL	
#endif
;
; ---------------------------------- utility to print string --------------
; print string pointed to by string table entry followed by contents of DISP_BUF
;----------------------------------------------------------------------------
;
; p_string_xy_nobuf: 
;	as per p_string_xy but clears DISP_BUF
;
; p_string_xy: 
;	get string from string index table, position cursor, 
;	print string & print DISP_BUF contents
;	parameter: index in WREG into strings table & string in DISP_BUF
; 
;p_string_at_cursor:
;	print string & DISP_BUF contents
;	parameter: pointer to string in TBLPTR & string in DISP_BUF
;
;p_string_indirect_at_cursor:
;	print string & DISP_BUF contents
;	parameter: pointer to address of string in TBLPTR & string in DISP_BUF
;
;pr_bufptr_string:
;	print string in DISP_BUF at current cursor location
;	parameter: string in DISP_BUF
;
p_string_indirect_at_cursor:
	TABLRD 	0, 1, WREG 	; Dummy read, get hibyte
	TLRD 	1, TBLPTRH	; store hibyte as new ptr high
	TLRD 	0, TBLPTRL	; store lobyte as new ptr low
	goto	p_string_at_cursor

p_string_xy_nobuf:
	clrf	DISP_BUF,F	; do not display anything from disp_buf
p_string_xy:
	addlw	LOW string_index
	movwf	TBLPTRL
	movlw	HIGH string_index
	movwf	TBLPTRH
	btfsc	ALUSTA,C
	incf	TBLPTRH,F
	TABLRD 	0, 1, WREG 	; Dummy read, get hibyte
	TLRD 	1, TBLPTRH	; store hibyte as new ptr high
	TLRD 	0, TBLPTRL	; store lobyte as new ptr low
				; we are now pointing to string data. Starts with XY
	TABLRD 	0, 1, WREG 	; Dummy read, get hibyte
	TLRD 	1, I		; store hibyte as new ptr high
	TLRD 	0, J		; store lobyte as new ptr low
	call	LCDGOTOXY	; move cursor
p_string_at_cursor:
	TABLRD 	0, 1, WREG 		; Dummy read, get hibyte
p_str_move_0:
	TLRD 	1, WREG
	movpf	WREG, WREG		; check for zero
	btfsc	ALUSTA,Z
	goto	p_str_now_read
	call	LCDDATA
	TABLRD 	0, 1, WREG
	movpf	WREG, WREG		; check for zero
	btfsc	ALUSTA,Z
	goto	p_str_now_read
	call	LCDDATA
	goto	p_str_move_0
p_str_now_read:
;	goto	pr_string		; ***fall through to pr_string ***
;
; ---------------------------------- utility to print string --------------
; print string from DISP_BUF to current XY location
;----------------------------------------------------------------------------
;
pr_bufptr_string:
	movlw	DISP_BUF
	movwf	FSR0
pstring_loop2:
	movpf	INDF0,WREG
	btfsc	ALUSTA,Z
	return
	call	LCDDATA
	goto 	pstring_loop2


; ---------------------- LCD Graphics data for s meter -------------------
; ---------------------- when using alpha num displays -------------------
;
#ifdef NON_GRAPHIC_SMETER

; We define 5 display characters, 0..4.
; where char 0 has column 5 on
;            1 has columns 5 and 4 on
;            2             5,4 & 3
;            3             5,4,3 & 2
;            4             all on
;
; so data is as follows to be written to LCD
; starting at cgram address 0, writing 40 bytes
; which in 5x8 mode gives the char data as above

CG_Data:
	db	0x0,0x00,0x00,0x00,0x10,0x10,0x10,0x10
	db	0x0,0x00,0x00,0x00,0x18,0x18,0x18,0x18
	db	0x0,0x00,0x00,0x00,0x1C,0x1C,0x1C,0x1C
	db	0x0,0x00,0x00,0x00,0x1E,0x1E,0x1E,0x1E
	db	0x0,0x00,0x00,0x00,0x1F,0x1F,0x1F,0x1F
CG_Data_End:
#endif

; ------------------------- draw lines on display ---------------------------
#ifdef DISPLAY_320_240
draw_display:
#ifndef OMIT_VERTICAL_SPACER_LINE
	; draw vertical line to left of button labels
	movlw	0x4F			; CURSOR DIRECTION
	call	LCDCTL			; now set to 'down'
	movlw	D'31'
	movwf	I
	clrf	J,F
	call	LCDGOTOXYGRAPH
	movlw	SMETERYLOC+0x10
	movwf	PRODH
draw_disp_loop_0:
	movlw	0x02			; send byte of 0xff
	call	LCDDATAFAST		; send byte
	decfsz	PRODH,F			; and loop round j times
	goto 	draw_disp_loop_0	; write vertical line
#endif
	movlw	0x4C			; CURSOR DIRECTION
	call	LCDCTL			; and set to 'right'

	; draw line above s meter
	movlw	HIGH( (SMETERYLOC-0x0e)*LCD_COLS )
	movwf	J
	movlw	LOW( (SMETERYLOC-0x0e)*LCD_COLS )
	movwf	I
	call	draw_disp_loop_1

	; draw line below s meter
	movlw	HIGH( (SMETERYLOC+0x10)*LCD_COLS )
	movwf	J
	movlw	LOW( (SMETERYLOC+0x10)*LCD_COLS )
	movwf	I
draw_disp_loop_1:
	call	LCDGOTOXYGRAPH
	movlw	D'31'
	movwf	PRODH
draw_disp_loop_2:
	movlw	0xff			; send byte of 0xff
	call	LCDDATAFAST		; send byte
	decfsz	PRODH,F			; and loop round j times
	goto 	draw_disp_loop_2
	movlw	0xfe
	goto	LCDDATAFAST		; send byte
	;; return implicit in goto
#endif

