;**********************************************************************************************
;
; this is taken from the microchip application note and code modules
; It has been modified to move parameters into page 0 ram
;
;**********************************************************************************************
;       
;       32x16 Bit Unsigned Fixed Point Multiply 32 x 16 -> 48
;
;       Input:  32 bit unsigned fixed point multiplicand in AARGB0, AARGB1,
;               AARGB2, AARGB3
;               16 bit unsigned fixed point multiplier in BARGB0, BARGB1
;
;       Use:    CALL    FXM3216U
;
;       Output: 48 bit unsigned fixed point product in AARGB0, AARGB1, 
;               AARGB2, AARGB3, AARGB4, AARGB5
;
;       Result: AARG  <--  AARG * BARG
;
;       Max Timing:     54 clks
;
;       Min Timing:     54 clks
;
;       PM: 53              DM: 9
;
; modified from Microchip reference code to move AARGBx into main memory
;
		NOLIST

FXM3216U	MOVfp	AARGB3,WREG
		movpf	WREG,TEMPB3	

;		MOVFP	AARGB3,WREG
		MULWF	BARGB1
		MOVPF	PRODH,AARGB4
		MOVPF	PRODL,AARGB5
		
		MOVFP	AARGB2,WREG
		MULWF	BARGB0
		MOVPF	PRODH,AARGB2
		MOVPF	PRODL,AARGB3

		MULWF	BARGB1
		MOVPF	PRODL,WREG
		ADDWF	AARGB4,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB3,F
		CLRF	WREG,F
		ADDWFC	AARGB2,F

		MOVFP	TEMPB3,WREG
		MULWF	BARGB0
		MOVPF	PRODL,WREG
		ADDWF	AARGB4,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB3,F
		CLRF	WREG,F
		ADDWFC	AARGB2,F
	
		MOVFP	AARGB1,WREG
		MULWF	BARGB1
		MOVPF	PRODL,WREG
		ADDWF	AARGB3,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB2,F
		MOVFP	AARGB1,WREG
		MULWF	BARGB0
		CLRF	AARGB1,W
		ADDWFC	AARGB1,F
		MOVPF	PRODL,WREG
		ADDWF	AARGB2,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB1,F

		MOVFP	AARGB0,WREG
		MULWF	BARGB1
		MOVPF	PRODL,WREG
		ADDWF	AARGB2,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB1,F
		MOVFP	AARGB0,WREG
		MULWF	BARGB0
		CLRF	AARGB0,W
		ADDWFC	AARGB0,F
		MOVPF	PRODL,WREG
		ADDWF	AARGB1,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB0,F

		RETLW	0x00

;**********************************************************************************************
;**********************************************************************************************
;       
;       32x32 Bit Unsigned Fixed Point Multiply 32 x 32 -> 64
;
;       Input:  32 bit unsigned fixed point multiplicand in AARGB0, AARGB1,
;               AARGB2, AARGB3
;               32 bit unsigned fixed point multiplier in BARGB0, BARGB1,
;               BARGB2, BARGB3
;
;       Use:    CALL    FXM3232U
;
;       Output: 64 bit unsigned fixed point product in AARGB0, AARGB1, 
;               AARGB2, AARGB3, AARGB4, AARGB5, AARGB6, AARGB7
;
;       Result: AARG  <--  AARG * BARG
;
;       Max Timing:     125 clks
;
;       Min Timing:     125 clks
;
;       PM: 124              DM: 18
;
; corrected to move AARGBx out of P register area to general purpose ram
;
;
FXM3232U
		movfp	AARGB0,WREG
		MOVPF	WREG,TEMPB0
		MOVFP	AARGB1,WREG
		movpf	WREG,TEMPB1	
		MOVFP	AARGB2,WREG
		movpf	WREG,TEMPB2	
		MOVfp	AARGB3,WREG
		movpf	WREG,TEMPB3	

		MOVFP	AARGB3,WREG
		MULWF	BARGB3
		MOVPF	PRODL,TBLPTRL
		MOVPF	PRODH,TBLPTRH
		
		MOVFP	AARGB2,WREG
		MULWF	BARGB2
		MOVPF	PRODL,AARGB5
		MOVPF	PRODH,AARGB4

		MULWF	BARGB3
		MOVPF	PRODL,WREG
		ADDWF	TBLPTRH,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB5,F
		CLRF	WREG,F
		ADDWFC	AARGB4,F

		MOVFP	TEMPB3,WREG
		MULWF	BARGB2
		MOVPF	PRODL,WREG
		ADDWF	TBLPTRH,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB5,F
		CLRF	WREG,F
		ADDWFC	AARGB4,F
	
		MOVFP	AARGB1,WREG
		MULWF	BARGB3
		MOVPF	PRODL,WREG
		ADDWF	AARGB5,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB4,F
		MOVFP	AARGB1,WREG
		MULWF	BARGB2
		CLRF	AARGB3,W
		ADDWFC	AARGB3,F
		MOVPF	PRODL,WREG
		ADDWF	AARGB4,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB3,F

		MOVFP	TEMPB3,WREG
		MULWF	BARGB1
		MOVPF	PRODL,WREG
		ADDWF	AARGB5,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB4,F
		CLRF	AARGB2,W
		ADDWFC	AARGB3,F
		ADDWFC	AARGB2,F

		MOVFP	TEMPB2,WREG
		MULWF	BARGB1
		MOVPF	PRODL,WREG
		ADDWF	AARGB4,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB3,F
		CLRF	WREG,F
		ADDWFC	AARGB2,F

		MOVFP	TEMPB1,WREG
		MULWF	BARGB1
		MOVPF	PRODL,WREG
		ADDWF	AARGB3,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB2,F

		MOVFP	AARGB0,WREG
		MULWF	BARGB2
		MOVPF	PRODL,WREG
		ADDWF	AARGB3,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB2,F
		MOVFP	AARGB0,WREG
		MULWF	BARGB1
		CLRF	AARGB1,W
		ADDWFC	AARGB1,F
		MOVPF	PRODL,WREG
		ADDWF	AARGB2,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB1,F

		MOVFP	TEMPB0,WREG
		MULWF	BARGB3
		MOVPF	PRODL,WREG
		ADDWF	AARGB4,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB3,F
		CLRF	WREG,F
		ADDWFC	AARGB2,F
		ADDWFC	AARGB1,F

		MOVFP	TEMPB0,WREG
		MULWF	BARGB0
		MOVPF	PRODH,AARGB0
		MOVPF	PRODL,WREG
		ADDWF	AARGB1,F
		CLRF	WREG,F
		ADDWFC	AARGB0,F

		MOVFP	TEMPB3,WREG
		MULWF	BARGB0
		MOVPF	PRODL,WREG
		ADDWF	AARGB4,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB3,F
		CLRF	WREG,F
		ADDWFC	AARGB2,F
		ADDWFC	AARGB1,F
		ADDWFC	AARGB0,F

		MOVFP	TEMPB2,WREG
		MULWF	BARGB0
		MOVPF	PRODL,WREG
		ADDWF	AARGB3,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB2,F
		CLRF	WREG,F
		ADDWFC	AARGB1,F
		ADDWFC	AARGB0,F

		MOVFP	TEMPB1,WREG
		MULWF	BARGB0
		MOVPF	PRODL,WREG
		ADDWF	AARGB2,F
		MOVPF	PRODH,WREG
		ADDWFC	AARGB1,F
		CLRF	WREG,F
		ADDWFC	AARGB0,F

		MOVFP	TBLPTRL,WREG
		movpf	WREG,AARGB7
		MOVFP	TBLPTRH,WREG
		movpf	WREG,AARGB6

                RETLW   0x00

;**********************************************************************************************
;**********************************************************************************************

UDIV3115 macro
;
; Max Timing: 5+8+30*12+6 = 379 clks
;
; Min Timing: 5+8+30*11+6 = 349 clks
;
; PM: 5+8+30*14+6 = 439 DM: 8
;
	variable i

	MOVFP BARGB1,WREG
	SUBWF REMB1, F
	MOVFP BARGB0,WREG
	SUBWFB REMB0, F
	RLCF AARGB0, F

	RLCF AARGB0,W
	RLCF REMB1, F
	RLCF REMB0, F
	MOVFP BARGB1,WREG
	ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F
	RLCF AARGB0, F

	variable i = D'2'

	while i < D'8'

	RLCF AARGB0,W
	RLCF REMB1, F
	RLCF REMB0, F
	MOVFP BARGB1,WREG
	BTFSS AARGB0,LSB
	GOTO UADD15#v(i)
	SUBWF REMB1, F
	MOVFP BARGB0,WREG
	SUBWFB REMB0, F
	GOTO UOK15#v(i)

UADD15#v(i) ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F

UOK15#v(i) RLCF AARGB0, F

	variable i = i + 1

	endw

	RLCF AARGB1,W
	RLCF REMB1, F
	RLCF REMB0, F
	MOVFP BARGB1,WREG
	BTFSS AARGB0,LSB
	GOTO UADD158
	SUBWF REMB1, F
	MOVFP BARGB0,WREG
	SUBWFB REMB0, F
	GOTO UOK158

UADD158 ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F
UOK158 RLCF AARGB1, F

	variable i = D'9'

	while i < D'16'

	RLCF AARGB1,W
	RLCF REMB1, F
	RLCF REMB0, F
	MOVFP BARGB1,WREG
	BTFSS AARGB1,LSB
	GOTO UADD15#v(i)
	SUBWF REMB1, F
	MOVFP BARGB0,WREG
	SUBWFB REMB0, F
	GOTO UOK15#v(i)

UADD15#v(i) ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F

UOK15#v(i) RLCF AARGB1, F

	variable i = i + 1

	endw

	RLCF AARGB2,W
	RLCF REMB1, F
	RLCF REMB0, F
	MOVFP BARGB1,WREG
	BTFSS AARGB1,LSB
	GOTO UADD1516
	SUBWF REMB1, F
	MOVFP BARGB0,WREG
	SUBWFB REMB0, F
	GOTO UOK1516

UADD1516 ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F

UOK1516 RLCF AARGB2, F


	variable i = D'17'

	while i < D'24'

	RLCF AARGB2,W
	RLCF REMB1, F
	RLCF REMB0, F
	MOVFP BARGB1,WREG
	BTFSS AARGB2,LSB
	GOTO UADD15#v(i)
	SUBWF REMB1, F
	MOVFP BARGB0,WREG
	SUBWFB REMB0, F
	GOTO UOK15#v(i)

UADD15#v(i) ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F

UOK15#v(i) RLCF AARGB2, F

	variable i = i + 1

	endw

	RLCF AARGB3,W
	RLCF REMB1, F
	RLCF REMB0, F
	MOVFP BARGB1,WREG
	BTFSS AARGB2,LSB
	GOTO UADD1524
	SUBWF REMB1, F
	MOVFP BARGB0,WREG
	SUBWFB REMB0, F
	GOTO UOK1524

UADD1524 ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F

UOK1524 RLCF AARGB3, F

	variable i = D'25'

	while i < D'32'

	RLCF AARGB3,W
	RLCF REMB1, F
	RLCF REMB0, F
	MOVFP BARGB1,WREG
	BTFSS AARGB3,LSB
	GOTO UADD15#v(i)
	SUBWF REMB1, F
	MOVFP BARGB0,WREG
	SUBWFB REMB0, F
	GOTO UOK15#v(i)

UADD15#v(i) ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F

UOK15#v(i) RLCF AARGB3, F

	variable i = i + 1

	endw

	BTFSC AARGB3,LSB
	GOTO UOK15
	MOVFP BARGB1,WREG
	ADDWF REMB1, F
	MOVFP BARGB0,WREG
	ADDWFC REMB0, F
UOK15

	endm

;**********************************************************************************************
;**********************************************************************************************
;
; 31/15 Bit Unsigned Fixed Point Divide 31/15 -> 31.15
;
; Input: 31 bit unsigned fixed point dividend in AARGB0, AARGB1,AARGB2,AARGB3
; 15 bit unsigned fixed point divisor in BARGB0, BARGB1
;
; Use: CALL FXD3115U
;
; Output: 31 bit unsigned fixed point quotient in AARGB0, AARGB1
; 15 bit unsigned fixed point remainder in REMB0, REMB1
;
; Result: AARG, REM <-- AARG / BARG
;
; Max Timing: 2+379+2 = 383 clks
;
; Min Timing: 2+349+2 = 353 clks
;
; PM: 2+439+1 = 442 DM: 8
;
FXD3115U 
	CLRF REMB0, F
	CLRF REMB1, F
	UDIV3115
	RETLW 0x00

	LIST

