;-------------------------------------------------------------------------------
; NAME: opto.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the shaft encoder handler.
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;
;-------------------------------------------------------------------------------		
;
do_opto:
	bsf	CPUSTA,GLINTD	; disable all interrupts

	movfp	OPTO_1_COUNT,WREG	; get count - keep in W for store later !
	tstfsz	OPTO_1_COUNT	; if zero, do nothing and clear acceleration
	goto	opto_do_1	; else go process opto controller data
	clrf	OLD_OPTO_1,F	; clear acceleration value
	clrf	OLD_OPTO_2,F
	goto	opto_not_1 	;******note enable ints !
opto_do_1:

	btfsc	PB_PROG_FLAGS,7	; check if in button pgm mode
	goto	opto_but_prog	; if so, go to it !
	clrf	PB_PROG_FLAGS,F	; else kill any pending request
;
	tstfsz	OPTO_1_MODE	; check if button down changed mode
	goto	opto_but_prog	; button down ? go handle it
;
	clrf	OPTO_1_COUNT,F	; clear count ( as we have read it into W )
	bcf	CPUSTA,GLINTD	; re-enable interrupts
	btfsc	RXFlags,RXF_Lock
	goto	opto_not_1	; if locked, do not change freq
	btfsc	RXFlags,RXF_On_Xmit
	goto	opto_not_1	; if on TX, do not change freq
	movwf	BARGB1

	btfss	ModeFlags,6	; tuning reversed ?
	goto	opto_1_not_rev	; no, skip
	comf	BARGB1,F	; invert count by complement
	incf	BARGB1,F	; and increment
opto_1_not_rev:

	clrf	BARGB0,F
	clrf	SIGN,F		; set sign to positive ( up )
	btfss	BARGB1,7	; if count not negative
	goto	opto_1_not_neg	; then skip
	incf	SIGN,F		; else set sign to say negative
	comf	BARGB1,F	; invert count by complement
	incf	BARGB1,F	; and increment
opto_1_not_neg:
	movfp	BARGB1,WREG

	btfsc	UPDATE_2,FREQ_ACCEL ; velocity sensitive ?
	goto	opto_no_accel	; nope.
	movwf	K		; store new count in K for now
	subwf	MIN_ACCEL,W	; if count < min_for_accelerate
	btfsc	ALUSTA,C
	goto 	opto_current_rate	; then keep tuning at current rate do not accelerate
	movfp	K,WREG
	addwf	OLD_OPTO_1,F	; add it in to accumulator
	btfsc	ALUSTA,C
	incf	OLD_OPTO_2,F	; and add in carry
	btfss	OLD_OPTO_2,1	; now check for overflow
	goto	opto_limit_ok	; skip if OK
	movlw	2		; else force limit to 0x400
	movwf	OLD_OPTO_2
	clrf	OLD_OPTO_1,F
	goto	opto_limit_ok

opto_current_rate:
	movfp	K,WREG		; read counter value
	addwf	OLD_OPTO_1,W	; add 8 bit counter
	movwf	BARGB1		; to 16 bit BARGB
	movlw	0
	addwfc	OLD_OPTO_2,W
	movwf	BARGB0
	goto	opto_no_accel

opto_limit_ok:
	movfp	OLD_OPTO_1,WREG
	movwf	BARGB1
	movfp	OLD_OPTO_2,WREG
	movwf	BARGB0		; set 16 bit BARG to step up/down count

opto_no_accel:
	movlw	STEPB1		; AARG := step value ( skip byte 0 see below)
	movwf	FSR0
;;	movpf	INDF0,AARGB0
	clrf	AARGB0,F	; note - byte 0 dummy used for checksum
	movpf	INDF0,AARGB1
	movpf	INDF0,AARGB2
	movpf	INDF0,AARGB3
	call	FXM3216U

; Make FSR1 point to DDS to be used
; i.e. VCOA or VCOB depending on which
; is being used for receive
	movlw	DDS1A_3
	btfsc	RXFlags,RXF_RX_B
	movlw	DDS1B_3
	movwf	FSR1
	bcf	ALUSTA,FS3	; set post decrement FSR1
	bcf	ALUSTA,FS2	; must correct on exit !

; AARGB now holds count * step.
; need either to add or subtract from dds count
	btfsc	SIGN,0
	goto	do_opto_1_down
	movfp	AARGB5,WREG	; let DDS := DDS + (count*step)
	addwf	INDF1,F		; DDS1?_3
	movfp	AARGB4,WREG
	addwfc	INDF1,F		; DDS1?_2
	movfp	AARGB3,WREG
	addwfc	INDF1,F		; DDS1?_1
	movfp	AARGB2,WREG
	addwfc	INDF1,F		; DDS1?_0
	goto	do_opto_1_check
do_opto_1_down:
	movfp	AARGB5,WREG	; let DDS := DDS - (count*step)
	subwf	INDF1,F		; DDS1?_3
	movfp	AARGB4,WREG
	subwfb	INDF1,F		; DDS1?_2
	movfp	AARGB3,WREG
	subwfb	INDF1,F		; DDS1?_1
	movfp	AARGB2,WREG
	subwfb	INDF1,F		; DDS1?_0
	; fall throught
do_opto_1_check:
;
; Make FSR1 point to DDS to be used
	movlw	4
	addwf	FSR1,F
	; fall through
;
; ------------ second entry point for initialisation ------------
;
; requires that FSR1 be pointing to last byte of DDS to be checked

do_opto_1_checkminimum:
;
;	DDS register now contains candidate new dds value
;	now need to check it against band boundary values
	bcf	ALUSTA,FS3
	bcf	ALUSTA,FS2
;if dds < min for band
;	dds = min
	movfp	BANDMIN3,WREG	; let flag := DDS - (count*step) >= 0 ?
	subwf	INDF1,W		; DDS?_3
	movfp	BANDMIN2,WREG
	subwfb	INDF1,W		; DDS?_2
	movfp	BANDMIN1,WREG
	subwfb	INDF1,W		; DDS?_1
	movfp	BANDMIN0,WREG
	subwfb	INDF1,W		; note pointing to DDS?_0-1 now
; check condition code
; skip if OK to check min
	btfsc	ALUSTA,C
	goto	opto1_min_ok
; set to min and skip max test
	movlw	BANDMIN0	; move from BANDMIN0..BANDMIN3 to DDS1_0..3
	goto	opto1_limit_change

;else if dds > max
;	dds = max
;endif
opto1_min_ok:
	movlw	4
	addwf	FSR1,F		; point to last byte again

	movfp	INDF1,WREG	; let flag := DDS - (count*step) >= 0 ?
	subwf	BANDMAX3,W
	movfp	INDF1,WREG
	subwfb	BANDMAX2,W
	movfp	INDF1,WREG
	subwfb	BANDMAX1,W
	movfp	INDF1,WREG
	subwfb	BANDMAX0,W
; check condition code
; skip if OK
	btfsc	ALUSTA,C
	goto	opto1_max_ok
; set to max
	movlw	BANDMAX0	; move from BANDMAX0..BANDMAX3 to DDS1_0..3
opto1_limit_change:
	incf	FSR1,F		; note we were pointing one byte below
	bsf	ALUSTA,FS2	; set FSR1 to auto increment
	movwf	FSR0		; note indirect for future s/w change
	movpf	INDF0,INDF1
	movpf	INDF0,INDF1
	movpf	INDF0,INDF1
	movpf	INDF0,INDF1
opto1_max_ok:
	bsf	ALUSTA,FS3	; turn off auto change FSR1
;
; OK, we now have a new DDS1 count. It has been checked
; for min / max limits and forced accordingly
; Now need to update DDS and signal screen update
;
;call setdds1
	bsf	UPDATE_1,DDS1_CHANGED	;set update_fdisp_flag
	bsf	UPDATE_1,DDS1_REWRITE	;set update_fdisp_flag
	clrf	EE_FREQ_IDLE,F		; set timer to zero seconds
;
	goto	opto_not_1

; ------------------------- button programming ------------------------
; ----------------------and button down handler -----------------------
opto_but_prog:
	movwf	I		; save temp value
	clrf	J,F		; sign = positive
	btfss	I,7		; if positive ( ms bit clear )
	goto	opto_but_0	; skip sign bits
	comf	J,F		; sign = negative
	comf	I,F		; negate count
	incf	I,F
opto_but_0:
	movlw	OPTO_SLOW	; check to see if > slow count
	subwf	I,F		; by subtracting OPTO_SLOW		
	btfss	ALUSTA,C	; no carry - not enough yet so
	goto	opto_not_1	; ignore & wait by exiting
	btfss	J,7		; what sign was it ?
	goto	opto_but_1	; positive - then go subtract W
	movlw	OPTO_SLOW
	addwf	OPTO_1_COUNT,F
	goto	opto_but_2
opto_but_1:
	subwf	OPTO_1_COUNT,F	; reduce pending count
opto_but_2:
	bcf	CPUSTA,GLINTD	; re-enable interrupts
;
; OK we now have J clear for increment
;                  set for decrement
; now need to determine what we are going to do - i.e.
; what function is to be called. Depends on mode.
;
	movfp	OPTO_1_MODE,WREG ; if not zero, we are doing button programming
	addlw	0
	btfss	ALUSTA,Z	 ; otherwise a button is simply held down
	goto	opto_button_down ; so go service it
;
	movfp	PB_PROG_ID,WREG	; point to button indirect table
	addlw	PBindirect	; index by button in question
	movwf	FSR1
;
	btfsc	J,7		; decrement ?
	goto	opto_but_3	; yes, jump
	incf	INDF1,F		; increment
	movlw	pb_func_index_end - pb_func_index	; check bound : too much ?
	subwf	INDF1,W		; no carry means overflow
	btfsc	ALUSTA,C	
	clrf	INDF1,F
	goto	opto_but_4
opto_but_3:
	movlw	pb_func_index_end - pb_func_index - 1
	decf	INDF1,F
	subwf	INDF1,W
	btfss	ALUSTA,C
	goto	opto_but_4
	movlw	pb_func_index_end - pb_func_index - 1
	movwf	INDF1
opto_but_4:
	btfss	PB_PROG_ID,4
	bsf	EEPROM_UPD_2,BUTTON_A_CH
	btfsc	PB_PROG_ID,4
	bsf	EEPROM_UPD_2,BUTTON_B_CH

	movfp	PB_PROG_ID,WREG
	call	pb_label
	goto	opto_not_1

; ------------------------ a button is depressed, service it ------------
;
; when we get here, we know a button is depressed ( state == 0 )
; and opto 1 count exceeds step size with direction in J
; ( zero == 'up', ff = 'down' )
; with button ID in PB_PROG_ID
; and INDF pointing to button function number
; execute push button function setting PB_STATE to 80 or 81
; ( 80 = up, 81 = down )


opto_button_down:
	movfp	PB_PROG_ID,WREG	; point to button indirect table
	addlw	PBindirect	; index by button in question
	movwf	FSR1
;
	movfp	J,WREG
	andlw	1
	iorlw	0x82
	movwf	PB_STATE
	movfp	INDF1,WREG		; find function associated with it
	call	exec_pb_func		; execute push button
;obd_one:

; ---------------------------- opto 2 handler ----------------------------
;
opto_not_1:
	bsf	CPUSTA,GLINTD	; disable all interrupts
;;	btfss	CPUSTA,GLINTD	; whilst we check count ( PIC14000 )
;;	goto	opto_not_1	; and clear it if non zero

	movfp	OPTO_2_COUNT,WREG	; get count - keep in W for store later !
	addlw	0
	btfsc	ALUSTA,Z	; if zero, do nothing and clear acceleration
	goto	opto_not_2 	;******note enable ints !
; handle opto 2 event
	tstfsz	OPTO_2_MODE	; if zero, no function set
	goto	opto_not_1_0
	goto	opto_not_2	; no function set for opto 2
;
opto_not_1_0:
	movwf	I		; save temp value
	clrf	J,F		; sign = positive
	clrf	K,F		; counter for number of increments/ decrements
	btfss	I,7		; if positive ( ms bit clear )
	goto	opto_but_1_0	; skip sign bits
	comf	J,F		; sign = negative
	comf	I,F		; negate count
	incf	I,F
opto_but_1_0:
	movlw	OPTO_SLOW2	; check to see if > slow count
	subwf	I,F		; by subtracting OPTO_SLOW		
	btfss	ALUSTA,C	; no carry - not enough yet so
	goto	opto_but_1_3	; ignore & wait by exiting
	incf	K,F		; bump count
	btfss	J,7		; what sign was it ?
	goto	opto_but_1_1	; positive - then go subtract W
;	movlw	OPTO_SLOW2
	addwf	OPTO_2_COUNT,F
	goto	opto_but_1_2
opto_but_1_1:
	subwf	OPTO_2_COUNT,F	; reduce pending count
opto_but_1_2:
	goto	opto_but_1_0
opto_but_1_3:
	movfp	K,WREG
	addlw	0
	btfsc	ALUSTA,Z
	goto	opto_not_2
	bcf	ALUSTA,C
	rlcf	K,F
	bcf	CPUSTA,GLINTD	; re-enable interrupts
;
; OK we now have J clear for increment
;                  set for decrement
; now need to determine what we are going to do - i.e.
; what function is to be called. Depends on mode.
;
	movfp	J,WREG
	andlw	1
	iorlw	0x80
	iorwf	K,W
	movwf	PB_STATE
	movfp	OPTO_2_MODE,WREG	; find function associated with it
	addlw	PBindirect
	movwf	FSR1			; set up FSR as if called by pb handler
	movfp	OPTO_2_MODE,WREG		; find function associated with it
	call	exec_pb_func		; execute push button
;obd_two:

opto_not_2:
	bcf	CPUSTA,GLINTD	; re-enable interrupts
	return

