
;-------------------------------------------------------------------------------
; NAME: pb.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the handler for push buttons.
;
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;		10 05 00	G8KBB	Port to 17C766
;
;-------------------------------------------------------------------------------		
;

#ifdef KBD_4X4

pb_scan:
	movlr	BANK1			; switch scan data is in RAM page 1
	bsf	CPUSTA,GLINTD		; disable all interrupts
	movlw	0xf0
	andwf	SW_scanbase,F
	andwf	SW_scanbase+1,F
	andwf	SW_scanbase+2,F
	andwf	SW_scanbase+3,F
	andwf	SW_scanbase+4,F
	andwf	SW_scanbase+5,F
	andwf	SW_scanbase+6,F
	andwf	SW_scanbase+7,F
;
	swapf	SW_scanbase+1,W
	iorwf	SW_scanbase,W
	movwf	SW_A
	swapf	SW_scanbase+3,W
	iorwf	SW_scanbase+2,W
	movwf	SW_B
	swapf	SW_scanbase+5,W
	iorwf	SW_scanbase+4,W
	movwf	SW_C
	swapf	SW_scanbase+7,W
	iorwf	SW_scanbase+6,W
	movwf	SW_D
	bcf	CPUSTA,GLINTD		; enable all interrupts
	movfp	SW_A,WREG	; get current switch settings
	cpfseq	TSW_STATE	; equal to old setting ?
	goto	pbs_0		; if not go service it
	goto	pb_scan_next	; if so, skip over
pbs_0:
	cpfseq	SW_A_TEMP	; we keep a 'temp' value - is that the same ?
	goto	pbs_1		; if not, make it so and set debounce timer
	decfsz	SW_COUNT1,F	; else decrement debounce counter
	goto	pb_scan_next	; if not zero, keep waiting
	xorwf	TSW_STATE,W	; zero. exor in changed switch value
	movwf	TSW_CHANGED	; and save this to show which have changed
	movfp	SW_A_TEMP,WREG	; get the new value again
	movwf	TSW_STATE	; and store it as new state'
	goto	pb_scan_next	; go do next switch
pbs_1:
	movwf	SW_A_TEMP	; save new value in temp
	movlw	4		; and set debounce counters running
	movwf	SW_COUNT1
	movwf	SW_COUNT2
pb_scan_next:
	movfp	SW_B,WREG
	cpfseq	TSW_STATE+1
	goto	pbs_2
	goto	pb_scan_end1
pbs_2:
	cpfseq	SW_B_TEMP
	goto	pbs_3
	decfsz	SW_COUNT2,F
	goto	pb_scan_end1
	xorwf	TSW_STATE+1,W
	movwf	TSW_CHANGED+1
	movfp	SW_B_TEMP,WREG
	movwf	TSW_STATE+1
	goto	pb_scan_end1
pbs_3:
	movwf	SW_B_TEMP
	movlw	3
	movwf	SW_COUNT1
	movwf	SW_COUNT2
; second port scan
pb_scan_end1:
	movfp	SW_C,WREG
	cpfseq	TSW_STATE+2
	goto	pbs_01
	goto	pb_scan_next1
pbs_01:
	cpfseq	SW_C_TEMP
	goto	pbs_11
	decfsz	SW_COUNT11,F
	goto	pb_scan_next1
	xorwf	TSW_STATE+2,W
	movwf	TSW_CHANGED+2
	movfp	SW_C_TEMP,WREG
	movwf	TSW_STATE+2
	goto	pb_scan_next1
pbs_11:
	movwf	SW_C_TEMP
	movlw	4
	movwf	SW_COUNT11
	movwf	SW_COUNT21
pb_scan_next1:
	movfp	SW_D,WREG
	cpfseq	TSW_STATE+3
	goto	pbs_21
	goto	pb_scan_end
pbs_21:
	cpfseq	SW_D_TEMP
	goto	pbs_31
	decfsz	SW_COUNT21,F
	goto	pb_scan_end
	xorwf	TSW_STATE+3,W
	movwf	TSW_CHANGED+3
	movfp	SW_D_TEMP,WREG
	movwf	TSW_STATE+3
	goto	pb_scan_end
pbs_31:
	movwf	SW_D_TEMP
	movlw	3
	movwf	SW_COUNT11
	movwf	SW_COUNT21
;
pb_scan_end:
	movlr	BANK0
	return

#else

pb_scan:
	movlr	BANK1			; switch scan data is in RAM page 1
	movlb	BANK7+1
	movfp	PORTJ,WREG
	cpfseq	TSW_STATE
	goto	pbs_0
	goto	pb_scan_next
pbs_0:
	cpfseq	PORTJ_TEMP
	goto	pbs_1
	decfsz	SW_COUNT1,F
	goto	pb_scan_next
	xorwf	TSW_STATE,W
	movwf	TSW_CHANGED
	movfp	PORTJ_TEMP,WREG
	movwf	TSW_STATE
	goto	pb_scan_next
pbs_1:
	movwf	PORTJ_TEMP
	movlw	4
	movwf	SW_COUNT1
	movwf	SW_COUNT2
pb_scan_next:
	movfp	PORTH,WREG
	cpfseq	TSW_STATE+1
	goto	pbs_2
	goto	pb_scan_end
pbs_2:
	cpfseq	PORTH_TEMP
	goto	pbs_3
	decfsz	SW_COUNT2,F
	goto	pb_scan_end
	xorwf	TSW_STATE+1,W
	movwf	TSW_CHANGED+1
	movfp	PORTH_TEMP,WREG
	movwf	TSW_STATE+1
	goto	pb_scan_end
pbs_3:
	movwf	PORTH_TEMP
	movlw	3
	movwf	SW_COUNT1
	movwf	SW_COUNT2
pb_scan_end:
	movlr	BANK0
	return

#endif ; KBD_4X4

;-------------------------------------------------------------------------------		
;
#ifdef KBD_4X4

pb_expand:
	movlr	BANK1
	movfp	TSW_CHANGED,WREG
	movwf	SW_CHANGED
	movfp	TSW_STATE,WREG
	movwf	SW_STATE
	movfp	TSW_CHANGED+1,WREG
	movwf	SW_CHANGED+1
	movfp	TSW_STATE+1,WREG
	movwf	SW_STATE+1
	movfp	TSW_CHANGED+2,WREG
	movwf	SW_CHANGED+2
	movfp	TSW_STATE+2,WREG
	movwf	SW_STATE+2
	movfp	TSW_CHANGED+3,WREG
	movwf	SW_CHANGED+3
	movfp	TSW_STATE+3,WREG
	movwf	SW_STATE+3
	clrf	TSW_CHANGED,F
	clrf	TSW_CHANGED+1,F
	clrf	TSW_CHANGED+2,F
	clrf	TSW_CHANGED+3,F
	movlr	BANK0
	return

#else

pb_expand:
	movlr	BANK1
	clrf	SW_CHANGED,F
	clrf	SW_CHANGED+1,F
	clrf	SW_CHANGED+2,F
	clrf	SW_CHANGED+3,F
	tstfsz	TSW_CHANGED
	goto	do_pb_0
	goto	do_pb_process
do_pb_0:
	tstfsz	TSW_CHANGED+1
	goto	do_pb_1
	goto	do_pb_process
do_pb_1:
	btfss	TSW_CHANGED+1,6
	goto	do_pb_bit5
	movfp	TSW_CHANGED,WREG
	andlw	0x7f
	movwf	SW_CHANGED
	movfp	TSW_STATE,WREG
	andlw	0x7f
	movwf	SW_STATE
	goto	do_pb_process
do_pb_bit5:
	btfss	TSW_CHANGED+1,5
	goto	do_pb_bit7
	rrcf	TSW_CHANGED,W
	btfsc	ALUSTA,C
	bsf	SW_CHANGED,7
	andlw	0x3f
	movwf	SW_CHANGED+1
	rrcf	TSW_STATE,W
	btfsc	ALUSTA,C
	bsf	SW_STATE,7
	andlw	0x3f
	movwf	SW_STATE+1
	goto	do_pb_process
do_pb_bit7:
	btfss	TSW_CHANGED+1,7
	goto	do_pb_process
	rrcf	TSW_CHANGED,W
	btfsc	ALUSTA,C
	bsf	SW_CHANGED+1,6
	rrcf	WREG,F
	btfsc	ALUSTA,C
	bsf	SW_CHANGED+1,7
	andlw	0x1f
	movwf	SW_CHANGED+2
	rrcf	TSW_STATE,W
	btfsc	ALUSTA,C
	bsf	SW_STATE+1,6
	rrcf	WREG,F
	btfsc	ALUSTA,C
	bsf	SW_STATE+1,7
	andlw	0x1f
	movwf	SW_STATE+2
do_pb_process:
	clrf	TSW_CHANGED,F
	clrf	TSW_CHANGED+1,F
	movlr	BANK0
	return

#endif
;
; end of button scan
;
;-------------------------------------------------------------------------------		
;
; now execute each button that has changed.
; There are 32 buttons.

do_pb:
	clrf	PB_ID,F			; start at button ID 0
	movlw	4			; 4 words of button data
	movwf	PB_INDCOUNT		; save byte index
	movlw	LOW SW_CHANGED		; button changed data
	movwf	PB_INDEXCH		; save as index
	movlw	LOW SW_STATE		; switch state data
	movwf	PB_INDEXST		; save starting index
pb_next_byte:
	movlw	8			; 8 bits per byte
	movwf	PB_BITCOUNT
pb_next_bit:
	movfp	PB_INDEXST,FSR1		; get state index
	movlw	0			; start from 0 and build data below
	bcf	ALUSTA,C		; clear for rotate below
	movlr	BANK1			; state data is in bank 1
	rrcf	INDF1,F			; get next bit in carry
	movlr	BANK0			; back to main page
	btfsc	ALUSTA,C		; was state 1 ( up ) or 2 ( down )?
	movlw	1			; replace 0 by 1 if button up
	iorlw	2			; set count to 1 event
	movwf	PB_STATE		; and save button state
; having set pb_state to reflect switch, now set bit 6 if we were
; waiting for switch programming ( i.e. set bit if PB_PROG_FLAGS is
; neither zero nor 255 (well, 128 to 255 !) ). 
; Do this before we initialise PB_PROG_FLAGS below !
	tstfsz	PB_PROG_FLAGS
	bsf	PB_STATE,6
	btfsc	PB_PROG_FLAGS,7
	bcf	PB_STATE,6
;
	movfp	PB_INDEXCH,FSR1		; now need to find if changed
	movlr	BANK1			; from data in bank 1
	movpf	INDF1,WREG		; get byte
	movlr	BANK0
	btfsc	ALUSTA,Z		; if no bits set, skip rest of byte
	goto	pb_no_byte_change
	bcf	ALUSTA,C
	movlr	BANK1
	rrcf	INDF1,F			; get next bit in C
	movlr	BANK0
	btfss	ALUSTA,C
	goto	pb_no_bit_change	; and skip if clear
; ok, a switch has changed state. If depressed, set second counter to 6 else clear it
	clrf	I,F
	btfsc	PB_PROG_FLAGS,7
	incf	I,F
	movlw	0
	btfss	PB_STATE,0
	movlw	PB_PROG_DELAY
	movwf	PB_PROG_FLAGS
	movfp	PB_ID,WREG
	movwf	PB_PROG_ID
	btfsc	I,0
	goto	pb_prog_end
;
	clrf	Idle_Count,F		; mark change to UI
	movfp	PB_ID,WREG		; get button number
	addlw	PBindirect		; index into pb table
	movwf	FSR1			; read indexed into pb table
	movfp	INDF1,WREG		; to find function associated with it
	call	exec_pb_func		; execute push button

pb_prog_end:
pb_no_bit_change:
	incf	PB_ID,F			; increment button id
	decf	PB_BITCOUNT,F		; decrement bit count
	btfss	ALUSTA,Z		; at zero ( of 8 bits ) ?
	goto	pb_next_bit		; if not, do more bits
	goto	pb_new_byte		; else do next byte
pb_no_byte_change:
	movfp	PB_BITCOUNT,WREG	; rest of byte was unchanged.
	addwf	PB_ID,F			; so add remaining count in
pb_new_byte:
	incf	PB_INDEXCH,F		; bump changed byte index
	incf	PB_INDEXST,F		; bump status byte index
	decfsz	PB_INDCOUNT,F		; reached the end ?
	goto	pb_next_byte		; no - loop
	return				; yes - return

; --------------------------- label push buttons ----------------------
; if W is 0xff, redraw all button labels
; else if W <= max number of buttons, label it
; else do nothing
; if button >= NUM_BUTTONS, then label at fdisp location

pb_label:
	movwf	K		; save button number / -1 in K
	incf	K,F		; check if equal to ff
	btfss	ALUSTA,Z	; if so, label all DISPLAY buttons
	goto	pb_label_chk	; if not, just label this one
#if NUM_BUTTONS > 0		; no buttons to label at all ?
	movlw	NUM_BUTTONS	; if there are start at max
	movwf	K		; keep index for loop
pb_label_loop:
	decf	K,W		; get next button ID
	call	pb_label_one	; label it
	decfsz	K,F		; decrement count
	goto	pb_label_loop	; and loop
#endif
	return			; all ( or nothing ) done

; entry point from above interface only.
; label a pushbutton. If button ID was FF it was trapped above
; as a command to label all, otherwise button ID is in W.
; If the value is > max button in existance, abort
; otherwise label it. Note second check on ID. If ID is
; greater than number of displayed buttons ( as opposed to
; total of buttons ) then label it at the fdisp location

pb_label_chk:
	sublw	PBindirectEnd - PBindirect	; check if > max that exist
	btfss	ALUSTA,C
	return					; if so retuen
	sublw	PBindirectEnd - PBindirect	; yes, I know, inefficient
	sublw	NUM_BUTTONS-1			; but easier to understand !
#if NUM_BUTTONS > 0
	btfss	ALUSTA,C			; if > display buttons, label at fdisp
#endif
	goto	pb_label_at_fdisp
	sublw	NUM_BUTTONS-1			; reinstate original value

; common point of entry from pb_label_loop, above code 
; or pb_label_at_fdisp ONLY !!!
; set X,Y coordinate on screen by table lookup on button ID
; find button function by button table in PBindirect
; locate corresponding text string
; display byte at a time until null

pb_label_one:
	movwf	DISP_BUF		; save button ID in DISP_BUF
	movlw	HIGH pb_lab_offset	; table loopup in
	movwf	TBLPTRH			; pb_lab_offset to get
	movfp	DISP_BUF,WREG		; display XY coordinates
	addlw	LOW pb_lab_offset	; in I & J
	movwf	TBLPTRL
	btfsc	ALUSTA,C
	incf	TBLPTRH,F
pb_label_one_0:
	TABLRD 	0, 1, I 		; Dummy read, get hibyte
	TLRD 	1, I			; store hibyte
	TABLRD 	0, 1, J			; store lobyte & increment
	call	LCDGOTOXY		; set cursor location
;
	movfp	DISP_BUF,WREG		; retrieve button ID
	addlw	PBindirect		; index into button table for
	movwf	FSR0			; function, and read it
	movpf	INDF0,DISP_BUF		; replacing button ID
;
	movlw	HIGH pbl_index		; table lookup on button
	movwf	TBLPTRH			; function names
	movfp	DISP_BUF,WREG		; to get string pointer
	addlw	LOW pbl_index		; in table pointer reg
	movwf	TBLPTRL
	btfsc	ALUSTA,C
	incf	TBLPTRH,F
	clrf	DISP_BUF,F		; clear disp_buf string
	call	p_string_indirect_at_cursor
	return

; local label from above. DIsplay function label not
; at designated position ( as it does not have one )
; but instead at the fdisp location

pb_label_at_fdisp:
	sublw	NUM_BUTTONS-1		; reinstate button ID
	movwf	DISP_BUF		; save it for later
	movlw	HIGH str_fdisptxt	; point directly to XY
	movwf	TBLPTRH			; coord of fdisp
	movlw	LOW str_fdisptxt
	movwf	TBLPTRL
	goto	pb_label_one_0		; and re-enter main code


