;-------------------------------------------------------------------------------
; NAME: pbfuncs.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the handler for push button functions.
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;               10/5/00		G8KBB	Ported to 17C766
;
;-------------------------------------------------------------------------------		
;
;	W contains function number
;	FSR1 points to button function indirection value from which W was read
;
;	note there must be EXACTLY as many entries in pb_func_index as there are
;	table entries in pblabels.asm in pbl_index.

exec_pb_func:
	movpf	WREG,I			; save button number for later
	addlw	LOW pb_func_index	; need to point to index table....
	movwf	TBLPTRL
	movlw	HIGH pb_func_index
	clrf	TBLPTRH,F
	addwfc	TBLPTRH,F		; ....now pointing to function address
	TABLRD 	0, 1, WREG 		; Dummy read, get hibyte
	movfp	I,WREG			; reinstate button number into W
	TLRD 	1, PCLATH		; store hibyte
	TLRD 	0, PCL			; store lobyte & jump to function

pb_func_index:
	dw	nullfn	; 00
	dw	bandup	; 01 
	dw	banddn	; 02 
pb160m	dw	f160m	; 03 ; fxxxm must be in sequence
	dw	f80m	; 04 
	dw	f40m	; 05 
	dw	f30m	; 06 
	dw	f20m	; 07
	dw	f17m	; 08
	dw	f15m	; 09 
	dw	f12m	; 0a 
	dw	f10m	; 0b 
;	dw	f6m	; 0c 
;	dw	f4m	; 0d 
;	dw	f2m	; 0e 
lbc:	dw	band_change	; 0f
	dw	accel	; 10
	dw	mode	; 11 
	dw	atten	; 12 
	dw	tuneup	; 13 
; following block are also opto 2 functions ( to pb_opto2_last )
pb_opto2_first:
pbvol:	dw	volume	; 14 
; following block are config functions ( to pbcfge )
pbcfg:	dw	config	; 15 - this is mic gain, config must be in sequence
	dw	config	; 16 - and the sequence must match order of variables
	dw	config	; 17 - in RAM !
	dw	config	; 18 
	dw	config	; 19 
	dw	config	; 1a 
	dw	config	; 1b ( rf power )
pbcfge:
pb_opto2_last:
	dw	opto2_mode ; 1c
	dw	vco_ab	; 1d
	dw	vco_split	; 1e
	dw	rd_mem	; 1f
	dw	rd_mem1	; 20
	dw	rd_mem2	; 21
	dw	rd_mem3	; 22
	dw	rd_mem4	; 23
	dw	wr_mem	; 24
	dw	wr_mem1	; 25
	dw	wr_mem2	; 26
	dw	wr_mem3	; 27
	dw	wr_mem4	; 28
	dw	f_lock	; 29
	dw	mode_cw	; 2a
	dw	mode_usb ; 2b
	dw	mode_lsb ; 2c
pbselc:	dw	select_cfg ; 2d	
pbsetc:	dw	set_cfg ; 2e
	dw	cw_filter ; 2f
	dw	sw_ver	; 30
#ifdef BACKLIGHT_CONTROL
	dw	backlight ; 31
#endif
	dw	vco_a_eq_b
	dw	menu_0
	dw	menu_1
	dw	menu_2
	dw	menu_3
	dw	vox_ctrl
#ifdef INCLUDE_KEYER
	dw	keyer_onoff
	dw	keyer_type
pbkeyspeed:
	dw	keyer_speed
#endif
	dw	disable_xmit
	dw	pll_check
#ifdef COMMON_DSP_FUNCTIONS
	dw	dsp_on
	dw	dsp_agc
	dw	dsp_filter
	dw	dsp_noise
#endif
#ifdef DSP_DDS
	dw	dsp_rx_lms
	dw	dsp_notch
	dw	dsp_tx_agc
	dw	dsp_2tone_tx
	dw	dsp_txproc
#endif
#ifdef RELAY_BOARD
	dw	indep_tx
	dw	indep_rx
#endif
	dw	dis_vox_startup
#ifdef BANDSCAN
pbscan:
	dw	bandscan_button
#endif

pb_func_index_end:

; ------------------------------------------------------------------
; index table for config parameters. Each entry holds flag for
; update control followed by max value for that control
;
config_index:
	db	1 << VOL_OR_MIC_CH, D'63'	; mic gain
	db	1 << IF_CHANGED, D'255'		; if gain
	db	1 << IF_CHANGED, D'255'		; hang time
	db	1 << VOX_CHANGED, D'63'		; vox gain 
	db	1 << VOX_CHANGED, D'63'		; anti vox 
	db	1 << VOX_CHANGED, D'255'	; vox delay
	db	1 << RF_CHANGE, D'255'		; RF Power

; ------------------------------------------------------------------
; button functions, indexed from control table above
; ------------------------------------------------------------------
; in all functions, the following hold true on entry:
;	PB_STATE defines operation as follows
;		bit 7 if set means originated from opto not button
;		bit 6 if set means PB_STATE was neither 0 nor 255
;		      i.e. we were awaiting switch programming
;		bit 5-1 are count of events in this call
;		bit 0 if set means button down, else up
;	W contains function number
;	FSR points to button function indirection value from which W was read
;
; ------------------------------------------------------------------
; Band change related functions

bandup:
	btfsc	PB_STATE,7	; check for opto change
	goto	band_up_down
	btfss	PB_STATE,0
nullfn:	return
band_up1:
	incf	BAND,F
	movlw	NUM_BANDS
	subwf	BAND,W
	btfsc	ALUSTA,C
	clrf	BAND,F
	goto	banddn0
;
band_change:
	btfsc	PB_STATE,7
	goto	band_up_down
	movlw	lbc-pb_func_index
	btfsc	PB_STATE,0
	movlw	0
	movwf	OPTO_1_MODE
	clrf	OPTO_1_COUNT,F
	return
band_up_down:
	btfss	PB_STATE,0
	goto 	band_up1
	goto	band_down1
;
banddn:
	btfsc	PB_STATE,7	; check for opto change
	goto	band_up_down
	btfss	PB_STATE,0
	return
band_down1:
	decf	BAND,F
	incf	BAND,W
	btfss	ALUSTA,Z
	goto	banddn0
	movlw	NUM_BANDS-1
fxxx:	movwf	BAND
banddn0:
	bsf	UPDATE_1,BAND_CHANGE
	return
;

f160m:
f80m:
f40m:
f30m:
f20m:
f17m:
f15m:
f12m:
f10m:
;f6m:
;f4m:
;f2m:
	addlw	-(pb160m - pb_func_index )
	goto	fxxx

; ------------------------------------------------------------------
; change acceleration flag

accel:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfsc	PB_STATE,0		; if button down, ignore
	btg	UPDATE_2, FREQ_ACCEL	; invert it
	return

; ------------------------------------------------------------------
; functions that change mode ( cw / lsb / cw )
 
mode:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlw	8			; increment mode
	addwf	ModeFlags,F		; note values 0..2 only
	movfp	ModeFlags,WREG		; so wrap 3 to 0
	andlw	0x18			; this is a trick !
	xorlw	0x18
	btfss	ALUSTA,Z
	goto	mode_ok
	movlw	~0x18			; overflow - set to 0
	andwf	ModeFlags,F
mode_ok:
	bsf	UPDATE_2,MODE_CHANGE
	movlw	0x18			; check for ssb
	andwf	ModeFlags,W		; by selecting the 2 bits
	btfsc	ALUSTA,Z		; and checking for zero ( cw )
	goto	mode_ok_exit		; skip if CW
	bsf	UPDATE_2,FILTER_CHANGE	; mark change
	bcf	RXFlags2, RXF2_CWFilter	; force wide
mode_ok_exit:
	bsf	UPDATE_1,DDS1_CHANGED	; redisplay frequency
	return

mode_cw:
	movlw	0			; cw, bits 3,4 clear
	goto	mode_set
mode_usb:
	movlw	8			; usb, bit 3 set; 4 clear
	goto	mode_set
mode_lsb:
	movlw	0x10			; lsb bit 3 clear; 4 set
mode_set:
	bcf	ModeFlags,3		; or new data into current
	bcf	ModeFlags,4		; so must clear bits first
	iorwf	ModeFlags,F
	goto	mode_ok

; ------------------------------------------------------------------
; set volume

volume:
	btfsc	PB_STATE,7		; if opto event,
	goto	vol_up_down		; jump to service it
	movlw	pbvol-pb_func_index	; get value of volume function
	btfsc	PB_STATE,0		; or if button went up
	movlw	0			; get null function ( 0 )
	movwf	OPTO_1_MODE		; and set that as opto 1 mode
	clrf	OPTO_1_COUNT,F		; clear any residual opto count
;	btfsc	PB_STATE,0		; finally, if button up
;	bsf	EEPROM_UPD_1,VOL_OR_MIC_CH	; mark change to volume
	return
vol_up_down:
	bsf	UPDATE_1,VOL_OR_MIC_CH	; mark change to volume
	rrncf	PB_STATE,W		; get amount to change by
	andlw	0x1f
	movwf	I			; and save for now
vol_up_down1:
	btfss	PB_STATE,0		; goimg down ?
	goto 	vol_up1			; nope - going up
; vol down
	decf	Volume,F		; decrement volume
	incf	Volume,W		; and check if < 0
	btfsc	ALUSTA,Z
	clrf	Volume,F		; if so, clear it
	goto	vol_up_down_end
vol_up1:
	incf	Volume,F		; increment volume
	btfsc	Volume,6		; check overflow ( > 63 )
	decf	Volume,F		; and decrement if so
vol_up_down_end:
	decf	I,F			; decrement count
	btfss	ALUSTA,Z
	goto	vol_up_down1		; and loop ( yes, it is inefficient )
	return

; ------------------------------------------------------------------
; change attenuator settings

atten:
	btfsc	PB_STATE,7		; ignore opto events
	return
	btfss	PB_STATE,0		; ignore button down
	return
	incf	RXFlags,W		; change to next atten setting
	andlw	3			; truncate overflow
	bcf	RXFlags,0		; zero bits so we can exor in
	bcf	RXFlags,1
	iorwf	RXFlags,F
	bsf	UPDATE_1,RF_CHANGE	; mark change
	return

; ------------------------------------------------------------------
; tx / rx related functions - currently just tuneup

tuneup:
	btfsc	PB_STATE,7		; ignore opto
	return
	btfss	PB_STATE,0		; ignore button down
	return
;	bsf	UPDATE_2, XMIT_CHANGED	; mark change
	btg	RXFlags, RXF_Tuneup	; toggle tuneup
	bcf	RXFlags2, RXF2_TUNEUP_TX
	btfsc	RXFlags, RXF_Tuneup
	bsf	RXFlags2, RXF2_TUNEUP_TX ; set xmit to same as tuneup
	return

; ------------------------------------------------------------------
; change settings of current config ( e.g. volume, power, vox etc )

config:
	btfsc	PB_STATE,7		; if source was opto not button
	goto	config_change		; then go increment / decrement
	btfsc	PB_STATE,0		; if button down, function is in W
	movlw	0			; else clear function
	movwf	OPTO_1_MODE		; and set opto 1 mode to this
	clrf	OPTO_1_COUNT,F		; get rid of any residual opto count
	btfss	PB_STATE,0		; if button down, that's all
	return				; so exit
	btfss	PB_STATE,6		; otherwise we need to step the button 
	return				; function if PB_STATE 6 was non zero
;	movf	PB_ID,W			; get button number
;	addlw	PBindirect		; index into pb table
;	movwf	FSR			; read indexed into pb table
; the above code exists in pb function that called config - FSR already set.
;
; increment function & wrap if needed
	incf	INDF1,F
	movlw	pbcfge - pb_func_index
	subwf	INDF1,W
	movlw	pbcfg - pb_func_index
	btfsc	ALUSTA,Z
	movwf	INDF1
;config_upd:
	movfp	INDF1,WREG
	addlw	MicGain -(pbcfg - pb_func_index)
	movwf	FSR1
	call	cfg_display
	movfp	PB_ID,WREG
	goto	pb_label
; return implicit in goto pb_label instead of return !

config_change:
	addlw	MicGain -(pbcfg - pb_func_index)
	movwf	FSR1
	addlw	-MicGain	; get index ( 0..n )
	addlw	LOW config_index
	movfp	WREG,TBLPTRL
	movlw	HIGH config_index
	btfsc	ALUSTA,C
	incf	WREG,F
	movfp	WREG,TBLPTRH
	TABLRD 	0, 1, WREG 	; Dummy read, get hibyte
	TLRD 	1, WREG		; get flags
	iorwf	UPDATE_1,F
	TLRD 	0, J		; set max value for this control

; wrong place !!	clrf	OPTO_1_COUNT
	rrncf	PB_STATE,W
	andlw	0x1f
	movwf	I
cfg_change1:
	btfss	PB_STATE,0
	goto 	cfg_up1
; cfg down
	decf	INDF1,F
	incf	INDF1,W
	btfsc	ALUSTA,Z
	clrf	INDF1,F
	goto	cfg_change_end
cfg_up1:
	movfp	J,WREG
	subwf	INDF1,W
	btfss	ALUSTA,Z
	incf	INDF1,F
cfg_change_end:
	decf	I,F
	btfss	ALUSTA,Z
	goto	cfg_change1
cfg_display:
;debug this please
	movpf	FSR1,I			; save FSR1 from corruption
	movfp	INDF1,WREG
	call	pr_to_buf		; assume no change to fsr !
	movfp	I,FSR1			; reinstate FSR1
	movfp	FSR1,WREG		; fsr had address of variable (above)
	addlw	s_configs - string_index - MicGain
	goto	p_string_xy
;	return
; return was implicit in goto

opto2_mode:
	btfsc	PB_STATE,7
	return
	btfss	PB_STATE,0
	return
	bsf	UPDATE_1,MISC_CHANGE		; flag change
	movfp	OPTO_2_MODE,WREG		; read current mode
	addlw	0
	btfsc	ALUSTA,Z			; if zero
	movlw	pb_opto2_first - pb_func_index -1	; set to first-1
	addlw	1				; increment
	movwf	OPTO_2_MODE			; store new mode
	movlw	pb_opto2_last - pb_func_index	; check overflow
	subwf	OPTO_2_MODE,W
	btfsc	ALUSTA,Z
	clrf	OPTO_2_MODE,F			; and go to null if so
	movlw	HIGH pbl_index			; now index into pb labels
	movwf	TBLPTRH
	movfp	OPTO_2_MODE,WREG		; by opto2mode
	addlw	LOW pbl_index
	movwf	TBLPTRL
	btfsc	ALUSTA,C
	incf	TBLPTRH,F
	TABLRD 	0, 1, WREG 		; Dummy read, get hibyte
	TLRD 	1, TBLPTRH		; read address of pb label text
	TLRD 	0, TBLPTRL		; and store in tablepointer
	movlw	DISP_BUF		; point to display buffer
	movwf	FSR0			; with fsr0
	TABLRD 	0, 1, WREG 		; Dummy read, get hibyte
opto2_mode_loop:			; loop, reading string to DISP_BUF
	TLRD 	1, WREG
	movpf	WREG, INDF0		; check for zero
	btfsc	ALUSTA,Z
	goto	opto2_mode_loop_end
	TABLRD 	0, 1, WREG
	movpf	WREG, INDF0		; check for zero
	btfss	ALUSTA,Z
	goto	opto2_mode_loop
opto2_mode_loop_end:			; we have pb label text in DISP_BUF
	movlw   s_opto2 - string_index	; so print out "Opto2 " followed by
	goto	p_string_xy		; the label in DISP_BUF
	; return implicit in goto

; ------------------------------------------------------------------
; frequency related operations
; VCO A/B, A=B, split & tuning lock

vco_ab:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfsc	PB_STATE,0		; if button down, ignore
	return
	btg	RXFlags, RXF_RX_B	; invert it
	goto	vco_split_0
vco_split:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfsc	PB_STATE,0		; if button down, ignore
	return
vco_split_0:
	btg	RXFlags, RXF_TX_B	; invert it
	bsf	UPDATE_1,DDS1_CHANGED	; set update_fdisp_flag
	movlw	DDS1A_3
	btfsc	RXFlags,RXF_RX_B
	movlw	DDS1B_3
	movwf	FSR1
	call	do_opto_1_checkminimum	; set dds and enable ints
	goto	do_other_display_if_needed
;;	return implicit in goto

f_lock:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfsc	PB_STATE,0		; if button down, ignore
	return
	btg	RXFlags, RXF_Lock		; invert it
	movlw	s_nolock-string_index
	btfsc	RXFlags, RXF_Lock
	movlw	s_lock-string_index
	goto	p_string_xy_nobuf
;;	return implicit in goto

;  VCO A=B
;
vco_a_eq_b:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlw	DDS1A_0		; move from DDS1?_0..3 to AARGB0..3
	btfsc	RXFlags,RXF_RX_B
	movlw	DDS1B_0
	movwf	FSR0
	btfsc	RXFlags,RXF_RX_B
	goto	b_to_a
	movpf	INDF0,DDS1B_0
	movpf	INDF0,DDS1B_1
	movpf	INDF0,DDS1B_2
	movpf	INDF0,DDS1B_3
	goto	do_other_display
;;	return				; return implicit in goto
b_to_a:
	movpf	INDF0,DDS1A_0
	movpf	INDF0,DDS1A_1
	movpf	INDF0,DDS1A_2
	movpf	INDF0,DDS1A_3
	goto	do_other_display
;;	return				; return implicit in goto


; -------------------------------------------------------------------
; memory related functions - read and write

rd_mem:
rd_mem1:
	movlw	0
	goto	mem_read
rd_mem2:
	movlw	4
	goto	mem_read
rd_mem3:
	movlw	8
	goto	mem_read
rd_mem4:
	movlw	0x0c
mem_read:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfsc	PB_STATE,0		; if button down, ignore
	return
	movwf	I
	swapf	BAND,W
	andlw	0xf0
	iorwf	I,F
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	return
	movlw	I2C_EEPROM | 2
	movwf	INDF0		; eeprom address of memories page
	movlr	BANK0
	movfp	I,WREG
	movlr	BANK2
	movwf	INDF0		; ram address of ( band << 4 ) + memory
	movlw	4
	movwf	INDF0		; read 4 bytes
	clrf	INDF0,F		; BSR = 0
	movlr	BANK0
	movlw	DDS1A_0
	btfsc	RXFlags,RXF_RX_B
	movlw	DDS1B_0
	movlr	BANK2
	movwf	INDF0		; destination
	call	read_eeprom_and_wait
	movlw	DDS1A_3
	btfsc	RXFlags,RXF_RX_B
	movlw	DDS1B_3
	movwf	FSR1
	goto	do_opto_1_checkminimum	; set dds and enable ints
;;	return implicit in goto

; -------------------------------------------------------------------

wr_mem:
wr_mem1:
	movlw	0
	goto	mem_write
wr_mem2:
	movlw	4
	goto	mem_write
wr_mem3:
	movlw	8
	goto	mem_write
wr_mem4:
	movlw	0x0c
mem_write:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfsc	PB_STATE,0		; if button down, ignore
	return
	movwf	I
	swapf	BAND,W
	andlw	0xf0
	iorwf	I,F
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	return
	movlw	I2C_EEPROM | 2
	movwf	INDF0		; eeprom address of memories page
	movlr	BANK0
	movfp	I,WREG
	movlr	BANK2
	movwf	INDF0		; eeprom address of ( band << 4 ) + memory
	movlw	4
	movwf	INDF0		; write 4 bytes
	clrf	INDF0,F		; BSR = 0
	movlr	BANK0
	movlw	DDS1A_0
	btfsc	RXFlags,RXF_RX_B
	movlw	DDS1B_0
	movlr	BANK2
	movwf	INDF0		; source = vco
	movlw	B'00111110'	; 3 retries, auto clear, retry, sub address, write
	movwf	INDF0
	goto	I2C_Startup
;;	return implicit in goto

; ------------------------------------------------------------------
; these next two functions deal with the setting
; of configuration data 
;
; The functions use ram data value CUR_CFG to remember function
;
; select configuration.
;   On button down, display currently selected function and value
;   On button up, do nothing
;   On opto event, change current function
;
select_cfg:
	btfsc	PB_STATE,7		; if opto event, change function
	goto	select_cfg_change
	movlw	pbselc-pb_func_index	; get value of volume function
	btfsc	PB_STATE,0		; or if button went up
	movlw	0			; get null function ( 0 )
	movwf	OPTO_1_MODE		; and set that as opto 1 mode
	clrf	OPTO_1_COUNT,F		; clear any residual opto count
	btfsc	PB_STATE,0		; if button down, redisplay
	goto	sel_redisplay
	return				; button up

; set configutation
;   On button down, display current selected function and value
;   On button up, write as required to eeprom
;   On opto event, change value of current function

set_cfg:
	btfsc	PB_STATE,7		; if opto event, change value
	goto	do_cfg_change
	movlw	pbsetc-pb_func_index	; get value of setcfg function
	btfsc	PB_STATE,0		; or if button went up
	movlw	0			; get null function ( 0 )
	movwf	OPTO_1_MODE		; and set that as opto 1 mode
	clrf	OPTO_1_COUNT,F		; clear any residual opto count
	btfss	PB_STATE,0		; if button down, redisplay
	goto	sel_redisplay
	movlw	config_data_entries	; button up. check index...
	cpfslt	CUR_CFG
	clrf	CUR_CFG,F		; zero if error
	call	get_cfg_data		; get update flags in K
	movfp	K,WREG
	andlw	0x07f			; only 7 ls bits !
	iorwf	EEPROM_UPD_3,F		; set flag for eeprom update
	return

sel_redisplay:
	movlw	config_data_entries	; check for overflow
	cpfslt	CUR_CFG
	clrf	CUR_CFG,F		; zero if so
	LOAD_XCOORD CONFIG_LOCN
	movwf	I
	LOAD_YCOORD CONFIG_LOCN
	movwf	J
	call	LCDGOTOXY
;
; now form display data as call to p_string_at_cursor
; with TBLPTR pointing to rom string and value
; in dispbuf
;
; first get CUR_CFG and index into config data table
; move that address into tblptr
; point to data byte
; convert to hex and put in dispbuf
; save update type in K
; leaving tblptr pointing to string
; and go for it !
;
	call	get_cfg_data
	movlw	' '
	movwf	DISP_BUF
	movwf	DISP_BUF+1
	movwf	DISP_BUF+2
	movwf	DISP_BUF+3
	clrf	DISP_BUF+4,F	; clear buffer just in case special case
	btfsc	K,7		; trap special cases
	goto	sel_redisplay0	; and if so skip value display
;	movlw	' '		; this was done above
;	movwf	DISP_BUF	; so no need to do here
	movlw	'9'+1		; set temp value for ascii convert
	movwf	I
	movfp	J,BSR
	swapf	INDF1,W
	clrf	BSR,F
	andlw	0x0f		; get nibble
	addlw	'0'		; convert to 0..??
	cpfsgt	I		; check overflow
	addlw	'A'-'0'-0x0a	; convert A..F
	movwf	DISP_BUF+1
	movfp	J,BSR
	movfp	INDF1,WREG
	clrf	BSR,F
	andlw	0x0f		; get nibble
	addlw	'0'		; convert to 0..??
	cpfsgt	I		; check overflow
	addlw	'A'-'0'-0x0a	; convert A..F
	movwf	DISP_BUF+2
;	movlw	' '		; these were done above
;	movwf	DISP_BUF+3	; so
;	clrf	DISP_BUF+4,F	; no need to do here
sel_redisplay0:
	goto	p_string_at_cursor
;;;	return			; return implicit in goto

get_cfg_data:
	movfp	CUR_CFG,WREG
	mullw	size_config_data_entry
	movlw	LOW config_data
	addwf	PRODL,W
	movwf	TBLPTRL
	movlw	HIGH config_data
	addwfc	PRODH,W
	movwf	TBLPTRH

	TABLRD 	0, 1, WREG 	; Dummy read, get hibyte
	TLRD 	1, FSR1		; store Address in ptr
	TABLRD	0,1,J		; read BSR for Address ptr
;;	TLRD	1,xxx		; spare byte in table - not used
	TLRD 	0, K		; store data type in K ( TEMP )
	return

select_cfg_change:
	btfss	PB_STATE,0
	goto 	cfg_next_up
	tstfsz	CUR_CFG
	decf	CUR_CFG,F
	goto	sel_redisplay
cfg_next_up:
	incf	CUR_CFG,F
	movlw	config_data_entries-1
	cpfsgt	CUR_CFG
	goto	sel_redisplay
	decf	CUR_CFG,F
	goto	sel_redisplay

do_cfg_change:
	movlw	config_data_entries	; check for overflow
	cpfslt	CUR_CFG
	clrf	CUR_CFG,F		; zero if so
	call	get_cfg_data
	btfsc	K,7			; trap special cases
	goto	reset_cfg
	btfss	PB_STATE,0
	goto	cfg_up
	movfp	J,BSR
	decf	INDF1,F
	goto	cfg_up_end
cfg_up:
	movfp	J,BSR
	incf	INDF1,F
cfg_up_end:
	clrf	BSR,F
	goto	sel_redisplay

reset_cfg:
	tstfsz	CUR_CFG			; check for case 0
	goto	reset_cfg_0		; if not skip
	goto	BandTableInitFromRom	; if so, reset band data
reset_cfg_0:
	dcfsnz	CUR_CFG,W
	goto	reset_dds_clock_data
	return

; ------------------------------------------------------------------
; cw filter control.
; when button is released, toggle cw rx filter wide / narrow
; by toggling the power control bit
;
cw_filter:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	bsf	UPDATE_2,FILTER_CHANGE	; mark change
	btg	RXFlags2, RXF2_CWFilter	; invert it
#ifdef BANDSCAN
	btfsc	RXFlags2, RXF2_BANDSCAN
	call	bandscan_start
#endif
	return

; ------------------------------------------------------------------
; software build details
;
sw_ver:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
;;	clrf	DISP_BUF,F		; print version data
	movlw	s_versionstr-string_index
	goto	p_string_xy_nobuf
;;;	return				; return implicit in goto

; ------------------------------------------------------------------
; backlight control
;
#ifdef BACKLIGHT_CONTROL
backlight:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	bsf	UPDATE_1,MISC_CHANGE
	movfp	UI_Flags,WREG		; **presumes UIF2_Backlight is bit 1,0
	addlw	1			; step to next value
	andlw	3			; modulo 4 - two bits
	bcf	UI_Flags,UIF_Backlight
	bcf	UI_Flags,UIF_Backlight2
	iorwf	UI_Flags,F		; store value
	andlw	3			; get Z bit again ( Z => off )
	btfss	ALUSTA,Z		; if Z clear ( 01, 10 or 11 )
	goto	backlight_on		; switch light on
	;; btfsc	ALUSTA,Z		; if Z set ( 00 )
	goto	backlight_off		; switch light off
	;; return implicit in BOTH gotos above
#endif

; ----------------------------------------------------------------
; menu control functions
;
menu_0:
	movlw	0x00
	goto	menu_set
menu_1:
	movlw	(1 << UIF_Menu_bit_0)
	goto	menu_set
menu_2:
	movlw	(1 << UIF_Menu_bit_1)
	goto	menu_set
menu_3:
	movlw	(1 << UIF_Menu_bit_0) | (1 << UIF_Menu_bit_1)
menu_set:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movwf	PRODH		; save for the moment
	movfp	UI_Flags,WREG	; get byte with UI flags in it then clear menu bits
	andlw	~( (1 << UIF_Menu_bit_0) | (1 << UIF_Menu_bit_1) )
	iorwf	PRODH,W		; now or in the new flags
	movwf	UI_Flags	; store flags
;	goto	menu_set_completion
;menu_01:
;	movfp	UI_Flags,WREG			; menu A->B->C->A etc
;	addlw	(1 << UIF_Menu_bit_0)		; increment
;	andlw	(1 << UIF_Menu_bit_0) | (1 << UIF_Menu_bit_1) ; trash overflow
;	btfss	WREG,UIF_Menu_bit_0		; check if reached B'11'
;	goto	menu_set_exit_0
;	btfsc	WREG,UIF_Menu_bit_1
;	clrf	WREG,F				; and if so, skip to B'00'
;	goto	menu_set_exit_0
;menu_02:
;	movfp	UI_Flags,WREG			; menu A->B->C->D->A etc
;	addlw	(1 << UIF_Menu_bit_0)
;	andlw	(1 << UIF_Menu_bit_0) | (1 << UIF_Menu_bit_1)
;menu_set_exit_0:
;	btfsc	PB_STATE,7		; if opto event, ignore
;	return
;	btfss	PB_STATE,0		; if button down, ignore
;	return
;	bcf	UI_Flags,UIF_Menu_bit_0
;	bcf	UI_Flags,UIF_Menu_bit_1
;	iorwf	UI_Flags,F
;menu_set_completion:
	call	read_pushbutton_data
	movlw	0xff			; parameter 255 means
	goto	pb_label		; label all buttons
	;; return implicit in goto


;
; ---------------------------------------------------------------------
; vox control ( on / off )
;
vox_ctrl:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	btg	UI_Flags,UIF_Vox_off	; toggle vox on / off bit
	bsf	UPDATE_1,MISC_CHANGE
	bsf	UPDATE_1,VOX_CHANGED
vox_ctrl_disp:
	movlw	s_vox_on-string_index
	btfsc	UI_Flags,UIF_Vox_off
	movlw	s_vox_off-string_index
	goto	p_string_xy_nobuf
	; return implicit in goto
;
; ---------------------------------------------------------------------
; keyer control ( on / off ) and keyer type ( iambic / straight )
; and set keyer speed

#ifdef INCLUDE_KEYER

keyer_onoff:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	btg	RXFlags2,RXF2_KEYER_DIS	; toggle keyer on / off bit
	clrf	Keyer_state,F		; force keyer to known state
	bcf	RXFlags2,RXF2_KEYER_TX	; and switch off if on
	return

keyer_type:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	bsf	UPDATE_1,MISC_CHANGE
	btg	UI_Flags, UIF_Keyer_Type
	movlw	s_keyer_iambic-string_index
	btfss	UI_Flags, UIF_Keyer_Type
	movlw	s_keyer_simple-string_index
	goto	p_string_xy_nobuf
;;;	return				; return implicit in goto

;
; change keyer speed
;	
keyer_speed:
	btfsc	PB_STATE,7		; if opto event,
	goto	keyer_up_down		; jump to service it
	movlw	pbkeyspeed-pb_func_index ; get value of keyspeed function
	btfsc	PB_STATE,0		; or if button went up
	movlw	0			; get null function ( 0 )
	movwf	OPTO_1_MODE		; and set that as opto 1 mode
	clrf	OPTO_1_COUNT,F		; clear any residual opto count
	goto	keyer_speed_display	; display current speed

keyer_up_down:
	bsf	EEPROM_UPD_3,KEYER_DATA_UPD	; mark change to keyer dot rate
	rrncf	PB_STATE,W		; get amount to change by
	andlw	0x1f
	movwf	I			; and save for now
keyer_up_down1:
	btfss	PB_STATE,0		; goimg down ?
	goto 	keyer_up1			; nope - going up
; speed down
	decf	Keyer_dot,F		; decrement keyer dot rate
	movlw	2
	cpfsgt	Keyer_dot		; check minimum
	movwf	Keyer_dot		; and force to >= 2
	goto	keyer_up_down_end
keyer_up1:
	incf	Keyer_dot,F		; increment keyer dot rate
	btfsc	Keyer_dot,5		; check overflow ( > 31 )
	decf	Keyer_dot,F		; and decrement if so
keyer_up_down_end:
	decf	I,F			; decrement count
	btfss	ALUSTA,Z
	goto	keyer_up_down1		; and loop ( yes, it is inefficient )
;
; display speed
;
keyer_speed_display:
	clrf	AARGB0,F		; set up dividend of 120
	movlw	D'120'			; in AARGB then divide by
	movwf	AARGB1			; keyer dot speed in BARGB0
	movfp	Keyer_dot,WREG
	movwf	BARGB0
	call	FXD1507U		; wpm is now in AARGB1
	movfp	AARGB1,WREG
	call	pr_to_buf		; wpm is in buffer ( 0..2 )
	movlw	' '			; append 'wpm' to it
	movwf	DISP_BUF+3
	movlw	'W'
	movwf	DISP_BUF+4
	movlw	'P'
	movwf	DISP_BUF+5
	movlw	'M'
	movwf	DISP_BUF+6
	movlw	' '
	movwf	DISP_BUF+7
	clrf	DISP_BUF+8,F
	movlw	s_key_speed-string_index ; and print preceded by 'keyer '
	goto	p_string_xy
;;;	return				; return implicit in goto


#endif


;
; ---------------------------------------------------------------------
; locking functions for transmit 
;
disable_xmit:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	btg	RXFlags2, RXF2_TX_DISABLE
	movlw	s_tx_disabled-string_index
	btfss	RXFlags2, RXF2_TX_DISABLE
	movlw	s_tx_enabled-string_index
	goto	p_string_xy_nobuf
;;;	return				; return implicit in goto

pll_check:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	bsf	UPDATE_1,MISC_CHANGE
	btg	UI_Flags, UIF_PLL_Check
	movlw	s_pll_lock-string_index
	btfss	UI_Flags, UIF_PLL_Check
	movlw	s_no_pll_lock-string_index
	goto	p_string_xy_nobuf
;;;	return				; return implicit in goto

; ---------------------------------------------------------------------
; DSP control functions

#ifdef COMMON_DSP_FUNCTIONS
; dsp on off
dsp_on:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_CTRL, DSP_CTRL_DSP_ON
dsp_change:
	movlr	BANK0
	bsf	UPDATE_2,DSP_CHANGED
	goto	dsp_display
;;;	return				; return implicit in goto

; dsp agc on off
dsp_agc:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_CTRL, DSP_CTRL_AGC_OFF
	goto	dsp_change

; dsp filter type ( bpf / hplp )
dsp_filter:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_CTRL, DSP_CTRL_DSP_BPF
	goto	dsp_change

; dsp noise reduction type
dsp_noise:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_CTRL, DSP_CTRL_DSP_NRCW
	goto	dsp_change

#ifdef DSP_DDS

dsp_rx_lms:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_CTRL, DSP_CTRL_LMS_ON
	goto	dsp_change

dsp_notch:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_CTRL, DSP_CTRL_NOTCH
	goto	dsp_change

dsp_tx_agc:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_TX_CTRL, DSP_TX_CTRL_AGC_ON
	goto	dsp_change

dsp_txproc:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_TX_CTRL, DSP_TX_CTRL_PROC_ON
	goto	dsp_change


dsp_2tone_tx:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlr	BANK1
	btg	DSP_TX_CTRL, DSP_TX_CTRL_2TONE
	goto	dsp_change

#endif ; DSP_DDS

dsp_display:
	movlr	BANK1
	movlw	s_dsp_off-string_index
	btfss	DSP_CTRL, DSP_CTRL_DSP_ON
	goto	dsp_display_clear
	movlw	s_dsp_on-string_index
	movlr	BANK0
	call	p_string_xy_nobuf
	movlr	BANK1
	movlw	s_dsp_agc_on-string_index
	btfsc	DSP_CTRL, DSP_CTRL_AGC_OFF
	movlw	s_dsp_agc_off-string_index
	movlr	BANK0
	call	p_string_xy_nobuf
	movlr	BANK1
	movlw	s_dsp_bpf-string_index
	btfsc	DSP_CTRL, DSP_CTRL_DSP_BPF
	movlw	s_dsp_hplp-string_index
	movlr	BANK0
	call	p_string_xy_nobuf
	movlr	BANK1
	movlw	s_dsp_nrssb-string_index
	btfsc	DSP_CTRL, DSP_CTRL_DSP_NRCW
	movlw	s_dsp_nrcw-string_index
dsp_display_clear:
	movlr	BANK0
	call	p_string_xy_nobuf

#ifdef DSP_DDS
	movlr	BANK1
	movlw	s_dsp_off2-string_index
	btfss	DSP_CTRL, DSP_CTRL_DSP_ON
	goto	dsp_display_clear2
	movlr	BANK1
	movlw	s_dsp_lms_off-string_index
	btfsc	DSP_CTRL, DSP_CTRL_LMS_ON
	movlw	s_dsp_lms_on-string_index
	movlr	BANK0
	call	p_string_xy_nobuf
	movlr	BANK1
	movlw	s_dsp_notch_off-string_index
	btfsc	DSP_CTRL, DSP_CTRL_NOTCH
	movlw	s_dsp_notch_on-string_index
dsp_display_clear2:
	movlr	BANK0
	call	p_string_xy_nobuf

	movlr	BANK1
	movlw	s_dsp_txagc_off-string_index
	btfsc	DSP_TX_CTRL, DSP_TX_CTRL_AGC_ON
	movlw	s_dsp_txagc_on-string_index
	movlr	BANK0
	call	p_string_xy_nobuf
	movlr	BANK1
	movlw	s_dsp_txproc_off-string_index
	btfsc	DSP_TX_CTRL, DSP_TX_CTRL_PROC_ON
	movlw	s_dsp_txproc_on-string_index
	movlr	BANK0
	call	p_string_xy_nobuf
#endif
#ifdef DSP_DDS
	movlr	BANK1
	movlw	s_2tone_on-string_index
	btfss	DSP_TX_CTRL, DSP_TX_CTRL_2TONE
	movlw	s_2tone_off-string_index
	movlr	BANK0
	call	p_string_xy_nobuf
#endif
	return
#endif ; DSP_COMMON_FUNCTIONS

;
; ---------------------------------------------------------------------
; independant tx / rx output functions

#ifdef RELAY_BOARD
indep_rx:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlw	3
	andwf	Bdata_misc,W		; if transverter in use
	btfss	ALUSTA,Z
	return				; then ignore button
	btg	UI_Flags2,UIF2_INDEP_RX
	goto	indep_txrx_common

indep_tx:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	movlw	3
	andwf	Bdata_misc,W		; if transverter in use
	btfss	ALUSTA,Z
	return				; then ignore button
	btg	UI_Flags2,UIF2_INDEP_TX
indep_txrx_common:
	bsf	UPDATE_1,MISC_CHANGE	; signal event change
indep_txrx_display:
	movlw	3
	andwf	UI_Flags2,W
	addlw	s_normal_txrx-string_index	; index appropriate string
	goto	p_string_xy_nobuf
	;; return implicit in goto
#endif

;
; ---------------------------------------------------------------------
; function to disable vox on startup if enabled
	
dis_vox_startup:
	btfsc	PB_STATE,7		; if opto event, ignore
	return
	btfss	PB_STATE,0		; if button down, ignore
	return
	btg	UI_Flags2,UIF2_VOX_DIS	; toggle flag showing disable st poweron
	movlw	s_vox_pwr_off-string_index
	btfss	UI_Flags2,UIF2_VOX_DIS
	movlw	s_vox_pwr_on-string_index
	bsf	UPDATE_1,MISC_CHANGE	; signal event change
	goto	p_string_xy_nobuf
	;; return implicit in goto

;
; ---------------------------------------------------------------------
; toggle on / off bandscan and configure display & rig
#ifdef BANDSCAN

bandscan_button:
	btfsc	PB_STATE,7		; if opto event,
	goto	scan_cursor		; jump to service it
	btfss	PB_STATE,0		; if button down, ignore
	return
	call	bandscan_clear
	clrf	OPTO_1_COUNT,F		; clear any residual opto count
	btg	RXFlags2,RXF2_BANDSCAN	; toggle flag 
	btfsc	RXFlags2,RXF2_BANDSCAN	; skip over if clear, ie if scan off
	goto	bandscan_start

	; if( cursor > 100 )
	;	offset ( 8 bit ) = cursor - 100
	;	offset ( 32 bit ) = offset * ScanStep
	;	freq = freq + offset
	;	do_opto_1_checkminimum()
	; else if ( cursor < 100 )
	;	offset ( 8 bit ) = 100 - cursor
	;	offset ( 32 bit ) = offset * ScanStep
	;	freq = freq - offset
	;	do_opto_1_checkminimum()
	; endif

	movlr	BANK1
	movlw	D'100'
	cpfsgt	ScanCursor
	goto	bandexitnextcheck
	; process cursor > 100
	subwf	ScanCursor,W		; WREG = cursor - 100
	; put WREG into BARGB0,1
	movlr	BANK0
	movwf	BARGB1
	call	bandscanexithelper		; common code
	movfp	AARGB5,WREG
	addwf	INDF1,F
	movfp	AARGB4,WREG
	addwfc	INDF1,F
	movfp	AARGB3,WREG
	addwfc	INDF1,F
	movfp	AARGB2,WREG
	addwfc	INDF1,F
	goto	bandexitdonechange

bandexitnextcheck:
	cpfslt	ScanCursor
	goto	bandexitdoneit
	; process cursor < 100
	subwf	ScanCursor,W		; WREG = cursor - 100
	; put WREG into BARGB0,1
	movlr	BANK0
	negw	BARGB1,F		; 100 - cursor
	call	bandscanexithelper		; commpn code
	movfp	AARGB5,WREG
	subwf	INDF1,F
	movfp	AARGB4,WREG
	subwfb	INDF1,F
	movfp	AARGB3,WREG
	subwfb	INDF1,F
	movfp	AARGB2,WREG
	subwfb	INDF1,F
bandexitdonechange:
	bsf	ALUSTA,FS3	; set fsr1 to not decrement
	movlw	4
	addwf	FSR1,F
	call	do_opto_1_checkminimum
bandexitdoneit:
	movlr	BANK0
	; reset agc hang
	; enable rx audio
	; reset dds freq
	movlw	(1<<DDS1_REWRITE)|(1<<IF_CHANGED)|(1<<DDS1_CHANGED)|(1<<VOL_OR_MIC_CH)
	iorwf	UPDATE_1,F

	; reset opto mode to normal
	movlw	0			; get null function ( 0 )
	movwf	OPTO_1_MODE		; and set that as opto 1 mode

	goto	do_other_display_if_needed
;;	return				; return implicit in goto

bandscanexithelper:
	clrf	BARGB0,F
	; put ScanStep into AARGB0,1,2,3
	movlr	BANK1
	movfp	ScanStepB0,WREG
	movfp	ScanStepB1,PRODH
	movfp	ScanStepB2,PRODL
	movfp	ScanStepB3,FSR0
	movlr	BANK0
	movpf	WREG,AARGB0
	movpf	PRODH,AARGB1
	movpf	PRODL,AARGB2
	movpf	FSR0,AARGB3
	; generate offset for DDS
	call	FXM3216U
	; result is in AARGB2,3,4,5
	movlw	DDS1A_3		 ; which VCO ?
	btfsc	RXFlags,RXF_RX_B ; VCOB ?
	movlw	DDS1B_3		 ; if so, change to VCOB
	movwf	FSR1
	bcf	ALUSTA,FS3	 ; fsr1 auto post decrement
	bcf	ALUSTA,FS2
	return

bandscan_start:
;
	; set opto mode to scan cursor
	movlw	pbscan-pb_func_index	; get value of volume function
	movwf	OPTO_1_MODE		; and set that as opto 1 mode

	movlr	BANK1
	clrf	XCount,F	; xcount = 0
	movlw	D'100'
	movwf	ScanCursor	; set cursor to current frequency (centre)
	setf	ScanCursorOld,F	; set 'old' position to -1 to force display
	movlr	BANK0

	; step = n * current_step_value (FRIG)

	clrf	AARGB0,F
	movfp	STEPB1,WREG
	movwf	AARGB1
	movfp	STEPB2,WREG
	movwf	AARGB2
	movfp	STEPB3,WREG
	movwf	AARGB3

	; determine scan rate based on filter bw
	; note prodh,l used as temp storage

	btfsc	RXFlags2,RXF2_CWFilter
	goto	bandscan_start_cwflt
	movlr	BANK1
	movfp	ScanStepSSBHigh,PRODH
	movfp	ScanStepSSBLow,PRODL
	goto	bandscan_start_0
bandscan_start_cwflt:
	movlr	BANK1
	movfp	ScanStepCWHigh,PRODH
	movfp	ScanStepCWLow,PRODL
bandscan_start_0:
	movlr	BANK0
	movpf	PRODH,BARGB0
	movpf	PRODL,BARGB1
	call	FXM3216U
	movfp	AARGB2,WREG
	movfp	AARGB3,PRODH
	movfp	AARGB4,PRODL
	movfp	AARGB5,FSR0
	movlr	BANK1
	movpf	WREG,ScanStepB0
	movpf	PRODH,ScanStepB1
	movpf	PRODL,ScanStepB2
	movpf	FSR0,ScanStepB3
	movlr	BANK0

	; start = centre - 100 * step
	; first calculate 100 * step

	movpf	WREG,AARGB0
	movpf	PRODH,AARGB1
	movpf	PRODL,AARGB2
	movpf	FSR0,AARGB3
	clrf	BARGB0,F
	movlw	D'100'
	movwf	BARGB1
	call	FXM3216U

	; subtract from current centre freq
	; leaving result in BARG

	movlw	DDS1A_3		 ; which VCO ?
	btfsc	RXFlags,RXF_RX_B ; VCOB ?
	movlw	DDS1B_3		 ; if so, change to VCOB
	bcf	ALUSTA,FS3	 ; fsr1 auto post decrement
	bcf	ALUSTA,FS2
	movwf	FSR1
	movfp	AARGB5,WREG
	subwf	INDF1,W
	movwf	BARGB3
	movfp	AARGB4,WREG
	subwfb	INDF1,W
	movwf	BARGB2
	movfp	AARGB3,WREG
	subwfb	INDF1,W
	movwf	BARGB1
	movfp	AARGB2,WREG
	subwfb	INDF1,W
	movwf	BARGB0
	bsf	ALUSTA,FS3	; set fsr1 to not decrement

	; copy into store ( leave result in BARGB for freq setting )
	; note BARGB0 already in WREG from above subtract

;;	movfp	BARGB0,WREG
	movfp	BARGB1,PRODH
	movfp	BARGB2,PRODL
	movfp	BARGB3,FSR0
	movlr	BANK1
	movpf	WREG,ScanStartB0
	movpf	PRODH,ScanStartB1
	movpf	PRODL,ScanStartB2
	movpf	FSR0,ScanStartB3
	movpf	WREG,ScanCurrentB0
	movpf	PRODH,ScanCurrentB1
	movpf	PRODL,ScanCurrentB2
	movpf	FSR0,ScanCurrentB3
	movlr	BANK0

	movlw	BARGB0		; set dds to start frequency
	call	setdds1_1

	bsf	UPDATE_1,IF_CHANGED	; set hang to low
	bsf	UPDATE_1,VOL_OR_MIC_CH	; mute

	; no need to disable tx - done in MAINLOOP

	return

;
; manage cursor location change
; update ScanCursor - the scan routine
; will detect change by comparison to ScanCursorOld
;
scan_cursor:
	rrncf	PB_STATE,W		; get amount to change by
	andlw	0x1f
	movwf	I			; and save for now
scan_up_down1:
	btfss	PB_STATE,0		; goimg down ?
	goto 	scan_up1		; nope - going up
; vol down
	movlr	BANK1
	decf	ScanCursor,F		; decrement cursor location
	incf	ScanCursor,W		; and check if < 0
	btfsc	ALUSTA,Z
	clrf	ScanCursor,F		; if so, clear it
	goto	scan_up_down_end
scan_up1:
	movlr	BANK1
	incf	ScanCursor,F		; increment location
	movlw	D'199'			; check overflow
	cpfslt	ScanCursor
	movwf	ScanCursor
scan_up_down_end:
	movlr	BANK0
	decf	I,F			; decrement count
	btfss	ALUSTA,Z
	goto	scan_up_down1		; and loop ( yes, it is inefficient )

	return

#endif


