;-------------------------------------------------------------------------------
; NAME: RMAP.h
;
; LABEL PREFIX: 
;
; Author:	Dave Roberts G8KBB
;
; PURPOSE:	This module defines the general purpose file register allocation for 
;   		variables.
;
;
; SPECIAL CONDITIONS: 
;
; REVISION HISTORY:
;  		15/06/99	G8KBB	Initial definitions
;		10/05/00	G8KBB	Ported to 17C766
;
;-------------------------------------------------------------------------------		

; Memory strategy
;
; Unbank used to hold interrupt data
; Page 0 used hold all main operating data
; Page 1 used to hold large data structures ( push button data etc )
; Page 2 used to hold I2C buffers
; Page 3 unused
;
; 
; The addresses that are used to store the CPUSTA and WREG values must be in the data memory
; address range of 1Ah - 1Fh. Up to 6 locations can be saved and restored using the MOVFP
; instruction. This instruction neither affects the status bits, nor corrupts the WREG register.
;
; Hence these are used in interrupt handling and also for the odd data move.
; Whenever used, interrupts MUST be disabled.

UNBANK1 EQU 0x01A ; Address for 1st location to save
UNBANK2 EQU 0x01B ; Address for 2nd location to save
UNBANK3 EQU 0x01C ; Address for 3rd location to save
UNBANK4 EQU 0x01D ; Address for 4th location to save
UNBANK5 EQU 0x01E ; Address for 5th location to save
UNBANK6 EQU 0x01F ; Address for 6th location to save
;
;

;
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 0  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

; Note next 13 must be contiguous
BAND		EQU		20H		; Current Band ( 0 - 1.8 etc )
OPTO_2_MODE	EQU		21H		; Current Opto switch 2 mode
MIN_ACCEL	EQU		22H		; min value to maintain acceleration
UI_Flags	EQU		23H		; misc display control flags
UIF_Backlight	EQU	0	; 00 = off, 01 = on, 
UIF_Backlight2	EQU	1	; 02 = long delay, 03 = short delay
UIF_Menu_bit_0	EQU	2	; next 2 bits hold menu number
UIF_Menu_bit_1	EQU	3	; 0,1,2 or 3
UIF_Vox_off	EQU	4	; this is set to disable vox ( vox gain -> 0 )
UIF_Keyer_Type	EQU	5	; if set, iambic keyer else simple keyer
UIF_PLL_Check	EQU	6	; if set, only go to xmit if pll locked
;
; following are config data for S meter
;
S0		EQU		24H	; ADC reading for noise floor
Smeter_slope_A	EQU		25h	; slope multiplier for s meter cal
Smeter_slope_B	EQU		26h	; slope divisor for s meter cal
dBm_offset	EQU		27h	; calibration offset for dBm
dB_per_spoint	EQU		28h	; as it says !
S0CW		EQU		29h	; as per S0 but for narrow filter
SHang		EQU		2Ah	; on screen s meter hang time (10's msec)
Pmeter_slope_A	EQU		2Bh	; slope multiplier for s meter cal
Pmeter_slope_B	EQU		2Ch	; slope divisor for s meter cal
UI_Flags2	EQU		2dH	; additional display control flags
UIF2_INDEP_RX	EQU	0	; 0 = rx via main tx/rx port, 1 = independant skt
UIF2_INDEP_TX	EQU	1	; 0 = tx via main tx/rx port, 1 = independant skt
UIF2_VOX_DIS	EQU	2	; 1 = vox disabled on powerup regardless of vox state
;; Spare 	EQU		2eh
;; spare	EQU		2fh

		; 30-4F reserved for push button indirections

PBindirect:	EQU		30H	; push buttons store their function codes 
;		EQU		34H	; in this table. Each button ( there are
;		EQU		38H	; 32 of them ) are represented by a byte
;		EQU		3CH	; For each one, stored here is the desired
;		EQU		3FH	; function code - an index into the table
;		EQU		40H	; in the pbfuncs table
;		EQU		44H	;
;		EQU		48H	;
;		EQU		4CH	;
PBindirectEnd	EQU		4FH	; end of push button indirections
;
		; 
Keyer_state   	EQU		50H	; keyer finite state machine
Keyer_delay	EQU		51H	; keyer timer value
Keyer_dot	EQU		52H	; single dot delay (*10msec)
Keyer_hang	EQU		53H	; timeout dot multiple
SHangCount	EQU		54H	; s meter hang timer counter
I		EQU		55H	; Temporary (working) registers for non-preemptive tasks.
J		EQU		56H	; " " "
K		EQU		57H	; " " "
;;;		EQU		58H	;
;;;		EQU		59H	; 
;;;		EQU		5aH	; 
;;;		EQU		5bH	; 
;;;		EQU		5cH	; 
;;;		EQU		5dH	; 
;;;		EQU		5eH	; 
;;;		EQU		5fH	; 

ONE_MSEC	EQU		60H	; count of milliseconds ( if >=10, subtract 10 )
TEN_MSEC	EQU		61H	; count of centiseconds ( if >=100, subtract 100 )
ONE_SEC		EQU		62H	; count of seconds. Wraps round. Not really used
OPTO_1_MODE	EQU		63H	; current opto switch 2 mode
OPTO_1_COUNT	EQU		64H	; pulse count for opto switch 1
OPTO_2_COUNT	EQU		65H    	; ditto 2
PB_ID		EQU		66H	; current button ID
PB_INDCOUNT	EQU		67H	; byte counter for button event decoding
PB_INDEXST	EQU		68H	; pointer to current switch state byte
PB_INDEXCH	EQU		69H	; pointer to current switch change byte
PB_BITCOUNT	EQU		6aH	; bit counter for button event decode
OLD_OPTO_2	EQU		6bh	; this and old_opto_1 used to hold accumuated
OLD_OPTO_1	EQU		6ch	; count for opto 1 acceleration
PB_PROG_ID	EQU		6dh	; identity of button being programmed
PB_STATE	EQU		6eh		; button down or up 
; bit usage in PB_STATE:
;	bit 0 - button up / down flag
;	bit 1-5 - number of button events to process
;	bit 6 - if set, PB_PROG_FLAGS was non zero before this event
;	bit 7 - if set, this is opto originated event
PB_PROG_FLAGS	EQU		6fh		; counter & flag for button programming
; usage of PB_PROG_FLAGS
; if zero, no switch programming event in progress
; if 255, switch programming in effect
; other values = seconds coundown to programming action
;
;--------  used to change operation of system --------------------
UPDATE_1	EQU		70H	; 
BAND_CHANGE	EQU	0		; band change has happened
VOL_OR_MIC_CH	EQU	1		; volume or mic gain changed
VOX_CHANGED	EQU	2		; vox gain, delay or anti vox changed
IF_CHANGED	EQU	3		; IF gain or hang time changed
RF_CHANGE	EQU	4		; atten or RF power changed
DDS1_CHANGED	EQU	5		; redo frequency
MISC_CHANGE	EQU	6		; opto 2 mode, accel, UI_Flags, relay board
DDS1_REWRITE	EQU	7		; change DDS1 settings

UPDATE_2	EQU		71H
BUTTON_A_CH	EQU	0		; button set A changed
BUTTON_B_CH	EQU	1		; button set B changed
FREQ_ACCEL	EQU	2		; flag says opto 1 accelerates
DSP_CHANGED	EQU	3		; change of state to dsp control
FILTER_CHANGE	EQU	4		; change to receive filters
MODE_CHANGE	EQU	5		; Mode change

; flag to eeprom module that update is needed

EEPROM_UPD_1	EQU		72h
EEPROM_UPD_2	EQU		73h
; bit definitions for above are same as for UPDATE_*

EE_FREQ_IDLE	EQU		74h	; number of seconds VCO stationary
EEPROM_UPD_3	EQU		75h
SEC_BAND_UPD	EQU	0		; update secondary band data
PRI_BAND_UPD	EQU	1		; update primary band data
ATTN_CFG_UPD	EQU	2		; update atten data
DDS_DATA_UPD	EQU	3		; update DDS ref data
ACCEL_DATA_UPD	EQU	4		; acceleration etc
KEYER_DATA_UPD	EQU	5		; keyer and scan data
DSP_DATA_UPD	EQU	6		; full DSP data change
EE_RESERVED_UPD	EQU	7		; reserved - reset data flag
; the above MUST MATCH the bit settings in config.asm

CUR_CFG		EQU		076H	; current configuration parameter
OLD_PORT_B	EQU		077H	; previous port B ( used to detect changes )
DDS1A_0		EQU		078h	; VCOA frequency as a dds count
DDS1A_1		EQU		079h
DDS1A_2		EQU		07ah
DDS1A_3		EQU		07bh
DDS1B_0		EQU		07ch	; VCOB frequency as a dds count
DDS1B_1		EQU		07dh
DDS1B_2		EQU		07eh
DDS1B_3		EQU		07fh

;;;
;;; 080 - 08F unused
;;; 090 - 09F unused
;;;

;~~~~~~~~~~~~~~~  used by Math Routines ~~~~~~~~~~~~~~~~~~~
MHExpa_B1	EQU		0A0H	; exponent argument a
AARGB0		EQU		MHExpa_B1
MHACCaHI_B1	EQU		0A1H	; mantissa for argument a MSB	
AARGB1		EQU		MHACCaHI_B1
MHACCaMI_B1	EQU		0A2H	; mantissa for argument a middle
AARGB2		EQU		MHACCaMI_B1
MHACCaLO_B1	EQU		0A3H	; mantissa for argument a LSB	
AARGB3		EQU		MHACCaLO_B1
MHFlags_B1	EQU		0A4H	; floating point exception flgas
AARGB4		EQU		MHFlags_B1
AARGB5		EQU		0A5H
INT_W2		EQU		0A6H	; also holds W in interrupt routine
;
MHPPHI_B1	EQU		0A7H	; partial product
AARGB6		EQU		MHPPHI_B1
MHPPMI_B1	EQU		0A8H	; partial product
AARGB7		EQU		MHPPMI_B1
MHPPLO_B1	EQU		0A9H	; partial product
temp_count	EQU		MHPPLO_B1
MHSign_B1	EQU		0AAH	; temp loc. for sign
;
MHExpb_B1	EQU		0ABH	; exponent argument b
MHACCbHI_B1	EQU		0ACH	; mantissa for argument b MSB
BARGB0 		EQU		MHACCbHI_B1
MHACCbMI_B1	EQU		0ADH	; mantissa for argument b middle
BARGB1		EQU		MHACCbMI_B1
MHACCbLO_B1	EQU		0AEH	; mantissa for argument b LSB
BARGB2		EQU		MHACCbLO_B1
MHTemp_B1	EQU		0AFH	; temporary storage
BARGB3		EQU		MHTemp_B1

TEMPB0		EQU		0b0h
TEMPB1		EQU		0b1h
TEMPB2		EQU		0b2h
TEMPB3		EQU		0b3h
SIGN		EQU		0b4h
LOOPCOUNT	EQU		0b5h
REMB0		EQU		0b6h
REMB1		EQU		0b7h
;
; Next 24 bytes are initialised as BandTable
; order of these 24 bytes must match bandtable.asm entries
; bandtableinit in init.asm needs to know size of entry also
Bdata_start	EQU		0b8h
Bdata_misc	EQU		0b8h
; bit definitions
; bits 0,1 select main output or transverter
; bits 2,3 control USB/LSB offsets
BDMisc_xvert0	EQU	0
BDMisc_xvert1	EQU	1
BDSsbOffset	EQU	2	; if set, apply usb / lst offsets to displayed freq
BDSsbReverse	EQU	3	; if clear, add lsb offset in lsb & subtract usb in usb
				; if set do the opposite ( e.g. add usb offset in usb )
 
Bdata_csum2	EQU		0b9h	; secondary band data checksum
SMeter_cal	EQU		0bah
VCO_Mode	EQU		0bbh
ModeFlags	EQU		0bch
;Mode Flag Definitions
; bits 0,1,2 - DDS step rate index
; bits 3,4   - Mode ( 00 = cw, 01 = usb, 10 = lsb )
; bits 5,6   - IF Offset calculation mode as follows:
;		bit 5 zero = subtract; 1 = add
;		bit 6 zero = Osc +/- IF; 1 = IF +/- Osc
;		hence values are as follows:
;			00 = Osc - IF
;			01 = Osc + IF
;			10 = IF - Osc
;			11 = IF + Osc ( but do not use - rfu )
BandSetLPF	EQU		0bdh
BandSetFEB1	EQU		0beh
BandSetFEB2	EQU		0bfh
BANDMIN0	EQU		0c0h
BANDMIN1	EQU		0c1h
BANDMIN2	EQU		0c2h
BANDMIN3	EQU		0c3h
BANDMAX0	EQU		0c4h
BANDMAX1	EQU		0c5h
BANDMAX2	EQU		0c6h
BANDMAX3	EQU		0c7h
Bdata_csum1	EQU		0c8h	; checksum and ms byte of step rate
STEPB0		EQU		0c8h	; share same location
STEPB1		EQU		0c9h
STEPB2		EQU		0cah
STEPB3		EQU		0cbh
fOffB0		EQU		0cch
fOffB1		EQU		0cdh
fOffB2		EQU		0ceh
fOffB3		EQU		0cfh

;
; dds reference frequencies and CW filter offset
;
DDS1_osc0	EQU		0d0h
DDS1_osc1	EQU		0d1h
DDS1_osc2	EQU		0d2h
DDS1_osc3	EQU		0d3h
DDS2_osc0	EQU		0d4h
DDS2_osc1	EQU		0d5h
DDS2_osc2	EQU		0d6h
DDS2_osc3	EQU		0d7h
DDS_cksum	EQU		0d8h
CWofs1		EQU		0d9h
CWofs2		EQU		0dah
CWofs3		EQU		0dbh
UsbOffset2	EQU		0dch	; ms byte of usb offset
UsbOffset3	EQU		0ddh	; ls byte
LsbOffset2	EQU		0deh	; ms byte of lsb offset
LsbOffset3	EQU		0dfh	; ls byte

;
; display buffer (also used by i2c code )
;
DISP_BUF	EQU		0xe0
DISP_BUF_END	EQU		0xef

;
; configuration data
;
Volume		EQU		0xf0	; current pot setting for volume
MicGain		EQU		0xf1	; current pot setting for mic gain	
IFGain		EQU		0xf2	; current DAC setting for IF gain
HangTime	EQU		0xf3	; current DAC setting for IF Hang Time
VoxGain		EQU		0xf4	; current pot setting for vox gain
AntiVox		EQU		0xf5	; current pot setting for ani vox
VoxDelay	EQU		0xf6	; current DAC setting for vox delay
RFPower		EQU		0xf7	; current DAC setting for RF power
RXFlags		EQU		0xf8	; misc flags as defined below
; bit definitions in RXFXlags :
;	Bits 0,1 - attenuator
;	Bit 2 - on transmit ( logical or of flags in RXFlags2 )
;	Bit 3 - tuneup
;	Bit 4 - antenna relay to tx if set
;	Bit 5 - Use VCO B not A for TX if bit set
;	Bit 6 - Use VCO B not A for RX if bit set
;	Bit 7 - if set, lock VCO
RXF_Atten	EQU	0
RXF_Atten2	EQU	1
RXF_On_Xmit	EQU	2
RXF_Tuneup	EQU	3
RXF_AntRly	EQU	4
RXF_TX_B	EQU	5
RXF_RX_B	EQU	6
RXF_Lock	EQU	7

RXFlags2	EQU		0xf9
; bit definitions
;	Bit 0 - set means use narrow CW filter
;	Bit 1 - set means ptt low requesting xmit
;	Bit 2 - set means keyer wants to be on xmit
;	Bit 3 - set means tuneup button pressed
;	bit 4 - set means disable keyer
;	bit 5 - set means bar transmit
; 	bit 6 - set means use second controller on double ctrl lcd displays
;	bit 7 - set means do bandscan
RXF2_CWFilter	EQU	0
RXF2_PTT_active	EQU	1
RXF2_KEYER_TX	EQU	2
RXF2_TUNEUP_TX	EQU	3
RXF2_KEYER_DIS	EQU	4
RXF2_TX_DISABLE	EQU	5
RXF2_LCD_USE_E2	EQU	6
RXF2_BANDSCAN	EQU	7

TX_REQUESTED_MASK EQU	((1<<RXF2_PTT_active)|(1<<RXF2_KEYER_TX)|(1<<RXF2_TUNEUP_TX))

tx_rx_state	EQU		0xfa	; current state of TX/RX FSM ( 0..7 )
S_Meter_ADC	equ		0xfb 	; current s meter ADC reading
PORTF_COPY	equ		0xfc	; ram version of port F output state
Atten_cal	EQU		0xfd	; attenuator calibration setting
Idle_Count	EQU		0xfe	; counter of ( 256 second ) idle periods
;;;		equ		0xff	; spare

;
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 1  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

ptt_counter	EQU		0x120		; millisec counter for ptt debounce
SW_SCAN1	EQU		0x121		; These are a set of temp storage
SW_SCAN2	EQU		0x122		; used by the switch scan code
SW_SCAN3	EQU		0x123		; They are defined like this to
SW_SCAN4	EQU		0x124		; allow redefinition for the
SW_SCAN5	EQU		0x125		; different switch scan options
SW_SCAN6	EQU		0x126		;
SW_SCAN7	EQU		0x127		;
SW_SCAN8	EQU		0x128		;
SW_SCAN9	EQU		0x129		;
SW_SCANA	EQU		0x12a		;
SW_SCANB	EQU		0x12b		;
SW_SCANC	EQU		0x12c		;
SW_SCAND	EQU		0x12d		;
SW_SCANE	EQU		0x12e		;
SW_SCANF	EQU		0x12f		;
SW_SCAN0	EQU		0x130		;
SW_SCAN11	EQU		0x131		;
SW_SCAN12	EQU		0x132		;
SW_SCAN13	EQU		0x133		;
SW_SCAN14	EQU		0x134		;
SW_SCAN15	EQU		0x135		;
SW_SCAN16	EQU		0x136		; end of redefinable block as above

#ifdef	KBD_4X4

SW_scancount	EQU	SW_SCAN1
SW_scanbase	EQU	SW_SCAN2 ; array of 4 * 2 ( 2..9 )
SW_scantemp	EQU	SW_SCANA
SW_A		EQU	SW_SCANB
SW_B		EQU	SW_SCANC
SW_C		EQU	SW_SCAND
SW_D		EQU	SW_SCANE
SW_A_TEMP	EQU	SW_SCANF
SW_B_TEMP	EQU	SW_SCAN0
SW_C_TEMP	EQU	SW_SCAN11
SW_D_TEMP	EQU	SW_SCAN12
SW_COUNT1	EQU	SW_SCAN13
SW_COUNT2	EQU	SW_SCAN14
SW_COUNT11	EQU	SW_SCAN15
SW_COUNT21	EQU	SW_SCAN16
#else

SW_COUNT1	EQU	SW_SCAN1
SW_COUNT2	EQU	SW_SCAN2
PORTJ_TEMP	EQU	SW_SCAN3
PORTH_TEMP	EQU	SW_SCAN4

#endif

		; Switch and other settings
TSW_CHANGED   	EQU		140H	; 4 bytes per data type
;		EQU		141H	; 
;		EQU		142H	; 
;		EQU		143H	; 
TSW_STATE	EQU		144H	; 
;		EQU		145H	; 
;		EQU		146H	; 
;		EQU		147H	; 
SW_CHANGED	EQU		148H	; 
;		EQU		149H	; 
;		EQU		14aH	; 
;		EQU		14bH	; 
SW_STATE	EQU		14cH	; 
;		EQU		14dH	; 
;		EQU		14eH	; 
;		EQU		14fH	; 

DSP_DDS_I2C_CMD	EQU		150h
DSP_CTRL	EQU		151h	; dsp control word
; bit definitions above
DSP_CTRL_DSP_ON		equ	0
DSP_CTRL_AGC_OFF	equ	1
DSP_CTRL_DSP_BPF	equ	2
DSP_CTRL_DSP_NRCW	equ	3
DSP_CTRL_VOX_ON		equ	4
DSP_CTRL_LMS_ON		equ	5
DSP_CTRL_NOTCH		equ	6

DSP_TX_CTRL	EQU		152h	; tx dsp control bits
; bit defintions for the above
DSP_TX_CTRL_PROC_ON	equ	0	; xmit speech processor
DSP_TX_CTRL_AGC_ON	equ	1	; mic automatic gain ctrl
DSP_TX_PROC_AGC_ON	equ	2	; tx post processor AGC
DSP_TX_CTRL_2TONE	equ	3

DSP_CW_SIDETONE_VOL	EQU	153h	; CW TX sidetone volume
DSP_CW_SIDETONE_FREQ_H	EQU	154h	; CW TX sidetone frequency
DSP_CW_SIDETONE_FREQ_L	EQU	155h	; CW TX sidetone frequency
DSP_LMS_BETA_H		EQU	156h
DSP_LMS_BETA_L		EQU	157h
DSP_LMS_DECAY_H		EQU	158h
DSP_LMS_DECAY_L		EQU	159h
DSP_NOTCH_BETA_H	EQU	15ah
DSP_NOTCH_BETA_L	EQU	15bh
DSP_NOTCH_DECAY_H	EQU	15ch
DSP_NOTCH_DECAY_L	EQU	15dh
DSP_TX_AGC_HANG_H	EQU	15eh
DSP_TX_AGC_HANG_L	EQU	15fh
DSP_TX_COMPRESSION	EQU	160h
DSP_INIT_END		EQU	DSP_TX_COMPRESSION ; label for end of initialisation

ScanStepB0		EQU	170h
ScanStepB1		EQU	171h
ScanStepB2		EQU	172h
ScanStepB3		EQU	173h
ScanCurrentB0		EQU	174h
ScanCurrentB1		EQU	175h
ScanCurrentB2		EQU	176h
ScanCurrentB3		EQU	177h
ScanStartB0		EQU	178h
ScanStartB1		EQU	179h
ScanStartB2		EQU	17Ah
ScanStartB3		EQU	17Bh
XCount			EQU	17Ch
YCount			EQU	17Dh
ValTemp			EQU	17Eh
ScanStartDelayCount	EQU	17Fh

BandScanReadings	EQU	180h
;; uses 180 - 187
ScanStepSSBHigh		EQU	188h
ScanStepSSBLow		EQU	189h
ScanStepCWHigh		EQU	18ah
ScanStepCWLow		EQU	18bh
ScanStartDelay		EQU	18ch
ScanCursor		EQU	18dh
ScanCursorOld		EQU	18eh

;
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 2  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;
I2C_PTR			EQU	0x220	; pointer to an active buffer
I2C_STATE		EQU	0x221	; internal I2C state memory
; bit defintitions for I2C_STATE
I2C_Active	EQU	1

; following are used to hold a copy of current I2C buffer

I2C_address		EQU	0x222	; copy of current slave address
I2C_subaddr		EQU	0x223	; copy of current sub address
I2C_count		EQU	0x224	; copy of byte count to send/receive
I2C_page		EQU	0x225	; copy of BSR for ram page for data
I2C_ram_ptr		EQU	0x226	; copy of address on above page
I2C_command		EQU	0x227	; copy of requested command byte
I2C_Temp		EQU	0x228	; temp storage for I2C interface routines

; these are the buffers used to hold I2C commands
; they must not extend beyond end of the page
#if (( HIGH I2C_FIRST_BUF ) != ( HIGH I2C_BUF_END ))
#error I2C buffer array spans boundary of a page !
#endif

I2C_FIRST_BUF		EQU	0x229
I2C_BUF_END		EQU	I2C_FIRST_BUF + ( I2C_BUF_SIZE * I2C_NUM_BUFS )

;
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! BANK 3  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;
;                 !!!!!!!!!!!!!!! IS ALL SPARE !!!!!!!!!!!!!!!!!
;


