;*****************************************************************************
; NAME: Start.asm
;
; LABEL PREFIX: "XCVR"
;
; COMPANY: 	Dave Roberts G8KBB
;
; PURPOSE:	contains the start vector and interrupt vector originations.
;
;
;		Goto Init	-  starts exectuion of program 
;		Goto Isr*	-  starts one of the interrupt service procedures
;
;REVISION HISTORY:
;		10/05/00	G8KBB	Created for 17C766
;****************************************************************************		
;
; program starts at reset vector address 0
; and isr routines vector to addresses 8,0x10 and 0x18
;
; fill all intermediate locations with jumps also for want of
; something better to use
; but note caution about pclath. It cannot be saved in the isr
; as jumping to the isr would corrupt it.
; Note also that the goto explicitly sets the pclath register
; so that it is valid initially for the isr routine if it needs it

        ORG     0
        goto    Init
        goto    Init
        goto    Init
        goto    Init
        goto    Init
        goto    Init
        goto    Init
        goto    Init

; ISR for RA0 pin interrupt events
; note first part done here to ensure pclath is not corrupted
;
	ORG	8
	MOVFP 	ALUSTA, UNBANK1 ; Push ALUSTA value
	MOVFP 	BSR, 	UNBANK2 ; Push BSR value
	MOVFP 	WREG, 	UNBANK3 ; Push WREG value
	MOVFP 	PCLATH, UNBANK4 ; Push PCLATH value
        goto    IsrRA0
        goto    IsrRA0
        goto    IsrRA0
        goto    IsrRA0

; ISR for timer 0 overflow
; note first part done here to ensure pclath is not corrupted
;
	ORG	0x10
	MOVFP 	ALUSTA, UNBANK1 ; Push ALUSTA value
	MOVFP 	BSR, 	UNBANK2 ; Push BSR value
	MOVFP 	WREG, 	UNBANK3 ; Push WREG value
	MOVFP 	PCLATH, UNBANK4 ; Push PCLATH value
	goto	IsrT0IF
	goto	IsrT0IF
	goto	IsrT0IF
	goto	IsrT0IF

; ISR for T0CKIF
; note first part must be done here to ensure pclath is not corrupted
; but as there is currently no event for this - reset !
	ORG	0x18
	goto	0
	goto	0
	goto	0
	goto	0
	goto	0
	goto	0
	goto	0
	goto	0
;	goto	IsrT0CKIF
;	goto	IsrT0CKIF
;	goto	IsrT0CKIF
;	goto	IsrT0CKIF
;	goto	IsrT0CKIF
;	goto	IsrT0CKIF
;	goto	IsrT0CKIF
;	goto	IsrT0CKIF

; ISR for peripheral interrupt events
; note first part done here to ensure pclath is not corrupted
;
	ORG	0x20
	MOVFP 	ALUSTA, UNBANK1 ; Push ALUSTA value
	MOVFP 	BSR, 	UNBANK2 ; Push BSR value
	MOVFP 	WREG, 	UNBANK3 ; Push WREG value
	MOVFP 	PCLATH, UNBANK4 ; Push PCLATH value
	goto	IsrPEIF	
;

