;--------------------------------------------------------------------------
; NAME: 	TxRx-FSM.asm
;
; Author:	G8KBB
;
; PURPOSE:	Finite state machine for TX / RX switching
;
;REVISION HISTORY:
;		Created 14 June 2000
;
;
; ------------------------------------------------------------------------
;
; Every 10 milliseconds, execute state machine.
; change state ( or not ) depending on whether
; tx set or not.

exec_txrx_state_machine:
	movfp	tx_rx_state,WREG	; read current state
	andlw	7			; as a precaution, limit value
	addlw	LOW txrx_state_table	; index into state jump table
	movwf	TBLPTRL
	movlw	HIGH txrx_state_table
	clrf	TBLPTRH,F
	addwfc	TBLPTRH,F
	tablrd	0,1,WREG		; dummy read loads address
	tlrd	1,PCLATH		; execute the jump
	tlrd	0,PCL
;
txrx_state_table:
	dw	txrx_state_0
	dw	txrx_state_1
	dw	txrx_state_2
	dw	txrx_state_3
	dw	txrx_state_4
	dw	txrx_state_5
	dw	txrx_state_6
	dw	txrx_state_7
;
; ------------------------------------------------------------------------
; The following are the indivdual state handlers. Every 10 msec, one
; of these is executed.

txrx_state_0:
	btfss	RXFlags,RXF_On_Xmit	; if on recevive
	return				; then no change
	bsf	RXFlags, RXF_AntRly	; else ant -> TX here
#ifdef RELAY_BOARD
	call	i2c_set_relay_board
#endif
	call	i2c_set_FEB1
	call	i2c_set_FEB2
	call	i2c_set_LPF
;;	clrf	DISP_BUF,F
	movlw	s_nxmit-string_index	
	call	p_string_xy_nobuf	; and blank "TX/RX" display
	movlw	1			; and go to state 1
txrx_set_state_and_exit:
	movpf	WREG,tx_rx_state	; save new state
	return
;
txrx_state_1:
	movlw	2
	btfsc	RXFlags,RXF_On_Xmit	; if still on transmit
	goto	txrx_set_state_and_exit	; then just go to state 2
txrx_ant_to_rx:
	bcf	RXFlags, RXF_AntRly	; going to RX. Ant -> RX here
	call	i2c_set_FEB1
	call	i2c_set_FEB2
	call	i2c_set_LPF
#ifdef RELAY_BOARD
	call	i2c_set_relay_board
#endif
;;	clrf	DISP_BUF,F
	movlw	s_rx-string_index
	call	p_string_xy_nobuf	; display "RX"
	clrf	tx_rx_state,F		; go to state 0
	return
;
txrx_state_2:
	btfss	RXFlags,RXF_On_Xmit	; if on receive
	goto	txrx_ant_to_rx		; then clean up & go to state 0
txrx_rx_off:
	movlb	BANK5
	bcf	PORTF_COPY,PWR_RX	; RX POWER OFF
	movfp	PORTF_COPY,PORTF
	bsf	UPDATE_1,DDS1_REWRITE	; ensure DDS1 reinitialised
	; IF MODE = SSB MUTE AUDIO
	movlw	3
	goto	txrx_set_state_and_exit	; and go to state 3
;
txrx_state_3:
	btfsc	RXFlags,RXF_On_Xmit	; if we are on transmit
	goto	txrx_state3tx		; then skip on
	movlb	BANK5
	bsf	UPDATE_1,DDS1_REWRITE	; else ensure DDS rewritten
	bsf	PORTF_COPY,PWR_RX		; RX POWER ON
	movfp	PORTF_COPY,PORTF
	; AUDIO ON
	goto	txrx_ant_to_rx		; cleanup and go to state 0
txrx_state3tx:
	btfss	UI_Flags,UIF_PLL_Check	; only check pll if required
	goto	txrx_state3tx_pllok
	call	I2C_Find_Buf2
	btfsc	ALUSTA,Z
	return				; wait if no eeprom buffers
	movlw	I2C_VCO_CTRL		; i2c address
	movwf	INDF0
	clrf	INDF0,F			; no i2c subaddress
	movlw	1			; read 1 byte
	movwf	INDF0			; count
	clrf	INDF0,F			; page
	movlw	I			; read 8574 into variable I
	movwf	INDF0			; ram pointer
	movlw	B'00000001'		; no buffer clear, no retry, no subaddress, read
	call	read_device_and_wait
	btfsc	ALUSTA,Z		; zero set means read failed so retry
	return				; by exiting and staying in state 3
	btfss	I,PLL_CHECK_BIT		; now check if locked
	return				; and wait if not by staying in state 3
txrx_state3tx_pllok:
	; OTHER RELAYS TO TX
	movlb	BANK5
	bsf	PORTF_COPY,PWR_TX	; TX POWER ON
#ifdef TWO_TONE_TUNE
	btfsc	RXFlags,RXF_Tuneup
	bsf	PORTF_COPY,PWR_TWOTONE	; tuneup for 2 tone osc
	bcf	PORTF_COPY,PWR_KEYER	; remove keydown signal
#else
	btfsc	RXFlags,RXF_Tuneup
	bsf	PORTF_COPY,PWR_KEYER	; key down on tuneup
#endif
	movfp	PORTF_COPY,PORTF
;;	clrf	DISP_BUF,F
	movlw	s_xmit-string_index
	call	p_string_xy_nobuf	; print "TX"
	clrf	S_Meter_ADC,F		; reset S meter hang value for power display
	movlw	7
	goto	txrx_set_state_and_exit	; and go to state 7
;
txrx_state_4:
	btfsc	RXFlags,RXF_On_Xmit	; if on transmit
	goto	txrx_rx_off		; then go to state 3 via state 2 actions
	goto	txrx_ant_to_rx		; cleanup and go to state 0
;
txrx_state_5:
	movlw	4			; go to state 4
	btfsc	RXFlags,RXF_On_Xmit	; if on transmit
	movlw	2			; otherwise ( TX ) go to state 2
	goto	txrx_set_state_and_exit
;
txrx_state_6:
	btfsc	RXFlags,RXF_On_Xmit	; if on transmit
	goto	txrx_rx_off		; then go to state 3 via state 2 actions
	bsf	UPDATE_1,DDS1_REWRITE	; else ensure DDS rewritten
	; OTHER RELAYS TO RX
	; AUDIO ON
	movlw	5
	goto	txrx_set_state_and_exit	; and go to state 5
;
txrx_state_7:
	btfsc	RXFlags,RXF_On_Xmit	; if on transmit
	return				; do nothing
	movlb	BANK5			; else
#ifdef TWO_TONE_TUNE
	bcf	PORTF_COPY,PWR_TWOTONE	; key up on tuneup for 2 tone osc
#endif
	bcf	PORTF_COPY,PWR_KEYER	; key up
	bcf	PORTF_COPY,PWR_TX	; TX POWER OFF
	bsf	PORTF_COPY,PWR_RX	; RX POWER ON
	movfp	PORTF_COPY,PORTF
;;	clrf	DISP_BUF,F
	movlw	s_nxmit-string_index	; blank "TX/RX" string
	call	p_string_xy_nobuf
	clrf	S_Meter_ADC,F		; reset power meter hang for S meter display
	movlw	6
	goto	txrx_set_state_and_exit	; and go to state 6


